/***
 * Portable GUI
 * Code for PGButton
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifndef _PGBUTTON_
#define _PGBUTTON_

class PGAbstractButton : public PGNormalComponent, public PGActionListenerParent {
protected:
	PGString m_sLabel;
	BOOL m_bDefault;
public:
	PGAbstractButton(char* label);
	PGAbstractButton(char* label, BOOL isDefault);
private:
	void init(char* label, BOOL isDefault);
};
 
class PGButton : public PGAbstractButton {
	
public:
	PGButton(char *text);
	PGButton(char *text, BOOL isDefault);
	virtual void toString(PGString* strg);
	virtual void getPreferredSize(PGDimension* dimension, BOOL recalc);
	virtual void addToWindow(PGWindow* parent);
	virtual void removeFromWindow(PGWindow* parent);
};

class PGCheckbox : public PGAbstractButton {
public:
	BOOL m_bState;	
public:
	PGCheckbox(char *text);	
	virtual void toString(PGString* strg);
	virtual void getPreferredSize(PGDimension* dimension, BOOL recalc);
	virtual void addToWindow(PGWindow* parent);
	virtual void removeFromWindow(PGWindow* parent);
	BOOL getState();
	void setState(BOOL state);
};

class PGRadioButton;

class PGButtonGroup : public PGActionListenerParent, public PGActionListener {
public:
	PGVector m_vButtons;
	int m_iIndex;
public:
	PGButtonGroup();
	void add(PGRadioButton* button);
	void remove(PGRadioButton* button);
	int getSelectedIndex();
	void setSelectedIndex(int idx);
	virtual void actionPerformed(PGActionListenerParent* from);	
// Internal
	void makeButtons(PGWindow* parent);	
	boolean shouldSelect();
};

class PGRadioButton : public PGAbstractButton {
public:
	BOOL m_bState;	
	PGButtonGroup* m_hGroup;
public:
	PGRadioButton();
	PGRadioButton(char *text);	
	virtual ~PGRadioButton();
	BOOL getState();
	void setState(BOOL state);	
	virtual void toString(PGString* strg);
	virtual void getPreferredSize(PGDimension* dimension, BOOL recalc);
// Internals		
	virtual void addToWindow(PGWindow* parent);
	virtual void removeFromWindow(PGWindow* parent);
	void makeButton(PGWindow* parent, PGRadioButton* prevBt);	
private:	
	void init(char *text);
};

#endif
