/***
 * Portable GUI
 * Code for PGComponent
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifndef _PGCOMPONENT_
#define _PGCOMPONENT_

#include <pgmain/pgmain.h> 
#include <pgutil/pgstring.h> 

#ifndef PGTINY
	#include <pgdraw/pggraphics.h> 
	#include <pgcomponent/pgstyle.h> 
#endif

class PGContainer;
class PGWindow;

#define PGCOMPONENT_VISIBLE		1
#define PGCOMPONENT_WINDOW		2
#define PGCOMPONENT_TOOLTIP		4
#define PGCOMPONENT_DISABLED		8
#define PGCOMPONENT_ALLFLAGS		0xFFFF

#define PGCOMPONENT_TEST_VISIBLE(c) (c->m_iFlags & PGCOMPONENT_VISIBLE) != 0
#define PGCOMPONENT_TEST_WINDOW(c)  (c->m_iFlags & PGCOMPONENT_WINDOW) != 0
#define PGCOMPONENT_TEST_TOOLTIP(c) (c->m_iFlags & PGCOMPONENT_TOOLTIP) != 0
#define PGCOMPONENT_TEST_ENABLE(c)  (c->m_iFlags & PGCOMPONENT_DISABLED) == 0

#ifdef PGFOCUS
class PGComponent : public PGObject, public PGFocusListenerParent {
#endif
#ifndef PGFOCUS
class PGComponent : public PGObject {
#endif
private:
	PGContainer* m_pParent;
public:
	unsigned int m_iFlags;
	PGString m_sToolTip;
#ifndef PGTINY		
	PGStyle* m_hStyle;	
#endif	
public:
	PGComponent();	
	virtual ~PGComponent();
	PGContainer* getParent();
// Do layout for container (when validated)
	virtual void setVisible(BOOL visible);
	void getPreferredSize(PGDimension* dimension);
	virtual void getPreferredSize(PGDimension* dimension, BOOL recalc) = 0;
	virtual void setBounds(PGRectangle* rectangle);
	virtual void setBounds(int x, int y, int w, int h) = 0;
	virtual void setLayoutSize(PGRectangle* rectangle);
	virtual void requestFocus();
#ifndef PGTINY	
	virtual void paint(PGGraphics* graph);	
	virtual void setStyle(PGStyle* style);	
#endif
	// Set text component flags
	virtual void changeFlags(int orval, int andval);
	BOOL isEnabled();
	void setEnabled(BOOL enable);
	void setFlag(int flag, BOOL enable);
	void setFlags(int flags);
	// Set tooltip text
	void setToolTipText(char* strg);
	BOOL isVisible();
	PGWindow* getWindow();
// Internals
	void addToParent(PGContainer* parent);
	void removeFromParent();
// Create the components native peer
	virtual void addToWindow(PGWindow* parent);
	virtual void removeFromWindow(PGWindow* parent);
};
 
#endif
