/***
 * Portable GUI
 * Code for PGMenu
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifndef _PGMENU_
#define _PGMENU_

#include <pgmain/pgmain.h> 
#include <pgutil/pgvector.h>
#include <pgcomponent/pgactionlistener.h>

class PGMenu;
class PGFrame;

class PGAbstractMenuItem : public PGObject {
public:
	virtual void addToMenu(PGMenu* menu, int idx) = 0;
	virtual void removeFromMenu(PGMenu* menu) = 0;	
	virtual void setFrame(PGFrame* frame);
};

class PGMenuSeparator : public PGAbstractMenuItem {

	#ifdef PGGTK
		#include <pgdep/gtk/pggtkmenu.h>
	#endif

public:
	PGMenuSeparator();
	virtual ~PGMenuSeparator();
	virtual void addToMenu(PGMenu* menu, int idx);
	virtual void removeFromMenu(PGMenu* menu);
};

class PGMenuItem : public PGActionListenerParent, public PGAbstractMenuItem {

	#ifdef PGGTK
		#include <pgdep/gtk/pggtkmenu.h>
	#endif
	
	#ifdef PGWIN
		int m_iId;		
	#endif

public:
	PGString m_sItem;
	PGString m_sStatus;
	PGFrame* m_hFrame;
public:
	PGMenuItem(char* title);
	PGMenuItem(char* title, char* status);
	virtual ~PGMenuItem();
	virtual void addToMenu(PGMenu* menu, int idx);
	virtual void removeFromMenu(PGMenu* menu);
	virtual void onSelect();
	virtual void setFrame(PGFrame* frame);
	void init(char* title, char* status);
};

class PGMenuBar;
 
class PGMenuClosingListener : public PGObject {

public:
	virtual ~PGMenuClosingListener();
	// Convert to string representation
	virtual void toString(PGString* str);
	// Invoked when window activated
	virtual void menuClosing(PGMenuBar* from) = 0;
};

class PGMenuBar : public PGObject {

	#ifdef PGWIN
		#include <pgdep/win/pgwinmenu.h>
	#endif

	#ifdef PGGTK
		#include <pgdep/gtk/pggtkmenu.h>
		#include <pgdep/gtk/pggtkchild.h>
	#endif

public:
	PGVector m_vMenus;
	PGVector m_vMenuClosingListeners;
public:
	PGMenuBar();
	virtual ~PGMenuBar();
	void add(PGMenu* menu);
	void addToFrame(PGFrame* frame);
	void removeFromFrame(PGFrame* frame);
	void onMenuClosing();
	void addMenuClosingListener(PGMenuClosingListener* action);
	void removeMenuClosingListener(PGMenuClosingListener* action);

};
 
class PGMenu : public PGAbstractMenuItem {

	#ifdef PGWIN
		#include <pgdep/win/pgwinmenu.h>
	#endif

	#ifdef PGGTK
		#include <pgdep/gtk/pggtkmenu.h>
		#include <pgdep/gtk/pggtkchild.h>
	#endif

public:
	PGVector m_vItems;
	PGString m_sItem;
	PGFrame* m_hFrame;	
public:
	PGMenu(char* title);
	virtual ~PGMenu();
	void add(PGAbstractMenuItem * item);
	
	void addToMenuBar(PGMenuBar* menu, int idx);
	void removeFromMenuBar(PGMenuBar* menu);
	
	virtual void addToMenu(PGMenu* menu, int idx);
	virtual void removeFromMenu(PGMenu* menu);
	
	virtual void setFrame(PGFrame* frame);
};
 
#endif
