/***
 * Portable GUI
 * Code for PGMouseListener
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifndef _PGMOUSELISTENER_
#define _PGMOUSELISTENER_

class PGMouseListenerParent;

#define PGMOUSEEVENT_BUTTON1_MASK	1
#define PGMOUSEEVENT_BUTTON2_MASK	2
#define PGMOUSEEVENT_BUTTON3_MASK	4
#define PGMOUSEEVENT_BUTTONS_MASK	7

#define PGMOUSELISTENER_DRAG		1
#define PGMOUSELISTENER_MOVE		2
#define PGMOUSELISTENER_ENTER		3
#define PGMOUSELISTENER_EXIT		4
#define PGMOUSELISTENER_PRESS		5
#define PGMOUSELISTENER_RELEASE		6
#define PGMOUSELISTENER_CLICK		7

class PGMouseEvent {
private:
	int m_iX, m_iY;
	int m_iModifiers;	
public:
	PGMouseEvent(int x, int y, int mods);
	// Returns the x,y position
	void getPoint(PGPoint* p);                  
	// Returns the horizontal x position
	int getX();
	// Returns the vertical y position
	int getY();
	// Returns the modifiers flag for this event.
	int getModifiers();
};

class PGMouseMotionListener : public PGListElement {
public:
	// Invoked when dragged (while holding down button).
	virtual void mouseDragged(PGMouseEvent* e);
	// Invoked when mouse moves
	virtual void mouseMoved(PGMouseEvent* e);
	// Convert to string representation
	virtual void toString(PGString* str);	
};
 
class PGMouseListener : public PGListElement {
public:
	// Invoked when the mouse has been clicked.
	virtual void mouseClicked(PGMouseEvent* e);
	// Invoked when the mouse enters a component.                
	virtual void mouseEntered(PGMouseEvent* e);                 
     	// Invoked when the mouse exits a component.                
	virtual void mouseExited(PGMouseEvent* e);            
	// Invoked when a mouse button has been pressed                
	virtual void mousePressed(PGMouseEvent* e);                                 
        // Invoked when a mouse button has been released
	virtual void mouseReleased(PGMouseEvent* e);                  	
	// Convert to string representation
	virtual void toString(PGString* str);
};
 
class PGMouseListenerParent {
public:
	PGAbstractLinkedList m_vMouseListeners;
	PGAbstractLinkedList m_vMotionListeners;
	int m_iModifiers;
	BOOL m_bInside;
public:	
	PGMouseListenerParent();
	virtual ~PGMouseListenerParent();	
	// Dispatching functions
	virtual void onMotion(int x, int y, int mods, int typ);
	virtual void onEvent(int x, int y, int mods, int typ);			
	// MouseListener
	void addMouseListener(PGMouseListener* mouse);
	void removeMouseListener(PGMouseListener* mouse);
	// MouseMotionListener
	void addMouseMotionListener(PGMouseMotionListener* mouse);
	void removeMouseMotionListener(PGMouseMotionListener* mouse);
// Internals	
	void mouseMove(int x, int y);
	void mouseUp(int x, int y, int button);
	void mouseDown(int x, int y, int button);
	void mouseExit(int x, int y);
};

#endif
