/***
 * Portable GUI
 * Code for PGTextArea
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifndef _PGTEXTAREA_
#define _PGTEXTAREA_

#define PGTEXTAREA_SCROLLBARS_NONE            0
#define PGTEXTAREA_SCROLLBARS_VERTICAL_ONLY   1
#define PGTEXTAREA_SCROLLBARS_HORIZONTAL_ONLY 2
#define PGTEXTAREA_SCROLLBARS_BOTH            3
 
class PGTextArea : public PGTextComponent {

public:
	int m_iScrollBar, m_iCols, m_iRows;
public:
	PGTextArea(const char* str, int rows, int cols, int scrollbar);
	// Return a string representation
	virtual void toString(PGString* strg);
	// Return preferred size
	virtual void getPreferredSize(PGDimension* dimension, BOOL recalc);
	// Gets the size of the text in this component
	virtual int getSize();
	// Gets the position of the text insertion caret
	virtual int getCaretPosition();
	// Sets the position of the text insertion caret
	virtual void setCaretPosition(int position);
	// Sets the text that is presented by this text component
	virtual void setText(const char* str);
	// Update component on every change?
	void setDirectUpdate(BOOL directupdate);
	// Appends the given text to the text area's current text.
	void append(const char* str);
	// Appends the given text to the text area's current text.
	void append(const char* str, BOOL addNewLine);
	// Inserts the specified text at the specified position in this text area.
	void insert(const char* str, int pos);
	// Replaces text between the indicated start and end positions with the specified replacement.
	void replaceRange(const char* str, int start, int end);
	// Sets the number of columns for this text area.
	void setColumns(int columns);
	// Sets the number of rows for this text area.
	void setRows(int rows);
// Connect to native peer
	virtual void addToWindow(PGWindow* parent);
	virtual void removeFromWindow(PGWindow* parent);
};
 
#endif

