/***
 * Portable GUI
 * Code for PGTextComponent
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifndef _PGTEXTCOMPONENT_
#define _PGTEXTCOMPONENT_

#define PGTEXTCOMPONENT_EDITABLE    512
#define PGTEXTCOMPONENT_NUMBER      1024
#define PGTEXTCOMPONENT_PASSWORD    2048
#define PGTEXTCOMPONENT_UPPERCASE   4096
 
class PGTextComponent : public PGNormalComponent {

#ifdef PGGTK
	#include <pgdep/gtk/pggtkchild.h>
#endif

public:
	PGString m_sText;	
public:
	PGTextComponent();
	// Returns string repr
	virtual void toString(PGString* strg);	
	// Gets the size of the text in this component
	virtual int getSize();
	// Gets the position of the text insertion caret
	virtual int getCaretPosition();
	// Sets the position of the text insertion caret
	virtual void setCaretPosition(int position);
	// Sets the text that is presented by this text component
	virtual void setText(const char* str);
	// Gets the selected text from the text
	void getText(PGString* str, int start, int end);
	// Gets the start/end position of the selected text
	void getSelection(int* start, int* end);
	// Selects the text between the specified start and end positions
	void select(int selectionStart, int selectionEnd);
	// Undo last edit operation
	void undoOperation();
	// Copy to clipboard
	void copy();
	// Cut to clipboard
	void cut();
	// Paste from clipboard
	void paste();
	
// Not implemented yet
	
	// Adds the specified text event listener
	//void addTextListener(TextListener l);
	// Removes the specified text event listener
	//void removeTextListener(TextListener l)

// Not depending on native implementation
	
	// Indicates whether or not this text component is editable
	BOOL isEditable();
	// Make component editable - works only just after creation
	void setEditable(BOOL yesNo);
	// Gets the text
	void getText(PGString* str);
	// Gets the selected text from the text
	void getSelectedText(PGString* str);
	// Selects all the text in this text component
	void selectAll();	
};

class PGTextField : public PGTextComponent {

public:
	int m_iCols;
public:
	PGTextField(const char* str);
	// Set text component flags
	virtual void changeFlags(int orval, int andval);
	// Return a string representation
	virtual void toString(PGString* strg);
	// Return preferred size
	virtual void getPreferredSize(PGDimension* dimension, BOOL recalc);
	// Gets the size of the text in this component
	virtual int getSize();
	// Gets the position of the text insertion caret
	virtual int getCaretPosition();
	// Sets the position of the text insertion caret
	virtual void setCaretPosition(int position);
	// Sets the text that is presented by this text component
	virtual void setText(const char* str);	
	// Sets the number of columns for this text area.
	void setColumns(int columns);	
// Connect to native peer
	virtual void addToWindow(PGWindow* parent);
	virtual void removeFromWindow(PGWindow* parent);
};

class PGIntegerField : public PGTextField {
public:
	PGIntegerField(int value);
	int getValue();
	void setValue(int value);
};
 
#endif
