/***
 * Portable GUI
 * Code for PGBmp
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _PGBMP_
#define _PGBMP_

#define PG_BMP_SIGNATURE_WORD 0x4D42

#define PGPIXELSIZE(depth) depth == 24 ? 3 : 1
#define PGCOLORCNT(depth) depth == 24 ? 0 : 1 << depth

int PGLoadBMP(PGInputStream* input, PGImageReader* reader);

enum PGStatusCodes {
   PG_OKAY        = 0x00,
   PG_ENDOFIMAGE  = 0x01,
   PG_ENDOFFILE   = 0x02,
   PG_IOERROR     = 0x04,
   PG_NOMEMORY    = 0x08,
   PG_OUTOFSYNC   = 0x10,
   PG_OVERFLOW    = 0x20,
   PG_BADPARAM    = 0x40,
   PG_UNSUPPORTED = 0x80
};

enum PGBiCompressionMethods {
   PG_BI_NONE = 0,
   PG_BI_RGB  = 0,
   PG_BI_RLE4 = 2,
   PG_BI_RLE8 = 1
};

class PGBitmapFileHeader {
public:
   char bfSignature[3];          // signature - 'BM'
   unsigned long  bfSize;        // file size in bytes
   unsigned long  bfOffBits;     // offset to bitmap
public:
   PGBitmapFileHeader();
   BOOL read(PGInputStream* input);
   BOOL isValid();
};

class PGRGBQuad {
public:
   unsigned char rgbBlue;       // 0..255
   unsigned char rgbGreen;      // 0..255
   unsigned char rgbRed;        // 0..255
   unsigned char rgbReserved;   // 0
public:
   PGRGBQuad();
   PGRGBQuad(int r, int g, int b);
   BOOL read(PGInputStream* input);
};

class PGBitmapInfoHeader {
public:
   long           biWidth;         // bmap width in pixels
   long           biHeight;        // bmap height in pixels
   unsigned short biPlanes;        // num planes - always 1
   unsigned short biBitCount;      // bits per pixel
   unsigned long  biCompression;   // compression flag
   unsigned long  biSizeImage;     // image size in bytes
   long           biXPelsPerMeter; // horz resolution
   long           biYPelsPerMeter; // vert resolution
   unsigned long  biClrUsed;       // 0 -> color table size
   unsigned long  biClrImportant;  // important color count
public:
   PGBitmapInfoHeader();
   BOOL read(PGInputStream* input);
};

class PGBmpDecoder {
   public:
      int cond;           // condition code

   protected:
      int   rle_method;   	   // RLE compression method
      int   pxl_size;		   // bits per pixel
      PGInputStream* bmp_file;     // input file

   private:
      void read_none(unsigned char *buf, int npxls);
      void read_rle4(unsigned char *buf, int npxls);
      void read_rle8(unsigned char *buf, int npxls);

   public:
      PGBmpDecoder(PGInputStream* input, int rle, int nbits);
      virtual int decode(unsigned char *buf, int npxls);
      virtual int status(void);
};

#endif

