/***
 * Portable GUI
 * Code for PGGif
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _PGGIF_
#define _PGGIF_

#include <pgio/pginputstream.h>
#include <pgdraw/pgimage.h>

#define LZW_MAXVAL  4096

int PGLoadGIF(PGInputStream* input, PGImageReader* reader);

class PGGIFFileHeader {
public:
   char gfSignature[4];          // signature - 'GIF'
   int  gfMajorVersion;
   char gfMinorVersion;
public:
   PGGIFFileHeader();
   BOOL read(PGInputStream* input);
   BOOL isValid();
};

class PGGIFLogicalScreenDescriptor {
public:
   unsigned int gfScreenWd;
   unsigned int gfScreenHi;
   BOOL gfGlobalColorFlag;
   unsigned char gfColorResolution;
   unsigned char gfColorTableSize;
   unsigned char gfBackgroundColor;
   unsigned char gfAspectRatio;
public:
   BOOL read(PGInputStream* input);
};

class PGGIFImageDescriptor {
public:
   PGWORD gfLeftPos, gfTopPos;
   PGWORD gfWidth, gfHeight;
   BOOL gfLocalColorTable;
   BOOL gfInterlaced;
   BOOL gfTranspFlag;
   PGBYTE gfColorTableSize;
   PGBYTE gfDisposal;   
   PGBYTE gfTranspIndex;
   PGWORD gfDelay;
public:
   PGGIFImageDescriptor();
   BOOL read(PGInputStream* input);   
};

class PGGIFDecoder {
public:
   int crLine, crPos, crPass;
   PGBYTE *last, *topGifBuff, *gifBuff, *scanLine;
   PGDWORD *first, rootCodeSize, codeSize, expected, mask, old;   
public:
   PGGIFDecoder(int wd);
   virtual ~PGGIFDecoder();
   void ClearTable();
   int Deinterlace(int nbrows);
   int PGGIFLoadImageData(PGInputStream* input, PGGIFImageDescriptor* descr, PGImageReader* reader);
   void StoreBytes(PGImageReader* reader, PGGIFImageDescriptor* descr, int bytes, PGBYTE* source);
};

class iCodeStream
{
   protected :

      PGInputStream* ifile;          // input file
      unsigned char  buffer[255];    // data buffer
      int            nbytes;         // bytes in buffer
      int            curbyte;        // index of current byte
      int            nbits;          // current element width
      int            bytecode;       // last byte read
      int            bytemask;       // mask for bytecode
      int            bitcode;        // last bit code read
      int            bitmask;        // mask for bitcode

   private :

      int  getbyte( void );

   public :

      iCodeStream(PGInputStream* f, int size );
      virtual ~iCodeStream( );
      int getcode( void );
      int lastcode( void )
      {
         return bitcode;
      }
      int width( void )
      {
         return nbits;
      }
      int width( int nb )
      {
        nbits = nb;
        return nbits;
      }
};

//..................class GifDecoder

class GifDecoder : public iCodeStream
{
   protected:
      int  cond;      // condition code
      int  nroots;    // 1 << pixelsize
      int  capacity;  // 1 << 12
      int  rwbytes;   // bytes per scan line

      int  minsize;   // code sizes
      int  cursize;
      int  maxsize;
      int  curcode;   // code values
      int  maxcode;

      int  newc;       // new code
      int  oldc;       // old code
      int  clrc;       // clear code
      int  endc;       // end code
      int  stkp;       // stack pointer

      int *prefix;
      unsigned char * suffix;
      unsigned char * stack;
      unsigned char * bytes;

   private:
      void clear( void );
      int  push( int s );

   public:
      GifDecoder( PGInputStream*, int cobits, int pxcnt );
     ~GifDecoder( );
      virtual int init( void );
      virtual int term( void );
      virtual int decode( unsigned char *line, int size );
      virtual int status( void );
};

#define PGGIF_OK	0
#define PGGIF_NOMEM	1
#define PGGIF_OVERFLOW	2
#define PGGIF_EOF	3
#define PGGIF_IOERR	4
#define PGGIF_SYNC	5

#endif
