/***
 * Portable GUI
 * Code for PGGraphics
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _PGGRAPHICS_
#define _PGGRAPHICS_

#include <pgdraw/pgcolor.h>
#include <pgdraw/pgfont.h>
#include <pgdraw/pgfontmetrics.h>

#ifdef PGSHAPE
	#include <pgdraw/pgshape.h>
#endif

class PGNormalComponent;
class PGImage;
class PGColor;

#define PGGRAPHICS_OWNER_COMPONENT	0
#define PGGRAPHICS_OWNER_BITMAP		1
#define PGGRAPHICS_OWNER_SYSTEM		2

class PGGraphics : public PGObject {

	#ifdef PGWIN
		#include <pgdep/win/pgwingraph.h>
	#endif

	#ifdef PGGTK
		#include <pgdep/gtk/pggtkgraph.h>
	#endif
public:
	PGColor m_Color;
	int m_iOwnerMode;
	int m_iLineWidth;
public:
	PGGraphics();
	virtual ~PGGraphics();
	// Clears the specified rectangle by filling it with the background color.
	void clearRect(int x, int y, int width, int height);
        // Intersects the current clip with the specified rectangle.
	void clipRect(int x, int y, int width, int height);
	// Copies an area of the component by a distance specified by dx and dy.
	void copyArea(int x, int y, int width, int height, int dx, int dy);
	// Draws a 3-D highlighted outline of the specified rectangle.
	// FIXME: Not implemented yet - don't know what it does :O)
 	void draw3DRect(int x, int y, int width, int height, BOOL raised);
        // Draws the outline of a circular or elliptical arc covering the specified rectangle.
	void drawArc(int x, int y, int width, int height, int startAngle, int arcAngle);
	// Draws a given image.
	void drawImage(PGImage* img, int x, int y);
	// Draws a part of a given image.
	void drawImage(PGImage* img, int x, int y, int width, int height);
	// Draws a part of a given image.
	void drawImage(PGImage* img, int dx, int dy, int sx, int sy, int width, int height);
	// Draws the specified image, scaling it on the fly.
	// FIXME: Not implemented yet
	void drawImage(PGImage* img, int dx1, int dy1, int dx2, int dy2, int sx1, int sy1, int sx2, int sy2);
        // Draws a line, using the current color, between the points (x1, y1) and (x2, y2)
	void drawLine(int x1, int y1, int x2, int y2);
	// Draws the outline of an oval.
	void drawOval(int x, int y, int width, int height);
        // Draws a closed polygon defined by arrays of x and y coordinates.
	void drawPolygon(int* xPoints, int* yPoints, int nPoints);
        // Draws a sequence of connected lines defined by arrays of x and y coordinates.
	void drawPolyline(int* xPoints, int* yPoints, int nPoints);
        // Draws the outline of the specified rectangle.
 	void drawRect(int x, int y, int width, int height);
        // Draws an outlined round-cornered rectangle using this graphics context's current color.
	void drawRoundRect(int x, int y, int width, int height, int arcWidth, int arcHeight);
        // Draws the text given by the specified string, using this graphics context's current font and color.
	void drawString(const char *str, int x, int y);
	// Draws the text given by the specified string, using this graphics context's current font and color.
	void drawString(PGString *str, int x, int y);
        // Paints a 3-D highlighted rectangle filled with the current color.
 	void fill3DRect(int x, int y, int width, int height, BOOL raised);
        // Fills a circular or elliptical arc covering the specified rectangle.
	void fillArc(int x, int y, int width, int height, int startAngle, int arcAngle);
        // Fills an oval bounded by the specified rectangle with the current color.
	void fillOval(int x, int y, int width, int height);
        // Fills a closed polygon defined by arrays of x and y coordinates.
	void fillPolygon(int* xPoints, int* yPoints, int nPoints);
#ifdef PGSHAPE	
        // Draws the outline of a polygon defined by the specified Polygon object.
        void drawPolygon(PGPolygon* p);	
        // Fills the polygon defined by the specified Polygon object.
 	void fillPolygon(PGPolygon* p);
#endif 	
        // Fills the specified rectangle.
	void fillRect(int x, int y, int width, int height);
        // Fills the specified rounded corner rectangle with the current color.
	void fillRoundRect(int x, int y, int width, int height, int arcWidth, int arcHeight);
	// Set the width for drawing lines
	void setLineWidth(int wd);
        // Gets the current clipping area.
	// Shape getClip();
	// Returns the bounding rectangle of the current clipping area.
	void getClipBounds(PGRectangle* bounds);
        // Gets this graphics context's current color.
 	PGColor* getColor();
        // Gets the font metrics of the current font.
 	void getFontMetrics(PGFontMetrics* fm);
 	// Gets the font metrics for the specified font.
	void getFontMetrics(PGNormalComponent* comp, PGFont* f, PGFontMetrics* fm);
        // Returns true if the specified rectangular area intersects the bounding rectangle of the current clipping area.
 	BOOL hitClip(int x, int y, int width, int height);
        // Sets the current clip to the rectangle specified by the given coordinates.
	void setClip(int x, int y, int width, int height);
        // Sets this graphics context's current color to the specified color.
	void setColor(PGColor* c);
        // Sets this graphics context's font
	void setFont(PGRealizedFont* font);
	// Sets this graphics context's font
	void setFont(PGNormalComponent* comp, PGFont* font);
// Internals
	void realize();
	void unrealize();
	void init();
};

#endif
