/***
 * Portable GUI
 * Code for PGINIFile
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifndef _PGINIFILE_
#define _PGINIFILE_

#include <pgio/pginputstream.h>
#include <pgio/pgoutputstream.h>
#include <pgutil/pgvector.h>

class PGINIFileValue : public PGObject {
protected:
	PGString m_sKey;
	PGString m_sValue;
public:	
	PGINIFileValue(const char* key, const char* value);
	void setValue(const char* value);
	PGString* getValue();
	BOOL equals(const char* key);
};

class PGINIFileSection : public PGObject {
protected:
	PGString m_sHeading;
	PGVector m_vValues;
public:
	PGINIFileSection(const char* heading); 
	void read(PGInputStream* stream);
	PGString* getStringValue(const char* key);
	PGINIFileValue* getValue(const char* key);
	BOOL equals(const char* heading);
};

class PGINIFileSectionReader : public PGObject {
protected:
	PGString m_sHeading;
public:
	PGINIFileSectionReader(const char* heading);
	BOOL equals(const char* heading);
	virtual void readSection(PGInputStream* stream) = 0;
};

class PGINIFile : public PGObject {
protected:
	PGVector m_vSections;
	PGVector m_vReaders;
public:
	void read(PGInputStream* stream);
	PGINIFileSection* getSection(const char* name);
	void addSectionReader(PGINIFileSectionReader* reader);
};

class PGINIFileWriter : public PGObject {
protected:
	PGOutputStream* m_hStream;
public:
 	PGINIFileWriter(PGOutputStream* stream);
 	void startSection(const char* heading);
 	void writeValue(const char* key, const char* value); 
 	void writeBoolValue(const char* key, BOOL value); 
 	void writeIntValue(const char* key, int value); 
};
 
#endif
