/***
 * Portable GUI
 * Code for PGInputStream
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifndef _PGINPUTSTREAM_
#define _PGINPUTSTREAM_

#include <pgmain/pgmain.h> 
#include <pgutil/pgstring.h> 

#define PGINPUTSTREAM_ERROR_NONE	0
#define PGINPUTSTREAM_ERROR_END		1
#define PGINPUTSTREAM_ERROR_OPEN	2

class PGInputStream : public PGObject {
public:
	long m_lPosition;
	int m_cPushBack;
public:
	PGInputStream();
	// Test if error (at end is also error!)
	virtual int getError() = 0;
	// Reads the next byte of data from the input stream
	virtual int read() = 0;
	// Reads up to len bytes of data from the input stream into an array of bytes
        virtual int read(unsigned char* buffer, int off, int len);
        // Skips over and discards n bytes of data from this input stream
	virtual long skip(long len);
	// Skips over chars like space, tab, CR and LF
	void skipSeparators();
	// Pushback a character (only one allowed)
	void pushback(int ch);
	// Return and reset pushback
	int getPushback();
	// Reads some number of bytes from the input stream and stores them into the buffer                  .
     	int read(unsigned char* buffer, int len);
	// Skips till given position (works only forward)
	long seek(long pos);
	// Returns current stream position
	long getPosition();
	// Reads in one line
	void readLine(PGString* str);
	// Read 16 bits LE integer
	int read16LEInt();
	// Read 32 bits LE integer
	long read32LEInt();
	// Test if at end
	BOOL isAtEnd();
};

BOOL PGIsSeparator(int ch);
 
#endif
