/***
 * Portable GUI
 * Code for PGString
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifndef _PGString_
#define _PGString_
 
class PGString : public PGObject {
private:
	char* m_pBuffer;
	int m_iSize;
public:
	PGString();
	PGString(const char* str);
	PGString(const PGString &str);
	PGString(int value);
	~PGString();
	void format(const char *format, ...);
	virtual void toString(PGString* str);		
	void setSize(int size);
	void growToSize(int size);
	void reset(int size);
	BOOL equals(PGString* str) const;
	BOOL equals(const char* str) const;
	void setCopyValue(const char* str);
	int charAt(int pos) const;	
	void copyTo(char* str) const;					// Not tested/implemented
	void copyRange(int start, int end, char* dest) const;		// Not tested/implemented
	void prepend(const char*str);					// Not tested/implemented
	void append(const char* str);					
	void append(char ch);
	void append(const char* str, BOOL newline);
	void insert(const char* str, int pos);				// Not tested/implemented
	void remove(int start, int end);				// Not tested/implemented
	void replace(char oldchar, char newchar);
	void substring(int ofs);
	void substring(int start, int end);
	void terminate(char ch);	
	int indexOf(const char* str);					// Not tested/implemented
	int indexOf(int ch);						// Not tested/implemented
	int lastIndexOf(int ch);	
	void toUpperCase();
	const char* getValue() const;
	char* getBuffer();
	int getLength() const;
	BOOL getBoolValue() const;
	int getIntValue() const;
	PGString operator+(PGString str);
	PGString & operator=(const PGString &str);
	PGString & operator+=(const PGString &str);
	operator const char*();
protected:
	char* changeSize(int size);	
private:
	void init();
};
 
#endif
