/***
 * Portable GUI
 * Code for PGWindow
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifndef _PGWINDOW_
#define _PGWINDOW_

#include <pgcomponent/pgnormalcontainer.h> 
#include <pgwindow/pgwindowlistener.h> 

#define PGWINDOW_ACTIVATED   0
#define PGWINDOW_CLOSED      1
#define PGWINDOW_CLOSING     2
#define PGWINDOW_DEACTIVATED 3
#define PGWINDOW_DEICONIFIED 4
#define PGWINDOW_ICONIFIED   5
#define PGWINDOW_OPENED      6

#define PGWINDOW_TOOLTIPS_NONE		0
#define PGWINDOW_TOOLTIPS_DISABLED	1
#define PGWINDOW_TOOLTIPS_ENABLED	2
#define PGWINDOW_TOOLTIPS_ERROR		3

#ifndef PGTINY
	class PGIcon;
	class PGLabel;
#endif
 
class PGWindow : public PGNormalContainer {
private:
	PGAbstractLinkedList m_vListeners;
	
#ifndef PGTINY
	PGVector m_vRealizedFonts;
	PGIcon* m_hIcon;
	PGLabel* m_hStatus;	
#endif

public:	
	#ifndef PGTINY	
		#ifdef PGWIN
			HWND m_hWndToolTip;
		#endif		
		
		#ifdef PGGTK
			GtkTooltips* m_hToolTips;
		#endif				
	#endif
public:
	BOOL m_bIconified;
	BOOL m_bFirstShow;	
	int m_iToolTipsFlags;	
public:
	PGWindow();
	virtual ~PGWindow();
	void addWindowListener(PGWindowListener* listener);
	void removeWindowListener(PGWindowListener* listener);
	virtual void pack();
	virtual void dispose();
	virtual void getPreferredSize(PGDimension* dimension, BOOL recalc);
	//PGImage* getIconImage();	
	void getTitle(PGString* title);
	void setTitle(const char* str);
	void setStatus(const char* status);	
	virtual PGWindow* getWindow();	
#ifndef PGTINY
	BOOL initToolTips();
	void setIconImage(PGIcon* icon);
	void createImage(int w, int h, PGImage* img);	
	void setStatusDisplay(PGLabel* display);	
	PGRealizedFont* getRealizedFont(PGFont* font);	
#endif
	virtual void notifyWindowListeners(int notify);
private:
	void init();
	void term();	
};

// Called when user tries to close window
class PGCloseWindowListener : public PGWindowListener {
	
	virtual void windowClosing(PGWindow* wnd);		
};
 
#endif
