/***
 * Portable GUI
 * Code for PGAnimatedImageViewer
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <pgcomponent/pganimatedimgviewer.h>
#include <portablegui.h>

PGAnimatedImageViewer::PGAnimatedImageViewer(PGAnimatedImage* image) : PGTimer(image->m_hOwner) {
	init();	
	setImage(image);
}

PGAnimatedImageViewer::PGAnimatedImageViewer(int w, int h, PGWindow* owner) : PGTimer(owner) {
	init();
	m_iWidth = w;
	m_iHeight = h;	
}

PGAnimatedImageViewer::~PGAnimatedImageViewer() {
	if (m_hImage != NULL) PGDECREF(m_hImage);
}

void PGAnimatedImageViewer::init() {
	m_bAnimationBusy = FALSE;
	m_hImage = NULL;
	m_iSpeed = 100;	
	m_iCrFrame = -1;
}

void PGAnimatedImageViewer::setImage(PGAnimatedImage* image) {
	if (m_hImage != NULL) PGDECREF(m_hImage);
	m_hImage = image;
	PGINCREF(image);
	m_bFirstTime = TRUE;
	m_bAnimationBusy = TRUE;
	reset();
}

void PGAnimatedImageViewer::startAnimation() {
	if (m_hImage != NULL) {
		m_bAnimationBusy = TRUE;
		if (m_bFirstTime) repaint();
		else tick();
	}
}

void PGAnimatedImageViewer::stopAnimation() {
	if (m_hImage != NULL) {	
		m_bAnimationBusy = FALSE;
		stopTimer();
	}
}

void PGAnimatedImageViewer::paint(PGGraphics* graph) {
	PGDimension dim;
	getSize(&dim);
	if (m_hImage != NULL) {
		if (m_bFirstTime && !m_hImage->hasError()) {
			m_hBuffer.reinitialize();
			createImage(m_hImage->m_iCols, m_hImage->m_iRows, &m_hBuffer);
			m_bFirstTime = FALSE;
			reset();
			tick();		
		}
		if (!m_hBuffer.hasError())  {
			int xpos = dim.width/2 - m_hImage->m_iCols/2;
			int ypos = dim.height/2 - m_hImage->m_iRows/2;
			graph->drawImage(&m_hBuffer, xpos, ypos, 0, 0, m_hImage->m_iCols, m_hImage->m_iRows);
		}
	}
}

void PGAnimatedImageViewer::reset() {
	m_iLoop = 0;
	initialframe();
}

void PGAnimatedImageViewer::initialframe() {
	// Fill with background color?
	if (!m_hBuffer.hasError())  {
		PGGraphics* graph = m_hBuffer.getGraphics();
//		graph->clearRect(0, 0, m_iWidth, m_iHeight);
		graph->setColor(PGCOLOR_WHITE);
		graph->fillRect(0, 0, m_hBuffer.m_iCols, m_hBuffer.m_iRows);
	}
	m_iCrFrame = -1;
}

void PGAnimatedImageViewer::showFrame(int idx) {
	if (m_hImage != NULL && !m_hBuffer.hasError())  {
		PGGraphics* graph = m_hBuffer.getGraphics();
		PGAnimatedImageFrame* frame = m_hImage->getFrame(idx);
		getPGApplication()->printf("HI: %d\n", frame->m_iDisposalMethod);		
		graph->drawImage(frame, frame->m_iXPos, frame->m_iYPos);			
	}
}

void PGAnimatedImageViewer::disposeFrame(int idx) {
	if (m_hImage != NULL && !m_hBuffer.hasError())  {
		PGGraphics* graph = null;
		PGAnimatedImageFrame* frame = m_hImage->getFrame(idx);
		switch (frame->m_iDisposalMethod) {
			case PG_DISPOSAL_NONE:
			case PG_DISPOSAL_LEAVE_IMAGE:
				break;
			case PG_DISPOSAL_RESTORE_BACKGROUND:
				graph = m_hBuffer.getGraphics();							
				// graph->clearRect(frame->m_iXPos, frame->m_iYPos, frame->m_iCols, frame->m_iRows);
				graph->setColor(PGCOLOR_WHITE);
				graph->fillRect(frame->m_iXPos, frame->m_iYPos, frame->m_iCols, frame->m_iRows);				
				break;
			case PG_DISPOSAL_RESTORE_PREVIOUS:
				getPGApplication()->printf("Animated gif: Restore previous not implemented yet..\n");	
				break;		
		}
	}
}

// if (frame->m_iDisposalMethod > 1) {
//			printf("Disposal: %d\n", frame->m_iDisposalMethod);
// }		

void PGAnimatedImageViewer::gotoFrame(int idx) {
	if (m_hImage != NULL) {	
		if (m_iCrFrame > idx) initialframe();
		while (m_iCrFrame < idx) {
			if (m_iCrFrame != -1) disposeFrame(m_iCrFrame);
			m_iCrFrame++;
			showFrame(m_iCrFrame);		
		}
	}
}

void PGAnimatedImageViewer::showNextFrame(BOOL skipZeroDelay) {
	if (m_hImage != NULL) {
		int nb = m_hImage->getNbFrames();
		int idx = (m_iCrFrame + 1) % nb;		
		if (skipZeroDelay) {
			int skip = 0;			
			while (m_hImage->getFrame(idx)->m_iDelay == 0 && skip < nb) {
				idx = (idx + 1) % nb;
				skip++;
			}
		}
		if (idx < m_iCrFrame) m_iLoop++;
		gotoFrame(idx);
	}
}

PGAnimatedImageFrame* PGAnimatedImageViewer::getFrame() {
	return m_hImage->getFrame(m_iCrFrame);
}

int PGAnimatedImageViewer::getNbFrames() {
	return m_hImage->getNbFrames();
}

int PGAnimatedImageViewer::getNbLoops() {
	return m_hImage->m_iLoops;
}

void PGAnimatedImageViewer::tick() {
	if (!m_hBuffer.hasError() && m_hImage != NULL)  {
		// Show next frame of animation
		showNextFrame(TRUE);
		repaint();
		// Reinitialize timer for next image
		if (m_bAnimationBusy && getFrame()->m_iDelay != 0) {
			long crdelay = (long)getFrame()->m_iDelay*1000/m_iSpeed;
			if (isRunning()) {
				if (m_iDelay != crdelay) {
					stopTimer();
					startTimer(crdelay);
				}
			} else {
				startTimer(crdelay);
			}
		} else {
			m_bAnimationBusy = FALSE;		
			stopTimer();
		}
	} else {
		m_bAnimationBusy = FALSE;	
		stopTimer();	
	}
}

void PGAnimatedImageViewer::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	if (m_hImage != NULL) {
		dimension->width = m_hImage->m_iCols;
		dimension->height = m_hImage->m_iRows;
	} else {
		dimension->width = m_iWidth;
		dimension->height = m_iHeight;
	}
}
