/***
 * Portable GUI
 * Code for PGButton
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

PGAbstractButton::PGAbstractButton(char* label) {
	init(label, FALSE);
}

PGAbstractButton::PGAbstractButton(char* label, BOOL isDefault) {
	init(label, isDefault);
}

void PGAbstractButton::init(char* label, BOOL isDefault) {
	m_sLabel.setCopyValue(label);
	m_bDefault = isDefault;
}

PGButton::PGButton(char* label) : PGAbstractButton(label) {
}

PGButton::PGButton(char* label, BOOL isDefault) : PGAbstractButton(label, isDefault) {
}

void PGButton::toString(PGString* str) {
	PGObject::toString(str);
	str->substring(8);
	str->prepend(m_sLabel.getValue());
	str->prepend("PGButton ");
}

PGCheckbox::PGCheckbox(char* label) : PGAbstractButton(label) {
	m_bState = FALSE;
}

void PGCheckbox::toString(PGString* str) {
	PGObject::toString(str);
	str->substring(8);
	str->prepend(m_sLabel.getValue());
	str->prepend("PGCheckbox ");
}

PGRadioButton::PGRadioButton(char* label) : PGAbstractButton(label) {
	m_bState = FALSE;
}

PGRadioButton::PGRadioButton() : PGAbstractButton("") {
	m_bState = FALSE;
}

PGRadioButton::~PGRadioButton() {
}

void PGRadioButton::toString(PGString* str) {
	PGObject::toString(str);
	str->substring(8);
	str->prepend(m_sLabel.getValue());
	str->prepend("PGRadioButton ");
}

PGButtonGroup::PGButtonGroup() {
	m_vButtons.setDoCleanUp(FALSE);	
	m_iIndex = 0;
}

void PGButtonGroup::add(PGRadioButton* button) {
	if (m_vButtons.getSize() == 0) button->setState(TRUE);
	m_vButtons.addElement(button);
	button->addActionListener(this);
	button->m_hGroup = this;
}

// FIXME: What if the last button is removed?
//        Up till now, the group is deleted!
void PGButtonGroup::remove(PGRadioButton* button) {
	m_vButtons.removeElement(button);
	button->removeActionListener(this);
}

int PGButtonGroup::getSelectedIndex() {
	return m_iIndex;
}

void PGButtonGroup::makeButtons(PGWindow* parent) {
	PGRadioButton* prev = null;
	for (int idx = 0; idx < m_vButtons.getSize(); idx++) {
		PGRadioButton* button = (PGRadioButton*)m_vButtons.getElementAt(idx);
		button->makeButton(parent, prev);
		prev = button;
	}
}

void PGButtonGroup::setSelectedIndex(int idx) {	
	if (idx != m_iIndex) {
		int size = m_vButtons.getSize();
		if (m_iIndex < size) {
			PGRadioButton* prev = (PGRadioButton*)m_vButtons.getElementAt(m_iIndex);
			prev->setState(FALSE);
		}		
		if (idx < size) {
			PGRadioButton* now =  (PGRadioButton*)m_vButtons.getElementAt(idx);
			now->setState(TRUE);
			m_iIndex = idx;
		}	
	}	
}

void PGButtonGroup::actionPerformed(PGActionListenerParent* from) {
	int idx = 0;
	int size = m_vButtons.getSize();	
	while (idx < size) {
		PGRadioButton* button = (PGRadioButton*)m_vButtons.getElementAt(idx);
		if (((PGActionListenerParent*)button) == from) {
			if (shouldSelect()) setSelectedIndex(idx);
			else m_iIndex = idx;
			break;
		}
		idx++;
	}	
}
