/***
 * Portable GUI
 * Code for PGCanvas
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

void PGCanvas::toString(PGString* str) {
	PGObject::toString(str);
	str->substring(8);
	str->prepend("PGCanvas ");
}

void PGCanvas::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	dimension->width = 200;
	dimension->height = 200;
}

PGDoubleBufferCanvas::~PGDoubleBufferCanvas() {
}

void PGDoubleBufferCanvas::setBounds(int x, int y, int w, int h) {
	PGCanvas::setBounds(x, y, w, h);
	if (m_hBufferImage.m_iCols != w || m_hBufferImage.m_iRows != h) {
		createImage(w, h, &m_hBufferImage);
	}
}

void PGDoubleBufferCanvas::paint(PGGraphics* graph) {
	PGRectangle rect;
	graph->getClipBounds(&rect);
	// Use double buffering
	PGGraphics* bufGrp = getBufferGraphics();
	bufGrp->setClip(rect.x, rect.y, rect.width, rect.height);
	// Call draw method
	draw(bufGrp);
	// Update screen
	redraw(rect.x, rect.y, rect.width, rect.height, graph);
}

PGGraphics* PGDoubleBufferCanvas::getBufferGraphics() {
	return m_hBufferImage.getGraphics();
}

void PGDoubleBufferCanvas::redraw(int x, int y, int w, int h, PGGraphics *graph) {
	graph->drawImage(&m_hBufferImage, x, y, x, y, w, h);
}

void PGDoubleBufferCanvas::redraw(int x, int y, int w, int h) {
	PGGraphics graph;
	getGraphics(&graph);
	redraw(x, y, w, h, &graph);
}






