/***
 * Portable GUI
 * Code for PGComponent
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <portablegui.h>

PGComponent::PGComponent() {
	m_pParent = (PGContainer*)NULL;	
	m_iFlags = 0;
#ifndef PGTINY
	m_hStyle = (PGStyle*)NULL;
#endif	
}

PGComponent::~PGComponent() {
#ifndef PGTINY	
	if (m_hStyle != NULL) PGDECREF(m_hStyle);
#endif
}

PGContainer* PGComponent::getParent() {
	return m_pParent;
}

void PGComponent::getPreferredSize(PGDimension* dimension) {
	getPreferredSize(dimension, FALSE);
}

void PGComponent::addToParent(PGContainer* parent) {
	m_pParent = parent;
}

void PGComponent::removeFromParent() {
	m_pParent = (PGContainer*)NULL;
}

void PGComponent::setLayoutSize(PGRectangle* rectangle) {
	setBounds(rectangle);
}

void PGComponent::setBounds(PGRectangle* rectangle) {
	setBounds(rectangle->x, rectangle->y, rectangle->width, rectangle->height);
}

void PGComponent::addToWindow(PGWindow* parent) {
	m_iFlags |= PGCOMPONENT_WINDOW;
}

void PGComponent::removeFromWindow(PGWindow* parent) {
	m_iFlags &= PGCOMPONENT_ALLFLAGS ^ PGCOMPONENT_WINDOW;
}

void PGComponent::setVisible(BOOL visible) {
	if (visible) m_iFlags |= PGCOMPONENT_VISIBLE;
	else m_iFlags &= PGCOMPONENT_ALLFLAGS ^ PGCOMPONENT_VISIBLE;
}

BOOL PGComponent::isVisible() {
	return (m_iFlags & PGCOMPONENT_VISIBLE) != 0;
}

void PGComponent::setEnabled(BOOL enable) {
	if (!enable) changeFlags(PGCOMPONENT_DISABLED, PGCOMPONENT_ALLFLAGS);
	else changeFlags(0, PGCOMPONENT_ALLFLAGS ^ PGCOMPONENT_DISABLED);
}

BOOL PGComponent::isEnabled() {
	return (m_iFlags & PGCOMPONENT_DISABLED) == 0;
}

#ifndef PGTINY
	void PGComponent::setStyle(PGStyle* style) {
		if (m_hStyle != NULL) PGDECREF(m_hStyle);
		m_hStyle = style;
		PGINCREF(style);
	}
#endif

void PGComponent::setToolTipText(char* strg) {
	m_sToolTip.setCopyValue(strg);
}

PGWindow* PGComponent::getWindow() {
	PGContainer* parent = getParent();
	if (parent != NULL) return parent->getWindow();
	return (PGWindow*)NULL;
}

void PGComponent::requestFocus() {
}

void PGComponent::changeFlags(int orval, int andval) {
	m_iFlags = (m_iFlags & andval) | orval;
}

void PGComponent::setFlag(int flag, BOOL enable) {
	if (enable) changeFlags(flag, PGCOMPONENT_ALLFLAGS);
	else changeFlags(0, PGCOMPONENT_ALLFLAGS ^ flag);
}

void PGComponent::setFlags(int flags) {
	changeFlags(flags, 0xFF);
}

#ifndef PGTINY
	void PGComponent::paint(PGGraphics* graph) {
	}


	void PGNormalComponent::getSize(PGDimension* dim) {
		PGRectangle rect;
		getBounds(&rect);
		dim->width = rect.width;
		dim->height = rect.height;	
	}
#endif
