/***
 * Portable GUI
 * Code for PGContainer
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <portablegui.h>

PGContainer::PGContainer() {
	m_pLayoutManager = (PGLayoutManager*)NULL;
}

PGContainer::~PGContainer() {
	if (m_pLayoutManager != NULL) PGDECREF(m_pLayoutManager);
}

void PGContainer::setLayout(PGLayoutManager* layout) {
	if (m_pLayoutManager != NULL) PGDECREF(m_pLayoutManager);
	m_pLayoutManager = layout;
	PGINCREF(layout);
}

#ifndef PGTINY
	void PGContainer::setStyle(PGStyle* style) {
		PGComponent::setStyle(style);
		for (int idx = 0; idx < getComponentCount(); idx++) {
			PGComponent* comp = getComponentAt(idx);
			if (comp->m_hStyle == NULL) comp->setStyle(style);
		}	
	}
#endif

void PGContainer::setLayoutSize(PGRectangle* rectangle) {
	if (m_pLayoutManager != NULL)
		m_pLayoutManager->layoutContainer(this, rectangle);
}

void PGContainer::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	if (m_pLayoutManager != NULL) {
		m_pLayoutManager->getPreferredSize(this, dimension, recalc);
	} else {
		dimension->width = 400;
		dimension->height = 150;
	}
}

PGWindow* PGContainer::getWindow() {
	PGContainer* parent = getParent();
	if (parent != NULL) return parent->getWindow();
	return (PGWindow*)NULL;
}

void PGContainerDoAdd(PGContainer* cont, PGComponent* comp) {
	comp->addToParent(cont);	
#ifndef PGTINY	
	if (cont->m_hStyle != NULL && comp->m_hStyle == NULL)
		comp->setStyle(cont->m_hStyle);		
#endif		
	if (PGCOMPONENT_TEST_WINDOW(cont)) 
		comp->addToWindow(cont->getWindow());
}

void PGContainer::add(PGComponent* comp) {
	m_vComponents.addElement(comp);
	PGContainerDoAdd(this, comp);
}

void PGContainer::add(PGComponent* comp, int idx) {
	PGComponent* prev = (PGComponent*)m_vComponents.getElementAt(idx);
	if (prev == comp) return;
	if (prev != NULL) {
		PGWindow* win = NULL;
		if (PGCOMPONENT_TEST_WINDOW(this)) {
			win = getWindow();
			if (win != NULL) prev->removeFromWindow(win);
		}
		prev->removeFromParent();		
	}
	m_vComponents.setElementAt(comp, idx);
	PGContainerDoAdd(this, comp);
}

void PGContainer::remove(PGComponent* comp) {
	m_vComponents.removeElement(comp);
	if (PGCOMPONENT_TEST_WINDOW(this)) 
		comp->removeFromWindow(getWindow());	
	comp->removeFromParent();
}

PGComponent* PGContainer::getComponentAt(int idx) {
	return (PGComponent*)m_vComponents.getElementAt(idx);
}

int PGContainer::getComponentCount() {
	return (int)m_vComponents.getSize();
}

void PGContainer::addToWindow(PGWindow* parent) {
	PGComponent::addToWindow(parent);
	for (int idx = 0; idx < getComponentCount(); idx++) {
		PGComponent* comp = getComponentAt(idx);
		if (comp != NULL) comp->addToWindow(parent);
	}
}

void PGContainer::removeFromWindow(PGWindow* parent) {
	PGComponent::removeFromWindow(parent);
	for (int idx = 0; idx < getComponentCount(); idx++) {
		PGComponent* comp = getComponentAt(idx);
		if (comp != NULL) comp->removeFromWindow(parent);
	}
}

void PGContainer::setVisible(BOOL visible) {
	for (int idx = 0; idx < getComponentCount(); idx++) {
		PGComponent* comp = getComponentAt(idx);
		if (comp != NULL) comp->setVisible(visible);
	}
}
