/***
 * Portable GUI
 * Code for PGMenu
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

#include <pgcomponent/pgmenu.h>

void PGAbstractMenuItem::setFrame(PGFrame* frame) {
}

PGMenuItem::PGMenuItem(char* title) {
	init(title, NULL);
}

PGMenuItem::PGMenuItem(char* title, char* status) {
	init(title, status);
}

void PGMenuItem::onSelect() {
	PGActionListenerParent::onSelect();	
	if (m_hFrame != NULL && m_sStatus.getLength() != 0) m_hFrame->setStatus(m_sStatus.getValue());
}

void PGMenuItem::setFrame(PGFrame* frame) {
	m_hFrame = frame;
}

void PGMenu::add(PGAbstractMenuItem * item) {
	m_vItems.addElement(item);
	item->addToMenu(this, m_vItems.getSize()-1);
}

void PGMenu::setFrame(PGFrame* frame) {
	m_hFrame = frame;
	for (int ctr = 0; ctr < m_vItems.getSize(); ctr++) {
		PGAbstractMenuItem* item = (PGAbstractMenuItem*)m_vItems.getElementAt(ctr);
		item->setFrame(frame);
	}
}

PGMenuClosingListener::~PGMenuClosingListener() {
}
 
void PGMenuClosingListener::toString(PGString* str) {
}

void PGMenuBar::add(PGMenu* menu) {
	m_vMenus.addElement(menu);
	menu->addToMenuBar(this, m_vMenus.getSize()-1);
}

void PGMenuBar::onMenuClosing() {
	int size = m_vMenuClosingListeners.getSize();
	for (int idx = 0; idx < size; idx++) {
		PGMenuClosingListener* action = (PGMenuClosingListener*)m_vMenuClosingListeners.getElementAt(idx);
		action->menuClosing(this);
	}
}

void PGMenuBar::addMenuClosingListener(PGMenuClosingListener* action) {
	m_vMenuClosingListeners.addElement(action);
}

void PGMenuBar::removeMenuClosingListener(PGMenuClosingListener* action) {
	m_vMenuClosingListeners.removeElement(action);
}

