/***
 * Portable GUI
 * Code for PGActionListener
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <portablegui.h>

// Mouse Event
PGMouseEvent::PGMouseEvent(int x, int y, int mods) {
	m_iX = x;
	m_iY = y;
	m_iModifiers = mods;
}

// Returns the x,y position
void PGMouseEvent::getPoint(PGPoint* p) {
	p->x = m_iX;
	p->y = m_iY;
}

// Returns the horizontal x position
int PGMouseEvent::getX() {
	return m_iX;
}

// Returns the vertical y position
int PGMouseEvent::getY() {
	return m_iY;
}

// Returns the modifiers flag for this event.
int PGMouseEvent::getModifiers() {
	return m_iModifiers;
}

// Invoked when dragged (while holding down button).
void PGMouseMotionListener::mouseDragged(PGMouseEvent* e) {
}

// Invoked when mouse moves
void PGMouseMotionListener::mouseMoved(PGMouseEvent* e) {
}

// Convert to string representation
void PGMouseMotionListener::toString(PGString* str) {
	PGObject::toString(str);
	str->substring(8);
	str->prepend("PGMouseMotionListener ");
}
 
// Invoked when the mouse has been clicked.
void PGMouseListener::mouseClicked(PGMouseEvent* e) {
}

// Invoked when the mouse enters a component.                
void PGMouseListener::mouseEntered(PGMouseEvent* e) {
}

// Invoked when the mouse exits a component.                
void PGMouseListener::mouseExited(PGMouseEvent* e) {
}

// Invoked when a mouse button has been pressed                
void PGMouseListener::mousePressed(PGMouseEvent* e) {
}

// Invoked when a mouse button has been released
void PGMouseListener::mouseReleased(PGMouseEvent* e) {
}

// Convert to string representation
void PGMouseListener::toString(PGString* str) {
	PGObject::toString(str);               
	str->substring(8);                     
	str->prepend("PGMouseListener ");	
}

PGMouseListenerParent::PGMouseListenerParent() {
	m_bInside = FALSE;
	m_iModifiers = 0;
}

PGMouseListenerParent::~PGMouseListenerParent() {
}

void PGMouseListenerParent::mouseMove(int x, int y) {
	if (!m_bInside) {
		m_bInside = TRUE;
		onEvent(x, y, m_iModifiers, PGMOUSELISTENER_ENTER);
	}	
	if ((m_iModifiers & PGMOUSEEVENT_BUTTONS_MASK) != 0)
		onMotion(x, y, m_iModifiers, PGMOUSELISTENER_DRAG);
	else 
		onMotion(x, y, m_iModifiers, PGMOUSELISTENER_MOVE);
}

void PGMouseListenerParent::mouseUp(int x, int y, int button) {
	m_iModifiers &= (PGMOUSEEVENT_BUTTONS_MASK ^ button);
	onEvent(x, y, button, PGMOUSELISTENER_RELEASE);
	onEvent(x, y, button, PGMOUSELISTENER_CLICK);
}

void PGMouseListenerParent::mouseDown(int x, int y, int button) {
	m_iModifiers |= button;
	onEvent(x, y, m_iModifiers, PGMOUSELISTENER_PRESS);
}

void PGMouseListenerParent::mouseExit(int x, int y) {
	m_bInside = FALSE;
	onEvent(x, y, m_iModifiers, PGMOUSELISTENER_EXIT);
}

// Dispatching functions
void PGMouseListenerParent::onMotion(int x, int y, int mods, int typ) {
	PGMouseEvent evt(x, y, mods);
	for (PGMouseMotionListener* mouse = (PGMouseMotionListener*)m_vMotionListeners.getFirst(); mouse != NULL; ) {	
		switch (typ) {
			case PGMOUSELISTENER_DRAG:				
				mouse->mouseDragged(&evt);				
				break;
			case PGMOUSELISTENER_MOVE:
				mouse->mouseMoved(&evt);
				break;
		}
		mouse = (PGMouseMotionListener*)mouse->getNextElement();		
	}
}

void PGMouseListenerParent::onEvent(int x, int y, int mods, int typ) {
	PGMouseEvent evt(x, y, mods);
	for (PGMouseListener* mouse = (PGMouseListener*)m_vMouseListeners.getFirst(); mouse != NULL; ) {
		switch (typ) {
			case PGMOUSELISTENER_ENTER:
				mouse->mouseEntered(&evt);
				break;			
			case PGMOUSELISTENER_EXIT:
				mouse->mouseExited(&evt);
				break;							
			case PGMOUSELISTENER_PRESS:
				mouse->mousePressed(&evt);
				break;							
			case PGMOUSELISTENER_RELEASE:
				mouse->mouseReleased(&evt);
				break;							
			case PGMOUSELISTENER_CLICK:
				mouse->mouseClicked(&evt);
				break;				
		}
		mouse = (PGMouseListener*)mouse->getNextElement();
	}
}

// MouseListener
void PGMouseListenerParent::addMouseListener(PGMouseListener* mouse) {
	m_vMouseListeners.add(mouse);
}

void PGMouseListenerParent::removeMouseListener(PGMouseListener* mouse) {
	//m_vMouseListeners.remove(mouse);
}

// MouseMotionListener
void PGMouseListenerParent::addMouseMotionListener(PGMouseMotionListener* mouse) {
	m_vMotionListeners.add(mouse);
}

void PGMouseListenerParent::removeMouseMotionListener(PGMouseMotionListener* mouse) {
	//m_vMotionListeners.removeElement(mouse);
}


