/***
 * Portable GUI
 * Code for PGTextComponent
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

PGTextComponent::PGTextComponent() {
	setFlag(PGTEXTCOMPONENT_EDITABLE, TRUE);
}

void PGTextComponent::toString(PGString* str) {
	PGObject::toString(str);
	str->substring(8);
	str->prepend("PGTextComponent");
}

// Indicates whether or not this text component is editable
BOOL PGTextComponent::isEditable() {
	return (m_iFlags & PGTEXTCOMPONENT_EDITABLE) != 0;
}

// Sets the flag that determines whether or not this text component is editable
void PGTextComponent::setEditable(BOOL yesNo) {
	setFlag(PGTEXTCOMPONENT_EDITABLE, yesNo);
}

// Selects all the text in this text component
void PGTextComponent::selectAll() {
	if (PGCOMPONENT_TEST_WINDOW(this)) select(0, getSize());
}

// Gets the text
void PGTextComponent::getText(PGString* str) {
	int len = getSize();
	getText(str, 0, len);
}

// Gets the selected text from the text
void PGTextComponent::getSelectedText(PGString* str) {
	int start = 0, end = 0;
	if (PGCOMPONENT_TEST_WINDOW(this)) getSelection(&start, &end);
	getText(str, start, end);
}
	
PGTextField::PGTextField(const char* str) {	
	m_sText.setCopyValue(str);
	m_iCols = strlen(str);
}

void PGTextField::toString(PGString* str) {
	PGObject::toString(str);
	str->substring(8);
	str->prepend("PGTextField");
}

// Sets the number of columns for this text area.
void PGTextField::setColumns(int columns) {
	m_iCols = columns;
}

PGIntegerField::PGIntegerField(int value) : PGTextField("") {
	setValue(value);
}

int PGIntegerField::getValue() {
	PGString str;
	getText(&str);	
	return str.getIntValue();
}
	
void PGIntegerField::setValue(int value) {
	char s[30];
	sprintf(s, "%d", value);
	setText(s);
}

