/***
 * Portable GUI
 * Code for PGBmp
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>

#include <portablegui.h>
#include <pgdraw/pgbmp.h>

PGBmpDecoder::PGBmpDecoder(PGInputStream* input, int rle, int nbits) {
   cond = PG_OKAY;
   rle_method = rle;
   pxl_size = nbits;
   bmp_file = input;
}

int PGBmpDecoder::status(void) {
 //  if( ferror(bmp_file) ) cond = PG_IOERROR;
 //  if( feof(bmp_file) ) cond = PG_OUTOFSYNC;
   return cond;
}

void PGBmpDecoder::read_none(unsigned char *buf, int npxls) {
   int i, j, nbytes;
   switch(pxl_size) {
      case 1 :
         nbytes = (npxls + 7) / 8;
         nbytes = (nbytes + 3) / 4;
         nbytes *= 4;
         while(npxls > 0) {
            nbytes--;
            int x = bmp_file->read();
            int m = 0x80;
            while((npxls > 0) && (m != 0)) {
               *buf++ = (x & m) ? 1 : 0;
               m >>= 1;
               npxls--;
            }
         }
         while(nbytes-- > 0) bmp_file->read();
         break;

      case 4 :
         nbytes = (npxls + 1) / 2;
         nbytes = (nbytes + 3) / 4;
         nbytes *= 4;
         while(npxls > 0) {
            nbytes--;
            int x = bmp_file->read();
            int p2 = x & 0x0F;
            int p1 = (x>>4) & 0x0F;
            *buf++ = p1;
            npxls--;
            *buf++ = p2;
            npxls--;
         }
         while(nbytes-- > 0) bmp_file->read();
         break;

      case 8 :
         nbytes = (npxls + 3) / 4;
         nbytes *= 4;
         bmp_file->read(buf, npxls);
         nbytes -= npxls;
         while(nbytes-- > 0) bmp_file->read();
         break;

      case 24 :
         nbytes = (npxls*3 + 3) / 4;
         nbytes *= 4;
         bmp_file->read(buf, npxls*3);
         nbytes -= npxls*3;
         while(nbytes-- > 0) bmp_file->read();
         for(i=0, j=0; i<npxls; i++, j+=3) { // reorder bgr to rgb
            int k = buf[j];
            buf[j] = buf[j+2];
            buf[j+2] = k;
         }
         break;
   }
}

void PGBmpDecoder::read_rle4(unsigned char *buf, int npxls) {
   int b1, b2;
   while(cond == PG_OKAY) {
      b1 = bmp_file->read();
      b2 = bmp_file->read();
      if(b1 == 0) {
         switch(b2) {
            case 0 :  //.... end-of-line
               return;

            case 1 :  //.... end-of-image
               cond = PG_ENDOFIMAGE;
               return;

            case 2 :  //.... delta
               cond = PG_UNSUPPORTED;
               return;

            default : //.... nonrepeating run
               while(b2--) {
                  if(npxls > 0) {
                     *buf++ = bmp_file->read();
                     npxls--;
                  }
               }
               break;
         }
      } else {
         unsigned char p1 = b2      & 0x0F;
         unsigned char p2 = (b2>>4) & 0x0F;
         unsigned char p;
         if(b1 & 1) { p=p1; p1=p2; p2=p; }
         while(b1--) {
            if(npxls > 0) {
               *buf++ = (b1 & 1 ) ? p1 : p2;
               npxls--;
            }
         }
      }
   }
}

void PGBmpDecoder::read_rle8(unsigned char *buf, int npxls) {
   int b1, b2;
   while(cond == PG_OKAY) {
      b1 = bmp_file->read();
      b2 = bmp_file->read();
      if(b1 == 0) {
         switch(b2) {
            case 0 :  //.... end-of-line
               return;

            case 1 :  //.... end-of-image
               cond = PG_ENDOFIMAGE;
               return;

            case 2 :  //.... delta
               cond = PG_UNSUPPORTED;
               return;

            default : //.... nonrepeating run
               while(b2--) {
                  if(npxls > 0) {
                     *buf++ = bmp_file->read();
                     npxls--;
                  }
               }
               break;
         }
      } else {
         while(b1--) {
            if(npxls > 0) {
               *buf++ = b2;
               npxls--;
            }
         }
      }
   }
}

int PGBmpDecoder::decode(unsigned char *buf, int npxls) {
   switch(rle_method) {
      case PG_BI_NONE :
           read_none(buf, npxls);
           break;

      case PG_BI_RLE8 :
           read_rle8(buf, npxls);
           break;

      case PG_BI_RLE4 :
           read_rle4(buf, npxls);
           break;

      default :
           cond = PG_BADPARAM;
           break;
   }
   return status();
}

PGBitmapFileHeader::PGBitmapFileHeader() {
	strcpy(bfSignature,"BM");
	bfSize = 0;
	bfOffBits = 0;
}

BOOL PGBitmapFileHeader::read(PGInputStream* input) {
	input->read((unsigned char*)bfSignature, 2);
	bfSize = input->read32LEInt();
	input->skip(4);
	bfOffBits = input->read32LEInt();
	return input->getError() == PGINPUTSTREAM_ERROR_NONE;
}

BOOL PGBitmapFileHeader::isValid() {
	return strcmp(bfSignature, "BM") == 0;
}

PGBitmapInfoHeader::PGBitmapInfoHeader() {
	biWidth = biHeight = 0;
	biPlanes = 1;
	biBitCount = 0;
	biCompression = biSizeImage = 0;
	biXPelsPerMeter = biYPelsPerMeter = 0;
	biClrUsed = biClrImportant = 0;
}

BOOL PGBitmapInfoHeader::read(PGInputStream* input) {
	biWidth = input->read32LEInt();         // bmap width in pixels
	biHeight = input->read32LEInt();        // bmap height in pixels
	biPlanes = input->read16LEInt();        // num planes - always 1
	biBitCount = input->read16LEInt();      // bits per pixel
	biCompression = input->read32LEInt();   // compression flag
	biSizeImage = input->read32LEInt();     // image size in bytes
	biXPelsPerMeter = input->read32LEInt(); // horz resolution
	biYPelsPerMeter = input->read32LEInt(); // vert resolution
	biClrUsed = input->read32LEInt();       // 0 -> color table size
	biClrImportant = input->read32LEInt();  // important color count
	return input->getError() == PGINPUTSTREAM_ERROR_NONE;
}

PGRGBQuad::PGRGBQuad() {
	rgbBlue = rgbGreen = rgbRed = 0;
	rgbReserved = 0;
}
 
PGRGBQuad::PGRGBQuad(int r, int g, int b) {
	rgbBlue  = b;
	rgbGreen = g;
	rgbRed   = r;
	rgbReserved = 0;
}

BOOL PGRGBQuad::read(PGInputStream* input) {
	rgbBlue  = input->read();
	rgbGreen = input->read();
	rgbRed   = input->read();
	rgbReserved = input->read();
	return input->getError() == PGINPUTSTREAM_ERROR_NONE;
}

int PGLoadBMP(PGInputStream* input, PGImageReader* reader) {	
	PGBitmapFileHeader bmfh;
	PGApplication* app = getPGApplication();
	// Read BMP header and check for signature
	if (!bmfh.read(input)) return PGIMAGE_ERROR_FILE;
	if(!bmfh.isValid()) return PGIMAGE_ERROR_HEADER;
	// Read size of next header (should be 40 for new version)
	long nbytes = input->read32LEInt();
	if (nbytes != 40) return PGIMAGE_ERROR_UNSUPPORT;
	// Read bitmap infor header
	PGBitmapInfoHeader bmih;
	if(!bmih.read(input)) return PGIMAGE_ERROR_FILE;
	int width = (int)bmih.biWidth; 
	int height = (int)bmih.biHeight;
	int depth = bmih.biBitCount;    
	int rle = (int)bmih.biCompression;	
	int pxlsize = PGPIXELSIZE(depth);
	int rowbytes = (width * pxlsize + 3) / 4;
	rowbytes *= 4;
	// Allocate memory for scanline and initialize PGImageReader
	unsigned char *scanline = new unsigned char[rowbytes];
	if (scanline == NULL) return PGIMAGE_ERROR_MEMORY;
	reader->initReader(PGIMGREADER_TRANSP_AUTO, height, width, pxlsize*8, 1);
	reader->setImageSize(height, width);
	// Calculate number of colors and read palette
	int imgcnt = PGCOLORCNT(depth);
	if(imgcnt > 0) {
		// Smaller palette in file?
		int index = -1;
		const PGColor* bkgcolor = reader->getBackground();
		long maxcnt = (bmfh.bfOffBits - input->getPosition())/4;
		imgcnt = MIN2(imgcnt, maxcnt);	
		// Allocate palette
		reader->setPalette(imgcnt);
		// Read from file
		PGRGBQuad q;
		for(int i = 0; i < imgcnt; i++) {
			if(!q.read(input)) return PGIMAGE_ERROR_FILE;
			reader->setColor(i, q.rgbRed, q.rgbGreen, q.rgbBlue);
			if (bkgcolor != null && bkgcolor->equals(q.rgbRed, q.rgbGreen, q.rgbBlue)) {
				bkgcolor = null;
				index = i;				
			}
		}
		reader->closePalette();
		reader->setBackgroundIndex(index);
	}
	// Output some information about bitmap
	char str[100];
	char* compr = "NONE";
	if (rle == PG_BI_RLE8) compr = "RLE8";
	if (rle == PG_BI_RLE4) compr = "RLE4";
	sprintf(str, "Loading bitmap %dx%d depth: %d (%s)", width, height, depth, compr);
	app->println(str);
	// Goto start of bitmap
	input->seek(bmfh.bfOffBits);
	// Decode BMP file
	PGBmpDecoder bmp(input, rle, depth);
	int n = height;	
	for(int i=0; i<height; i++) {
		bmp.decode(scanline, width);
		reader->put(scanline, --n);
	}
	// Free scanline
	delete[] scanline;
	reader->realizeImage();	
	return PGIMAGE_ERROR_NONE;
}

