/***
 * Portable GUI
 * Code for PGColor
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

PGColor::PGColor(PGBYTE red, PGBYTE green, PGBYTE blue) {
	m_iRed = red;
	m_iGreen = green;
	m_iBlue = blue;
}

PGColor::PGColor(const PGColor* color) {
    setCopyValue(color);
}

void PGColor::setColor(PGBYTE red, PGBYTE green, PGBYTE blue) {
	m_iRed = red;
	m_iGreen = green;
	m_iBlue = blue;
}

BOOL PGColor::equals(PGBYTE red, PGBYTE green, PGBYTE blue) const {
	return m_iRed == red &&
	       m_iGreen == green &&
	       m_iBlue == blue;
}

BOOL PGColor::equals(PGColor* other) const {
	return m_iRed == other->m_iRed &&
	       m_iGreen == other->m_iGreen &&
	       m_iBlue == other->m_iBlue;
}

void PGColor::setCopyValue(const PGColor* other) {
	m_iRed = other->m_iRed;
	m_iGreen = other->m_iGreen;
	m_iBlue = other->m_iBlue;
}

unsigned long PGColor::getBGR() {
	unsigned long color = m_iRed;
	color |= ((unsigned long)m_iGreen) << 8;
	color |= ((unsigned long)m_iBlue)  << 16;
	return color;
}

unsigned long  PGColor::getRGB() {
	unsigned long color = m_iBlue;
	color |= ((unsigned long)m_iGreen) << 8;
	color |= ((unsigned long)m_iRed)  << 16;
	return color;
}

void PGColor::darker(PGColor* nc) {
    nc->m_iRed = PGMathRound(m_iRed * 0.7);
    nc->m_iGreen = PGMathRound(m_iGreen * 0.7);
    nc->m_iBlue = PGMathRound(m_iBlue * 0.7);
}

void PGColor::brighter(PGColor* nc) {
    nc->m_iRed = PGMathMin(255, PGMathRound(m_iRed * 1.3));
    nc->m_iGreen = PGMathMin(255, PGMathRound(m_iGreen * 1.3));
    nc->m_iBlue = PGMathMin(255, PGMathRound(m_iBlue * 1.3));
}


BOOL PGColor::read(PGInputStream* input) {
	m_iRed = input->read();
	m_iGreen = input->read();
	m_iBlue = input->read();
	return input->getError() == PGINPUTSTREAM_ERROR_NONE;
}

PGColor & PGColor::operator=(const PGColor &color) {
	setCopyValue(&color);
	return *this;
}

PGColor PGColorBlack(0,0,0);
PGColor PGColorblue(0,0,255);
PGColor PGColorCyan(0,255,255);
PGColor PGColorDarkGray(64,64,64);
PGColor PGColorGray(128,128,128);
PGColor PGColorGreen(0,255,0);
PGColor PGColorLightGray(192,192,192);
PGColor PGColorMagenta(255,0,255);
PGColor PGColorOrange(255,200,0);
PGColor PGColorPink(255,175,175);
PGColor PGColorRed(255,0,0);
PGColor PGColorWhite(255,255,255);
PGColor PGColorYellow(255,255,0);

