/***
 * Portable GUI
 * Code for PGButton
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

PGFont::PGFont(const char *name, int style, int size) {
	m_sName.setCopyValue(name);
	m_iFlags = style;
	m_iSize = size;
}

PGFont::~PGFont() {
}

void PGFont::getStringRepresentation(PGString* fontstr) {
	char size[20];	
	const char *weight = (m_iFlags & PGFONT_BOLD) != 0 ? "bold" : "regular";
	const char italic = (m_iFlags & PGFONT_ITALIC) != 0 ? 'i' : 'r';
	sprintf(size,"%d",m_iSize*20/16);
	int len = 25 + strlen(weight) + 1 + strlen(size);
	fontstr->setSize(len);
	sprintf(fontstr->getBuffer(), "-*-*-%s-%c-*-*-%s-*-*-*-*-*-*-*", weight, italic, size);	
}

void PGFont::toString(PGString* str) {
	PGObject::toString(str);
	str->substring(8);
	str->prepend(m_sName.getValue());
	str->prepend("PGFont ");
}

BOOL PGFont::equals(PGFont* font) {
	return (m_iSize == font->m_iSize) &&
	       (m_iFlags == font->m_iFlags) &&
	       (m_sName.equals(&(font->m_sName)));
}

PGRealizedFont::PGRealizedFont(PGFont* font) {
	m_hFont = font;
	PGINCREF(font);
}

PGRealizedFont::~PGRealizedFont() {
#ifdef PGDEBUG_CLEANUP
	if (m_hFont != NULL) {
		char s[100];
		sprintf(s, "UnRealizing font '%s' [%d]", m_hFont->m_sName.getValue(),m_hFont->m_iSize);
		getPGApplication()->println(s);
	}
#endif	
	// Unrealize and delete the font
	unrealize();
	if (m_hFont != NULL) PGDECREF(m_hFont);
	// Done?
#ifdef PGDEBUG_CLEANUP	
	getPGApplication()->println("OK");		
#endif		
}
