/***
 * Portable GUI
 * Code for PGGif
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>

#include <portablegui.h>
#include <pgdraw/pggif.h>

#ifdef PGGTK
	#include <string.h>
	#include <stdlib.h>
#endif

int gif_delta[]  = { 8, 8, 4, 2 };
int gif_origin[] = { 0, 4, 2, 1 };

PGGIFFileHeader::PGGIFFileHeader() {
	strcpy(gfSignature,"GIF");
	gfMajorVersion = 89;
	gfMinorVersion = 'a';
}

BOOL PGGIFFileHeader::read(PGInputStream* input) {
	unsigned char version[3];
	input->read((unsigned char*)gfSignature, 3);
	input->read(version, 3);
	gfMinorVersion = version[2]; version[2] = 0;
	gfMajorVersion = atoi((const char*)version);	
	return input->getError() == PGINPUTSTREAM_ERROR_NONE;
}

BOOL PGGIFFileHeader::isValid() {
	return strcmp(gfSignature, "GIF") == 0;
}

BOOL PGGIFLogicalScreenDescriptor::read(PGInputStream* input) {
	gfScreenWd = input->read16LEInt();
	gfScreenHi = input->read16LEInt();
	unsigned char packed = input->read();
	gfBackgroundColor = input->read();	
	gfAspectRatio = input->read();
	gfGlobalColorFlag = (packed & 0x80) != 0;
   	gfColorResolution = (packed >> 4) & 0x07;
	gfColorTableSize = packed & 0x07;		
	return input->getError() == PGINPUTSTREAM_ERROR_NONE;	
}

PGGIFImageDescriptor::PGGIFImageDescriptor() {
	gfDisposal = 0;
	gfTranspFlag = FALSE;
	gfDelay = 0;
}

BOOL PGGIFImageDescriptor::read(PGInputStream* input) {
	gfLeftPos = input->read16LEInt();
	gfTopPos = input->read16LEInt();
   	gfWidth = input->read16LEInt();
   	gfHeight = input->read16LEInt();
   	PGBYTE packed = input->read();   	
	gfLocalColorTable = (packed & 0x80) != 0;
	gfInterlaced = (packed & 0x40) != 0;
	gfColorTableSize = packed & 0x07;
	return TRUE;
}

BOOL PGLoadGIFHeader(PGInputStream* input, PGApplication* app) {
	PGGIFFileHeader gfh;	
	// Read GIF header and check for signature
	if (!gfh.read(input)) {
		return FALSE;
	}
	if(!gfh.isValid()) {
	  	return FALSE;
	}
	return TRUE;	
}

BOOL PGGIFLoadApplicationExtention(PGInputStream* input, PGImageReader* reader) {
	PGBYTE gfAppIdentifier[9];
	PGBYTE gfAppAuthentication[4];
	input->read(); // Block Size 11	
	gfAppIdentifier[8] = 0;
	gfAppAuthentication[3] = 0;
	input->read(gfAppIdentifier, 8);
	input->read(gfAppAuthentication, 3);		
	PGBYTE size = input->read();
	// Support Netscape looping extention	
// FIXME : stricmp
	if (strcmp((const char*)gfAppIdentifier, "NETSCAPE") == 0 && size == 3) {		
		input->read(); // Always 1
		reader->setLoops(input->read16LEInt());
		size = input->read(); 	
	}	
	// Skip all application data blocks
	while (size != 0) {
		input->skip(size);
		size = input->read();
	}
	return TRUE;
}

BOOL PGGIFLoadGraphicControlExtention(PGInputStream* input, PGGIFImageDescriptor* descr) {
	input->read(); // Block Size 4
	PGBYTE packed = input->read();
	descr->gfDelay = input->read16LEInt();
	descr->gfTranspIndex = input->read();
	descr->gfTranspFlag = (packed & 0x01) != 0;
	descr->gfDisposal = (packed >> 2) & 0x07;		
	input->read(); // Block terminator
	return TRUE;
}

BOOL PGGIFLoadCommentExtention(PGInputStream* input) {
	PGBYTE comment[256];
	PGBYTE size = input->read();
	while (size != 0) {
		input->read(comment, size);
		comment[size] = 0;
		getPGApplication()->println((char*)comment);
		size = input->read();
	}
	return TRUE;
}

PGGIFDecoder::PGGIFDecoder(int wd) {
	first = new PGDWORD[LZW_MAXVAL];
	last = new PGBYTE[LZW_MAXVAL];
	gifBuff = new PGBYTE[LZW_MAXVAL];
	scanLine = new PGBYTE[wd];
} 

PGGIFDecoder::~PGGIFDecoder() {
	delete first;
	delete last;
	delete gifBuff;
	delete scanLine;
} 

void PGGIFDecoder::ClearTable() {
	int maxi = 1 << rootCodeSize;
	expected = maxi + 2;
	old = LZW_MAXVAL;
	codeSize = rootCodeSize + 1;
	mask = (1 << codeSize) - 1;
	for (int i = 0; i < maxi; i++) {
		first[i] = LZW_MAXVAL;
		last[i] = i & 0xFF;
	}
	topGifBuff = gifBuff;
}

int PGGIFDecoder::Deinterlace(int nrows) {
	crLine += gif_delta[crPass];
	if (crLine >= nrows) {
		crPass++;
		crLine = gif_origin[crPass];
	}
	return crLine;
}

void PGGIFDecoder::StoreBytes(PGImageReader* reader, PGGIFImageDescriptor* descr, int bytes, PGBYTE* source) {
	int wd = (int)descr->gfWidth;
	int i = bytes - 1;
	while (i >= 0) {	
		int j = i-wd+crPos+1;			
		if (j < 0) j = 0;
		while (i >= j) scanLine[crPos++] = source[i--];		
		if (crPos >= wd) {
			crPos = 0;			
			if (descr->gfInterlaced) reader->put(scanLine, Deinterlace(descr->gfHeight));
			else reader->put(scanLine, crLine++);
		}
	}
}

int PGGIFDecoder::PGGIFLoadImageData(PGInputStream* input, PGGIFImageDescriptor* descr, PGImageReader* reader) {
	int need = 0;	
	PGDWORD bits = 0, code = 0, codeFeched;
	PGBYTE *chPos, firstCodeOut = 0, charBuff[256];	
	// Read data block header
	rootCodeSize = input->read();
  	PGDWORD CLEAR = 1 << rootCodeSize;
	PGDWORD EOI = CLEAR + 1;	
  	ClearTable();
	// Read image data
	crPass = crPos = 0;	
	crLine = descr->gfInterlaced ? -8 : 0;	
	PGDWORD buffCount = input->read();
	if (buffCount == 0) return PGIMAGE_ERROR_DATA;	
	while(buffCount > 0) {
    		if ((PGDWORD)input->read(charBuff, buffCount) != buffCount) return PGIMAGE_ERROR_FILE;
		for(chPos = charBuff; buffCount-- > 0; chPos++) {
      			need += (int) *chPos << bits;
			bits += 8;
			while (bits >= codeSize) {
				code = need & mask;
				need >>= codeSize;
				bits -= codeSize;
				if (code > expected) return PGIMAGE_ERROR_DATA; // Neither LZW nor RunLength
				if (code == EOI) return PGIMAGE_ERROR_NONE;
				if (code == CLEAR) {
					ClearTable();
					continue;
				}
				if (old == LZW_MAXVAL) {  // First code after clear table
		  			StoreBytes(reader, descr, 1, &last[code]);
					firstCodeOut = last[code];
					old = code;
					continue;
				}
				codeFeched = code;
				if (code == expected) {
					*topGifBuff++ = firstCodeOut;
					code = old;
				}
				while (code > CLEAR) {
					*topGifBuff++ = last[code];
					code = first[code];
				}
				*topGifBuff++ = firstCodeOut = last[code];
				first[expected] = old;
				last[expected] = firstCodeOut;
				if (expected < LZW_MAXVAL) expected++;
				if (((expected & mask) == 0) && (expected < LZW_MAXVAL)) {
					codeSize++;
					mask += expected;
				}
				old = codeFeched;
				StoreBytes(reader, descr, topGifBuff - gifBuff, gifBuff);
				topGifBuff = gifBuff;
			}
		}
		buffCount = input->read();
		if (buffCount == 0) return PGIMAGE_ERROR_DATA;
	}
	return PGIMAGE_ERROR_NONE;
}

int PGLoadGIF(PGInputStream* input, PGImageReader* reader) {
	PGColor color;	
	int ncolors = 0;
	int iReturnCode = PGIMAGE_ERROR_NONE;
	// Load GIF file header
	PGApplication* app = getPGApplication();
	if (!PGLoadGIFHeader(input, app)) return PGIMAGE_ERROR_HEADER;
	// Load logical screen descriptor
	PGGIFLogicalScreenDescriptor logdescr;
	if (!logdescr.read(input)) return PGIMAGE_ERROR_FILE;
	reader->initReader(PGIMGREADER_TRANSP_ENABLE, logdescr.gfScreenHi, logdescr.gfScreenWd, 8, 2);
	// Load in optional global color table
	if (logdescr.gfGlobalColorFlag) {
		ncolors = 1 << (logdescr.gfColorTableSize+1);
		reader->setPalette(ncolors);
		for (int i = 0; i < ncolors; i++) {
			color.read(input);
			reader->setColor(i, color.m_iRed, color.m_iGreen, color.m_iBlue);
		}
		reader->closePalette();		
	}
	BOOL done = false;
	PGGIFDecoder decoder(logdescr.gfScreenWd);	
	PGGIFImageDescriptor crimage;
	while (!done) {
		PGBYTE exttype = 0;
		if (input->getError() == PGINPUTSTREAM_ERROR_END) break;
		PGBYTE blocktype = input->read();
		switch (blocktype) {
			case 0x21: // Extention block
				exttype = input->read(); // Extention Label
				switch (exttype) { 
					case 0xFF: // Application Extention
						PGGIFLoadApplicationExtention(input, reader);
						break;
					case 0xF9: // Graphic Control Extention
						PGGIFLoadGraphicControlExtention(input, &crimage);
						break;
					case 0xFE: // Comment Extenstion
						PGGIFLoadCommentExtention(input);
						break;
				}												
				break;
			case 0x2C: // Image descriptor
				crimage.read(input);
				// Create new image and add to sequence				
				reader->nextImage(0, crimage.gfLeftPos, crimage.gfTopPos, crimage.gfDelay, crimage.gfDisposal);
				// Create reader for image
				reader->setBackgroundIndex(crimage.gfTranspFlag ? crimage.gfTranspIndex : -1);
				reader->setImageSize(crimage.gfHeight, crimage.gfWidth);				
				// Read local palette
				if (crimage.gfLocalColorTable) {					
					ncolors = 1 << (crimage.gfColorTableSize+1);
					reader->selectPalette(1);
					reader->setPalette(ncolors);
					for (int i = 0; i < ncolors; i++) {
						color.read(input);
						reader->setColor(i, color.m_iRed, color.m_iGreen, color.m_iBlue);
					}
					reader->closePalette();					
				} else {
					reader->selectPalette(0);
				}				
				// Load image
				iReturnCode = decoder.PGGIFLoadImageData(input, &crimage, reader);
				reader->realizeImage();
				if (reader->isAnimated()) {
					crimage.gfDelay = 0;
					crimage.gfTranspFlag = false;
					crimage.gfDisposal = 0;
				} else {
					done = true;
				}
				break;
			case 0x00:
				break;
			case 0x3B:
			default:
				done = TRUE;
		}			
	}
	return iReturnCode;
}
