/***
 * Portable GUI
 * Code for PGButton
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

PGGraphics::PGGraphics() {
	m_iLineWidth = 1;
	init();
}

PGGraphics::~PGGraphics() {
	unrealize();
}

void PGGraphics::getFontMetrics(PGFontMetrics* fm) {
	fm->m_hGraphics = this;
	fm->realize();
}

// Gets the font metrics for the specified font. 
void PGGraphics::getFontMetrics(PGNormalComponent* comp, PGFont* f, PGFontMetrics* fm) {
}

// Sets this graphics context's font
void PGGraphics::setFont(PGNormalComponent* comp, PGFont* font) {
	PGWindow* win = comp->getWindow();
	if (win != NULL) {
		PGRealizedFont* rfont = win->getRealizedFont(font);
		setFont(rfont);
	}
}

// Draws a given image.
void PGGraphics::drawImage(PGImage* image, int x, int y) {
	drawImage(image, x, y, 0, 0, image->m_iCols, image->m_iRows);
}

// Draws a part of a given image.           
void PGGraphics::drawImage(PGImage* image, int x, int y, int width, int height) {
	drawImage(image, x, y, 0, 0, width, height);
}

// Draws a 3-D highlighted outline of the specified rectangle.
void PGGraphics::draw3DRect(int x, int y, int width, int height, BOOL raised) {
    PGColor cc(getColor());
    PGColor upperleft;
    PGColor downright;
    if (raised) {
        cc.brighter(&upperleft);
        cc.darker(&downright);
    } else {
        cc.darker(&upperleft);
        cc.brighter(&downright);
    }
    setColor(&upperleft);
    drawLine(x, y, x+width, y);
    drawLine(x, y+1, x+width-1, y+1);
    drawLine(x, y, x, y+height);
    drawLine(x+1, y, x+1, y+height-1);
    setColor(&downright);
    drawLine(x, y+height, x+width, y+height);
    drawLine(x+1, y+height-1, x+width, y+height-1);
    drawLine(x+width, y, x+width, y+height);
    drawLine(x+width-1, y+1, x+width-1, y+height);
    setColor(&cc);
}

