/***
 * Portable GUI
 * Code for PGInputStream
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <pgio/pgfileinputstream.h>

PGFileInputStream::PGFileInputStream(const char* fname) {
	m_sFile.setCopyValue(fname);
	m_sFile.replace('/', PGSTRING_FILE_SEP);
	m_hFile = fopen(m_sFile.getValue(), "rb");
	m_iError = m_hFile != NULL ? PGINPUTSTREAM_ERROR_NONE : PGINPUTSTREAM_ERROR_OPEN;
}

PGFileInputStream::~PGFileInputStream() {
	close();
}

int PGFileInputStream::getError() {
	if (m_iError != PGINPUTSTREAM_ERROR_NONE) return m_iError;
	if (feof(m_hFile)) return PGINPUTSTREAM_ERROR_END;
	return PGINPUTSTREAM_ERROR_NONE;
}

// Reads the next byte of data from the input stream
int PGFileInputStream::read() {
	if (m_cPushBack != 0) {		
		return getPushback();
	} else {
		if (feof(m_hFile)) {
			m_iError = PGINPUTSTREAM_ERROR_END;
			return -1;
		}
		m_lPosition++;
		return fgetc(m_hFile);
	}
}

// Reads up to len bytes of data from the input stream into an array of bytes
int PGFileInputStream::read(unsigned char* buffer, int off, int len) {
	size_t result = 0;
	if (m_cPushBack != 0) {
		buffer[off] = getPushback();
		len--; off++; result++;
	}
	if (len > 0) {
		result += fread(buffer+off, 1, len, m_hFile);
		m_lPosition += result;
	}
	return result;
}

// Close the file
void PGFileInputStream::close() {
	if (m_hFile != NULL) fclose(m_hFile);
	m_hFile = NULL;
}
