/***
 * Portable GUI
 * Code for PGInputStream
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <pgio/pgfileoutputstream.h>

PGFileOutputStream::PGFileOutputStream(const char* fname) {
	m_sFile.setCopyValue(fname);
	m_sFile.replace('/', PGSTRING_FILE_SEP);
	m_hFile = fopen(m_sFile.getValue(), "wb");
	m_iError = m_hFile != NULL ? PGOUTPUTSTREAM_ERROR_NONE : PGOUTPUTSTREAM_ERROR_OPEN;
}

PGFileOutputStream::~PGFileOutputStream() {
	close();
}

int PGFileOutputStream::getError() {	
	return m_iError;
}

// Write the next byte of data to the output stream
int PGFileOutputStream::write(int byte) {
	m_lPosition++;
	int res = fputc(byte, m_hFile);
	return res;
}

// Write len bytes of data to the output stream starting from off in the buffer
int PGFileOutputStream::write(const unsigned char* buffer, int off, int len) {
	size_t result = fwrite(buffer+off, 1, len, m_hFile);
	m_lPosition += result;
	return result;
}

// Close the file
void PGFileOutputStream::close() {
	if (m_hFile != NULL) fclose(m_hFile);
	m_hFile = NULL;
}
