/***
 * Portable GUI
 * Code for PGINIFile
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <pgio/pginifile.h>
 
PGINIFileValue::PGINIFileValue(const char* key, const char* value) {
	m_sKey.setCopyValue(key);
	m_sValue.setCopyValue(value);
}

void PGINIFileValue::setValue(const char* value) {
	m_sValue.setCopyValue(value);
}

PGString* PGINIFileValue::getValue() {
	return &m_sValue;
}

BOOL PGINIFileValue::equals(const char* key) {
	return m_sKey.equals(key);
}

void PGINIFileSection::read(PGInputStream* stream) {
	PGString key, value;	
	while (TRUE) {
		stream->skipSeparators();
		int ch = stream->read();
		if (ch == '[') {
			stream->pushback(ch);
			return;	
		}
		key.reset(100);
		while (TRUE) {
			if (ch == -1) return;
			if (PGIsSeparator(ch) || ch == '=') break;
			else key.append(ch);
			ch = stream->read();
		}
		if (ch != '=') {
			stream->skipSeparators();
			ch = stream->read();
			if (ch != '=') return;
			stream->skipSeparators();
		}
		value.reset(100);
		while (TRUE) {
			ch = stream->read();
			if (ch == -1) return;
			if (PGIsSeparator(ch)) break;
			else value.append(ch);			
		}		
		PGINIFileValue* nval = new PGINIFileValue(key.getValue(), value.getValue());
		m_vValues.addElement(nval);	
	}		
}

PGINIFileValue* PGINIFileSection::getValue(const char* key) {
	for (int ctr = 0; ctr < m_vValues.getSize(); ctr++) {
		PGINIFileValue* value = (PGINIFileValue*)m_vValues.getElementAt(ctr);	
		if (value->equals(key)) return value;
	}
	return NULL;
}

PGINIFileSection::PGINIFileSection(const char* heading) {
	m_sHeading.setCopyValue(heading);
}

PGString* PGINIFileSection::getStringValue(const char* key) {
	PGINIFileValue* value = getValue(key);
	if (value != NULL) return value->getValue();
	else return NULL;
}

BOOL PGINIFileSection::equals(const char* heading) {
	return m_sHeading.equals(heading);
}

PGINIFileSectionReader::PGINIFileSectionReader(const char* heading) {
	m_sHeading.setCopyValue(heading);
}

BOOL PGINIFileSectionReader::equals(const char* heading) {
	return m_sHeading.equals(heading);
}

void PGINIFile::read(PGInputStream* stream) {	
	PGString name;	
	while (TRUE) {
		stream->skipSeparators();
		int ch = stream->read();
		if (ch != '[') return;
		name.reset(100);
		while (TRUE) {
			int ch = stream->read();
			if (ch == -1 || ch == PGCR || ch == PGLF) return;
			if (ch != ']') name.append(ch);
			else break;			
		}
		PGINIFileSection* section = new PGINIFileSection(name.getValue());
		m_vSections.addElement(section);
		section->read(stream);
	}
}

PGINIFileSection* PGINIFile::getSection(const char* name) {
	for (int ctr = 0; ctr < m_vSections.getSize(); ctr++) {
		PGINIFileSection* section = (PGINIFileSection*)m_vSections.getElementAt(ctr);	
		if (section->equals(name)) return section;
	}
	return NULL;
}

void PGINIFile::addSectionReader(PGINIFileSectionReader* reader) {
}

PGINIFileWriter::PGINIFileWriter(PGOutputStream* stream) {
	m_hStream = stream;
}

void PGINIFileWriter::startSection(const char* heading) {
	m_hStream->write((int)'[');
	m_hStream->write((unsigned char*)heading, strlen(heading));
	m_hStream->write((int)']');
	m_hStream->write((unsigned char*)PGSTRING_NEWLINE, PGSTRING_NB_NEWLINE);
}

void PGINIFileWriter::writeValue(const char* key, const char* value) {
	m_hStream->write((unsigned char*)key, strlen(key));
	m_hStream->write((unsigned char*)" = ", 3);
	m_hStream->write((unsigned char*)value, strlen(value));
	m_hStream->write((unsigned char*)PGSTRING_NEWLINE, PGSTRING_NB_NEWLINE);
}

void PGINIFileWriter::writeBoolValue(const char* key, BOOL value) {
	if (value) writeValue(key, "On");
	else writeValue(key, "Off");
}

void PGINIFileWriter::writeIntValue(const char* key, int value) {
	char s[30];
	sprintf(s, "%d", value);
	writeValue(key, s);
}



