/***
 * Portable GUI
 * Code for PGInputStream
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <pgio/pginputstream.h>

PGInputStream::PGInputStream() {
	m_lPosition = 0;
	m_cPushBack = 0;
}

// Reads up to len bytes of data from the input stream into an array of bytes
int PGInputStream::read(unsigned char* buffer, int off, int len) {
	for (int ctr = 0; ctr < len; ctr++) {
		int ch = read();
		if (ch == -1) return -1;
		buffer[off++] = (unsigned char)ch;
	}
	return len;
}

// Reads some number of bytes from the input stream and stores them into the buffer                  .
int PGInputStream::read(unsigned char* buffer, int len) {
	return read(buffer, 0, len);
}

// Skips over and discards n bytes of data from this input stream
long PGInputStream::skip(long len) {
	for (long ctr = 0; ctr < len; ctr++) {
		int ch = read();
		if (ch == -1) return -1;
	}
	return len;
}

// Skips over chars like space, tab, CR and LF
void PGInputStream::skipSeparators() {
	int ch = ' ';
	while (PGIsSeparator(ch)) ch = read();
	pushback(ch);
}

// Pushback a character (only one allowed)
void PGInputStream::pushback(int ch) {
	if (ch != -1) {
		m_cPushBack = ch;
		m_lPosition--;
	}
}

// Return and reset pushback
int PGInputStream::getPushback() {
	int ch = m_cPushBack;
	m_lPosition++;
	m_cPushBack = 0;
	return ch;	
}

// Reads in one line
void PGInputStream::readLine(PGString* str) {
	str->reset(200);
	while (TRUE) {
		int ch = read();
		if (ch == -1 || ch == PGLF) {
			break;
		} else {
			if (ch != PGCR) str->append((char)ch);
		}
	}	
}

// Skips till given position (works only forward)
long PGInputStream::seek(long pos) {
	skip(pos-m_lPosition);
	return m_lPosition;
}

// Returns current stream position
long PGInputStream::getPosition() {
	return m_lPosition;
}

// Read 16 bits LE integer
int PGInputStream::read16LEInt() {
	unsigned int ch1 = read();
	unsigned int ch2 = read();
	return ch1 | ch2<<8;
}

// Read 32 bits LE integer
long PGInputStream::read32LEInt() {
	unsigned long ch1 = read();
	unsigned long ch2 = read();
	unsigned long ch3 = read();
	unsigned long ch4 = read();
	return ch1 | ch2<<8 | ch3<<16 | ch4<<24;
}

// Test if at end
BOOL PGInputStream::isAtEnd() {
	return getError() == PGINPUTSTREAM_ERROR_END;
}

BOOL PGIsSeparator(int ch) {
	return ch == ' ' || ch == '\t' || ch == PGCR || ch == PGLF;
}
