/***
 * Portable GUI
 * Code for PGInputStream
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <pgmain/pgmain.h> 
#include <pgutil/pgobject.h> 
#include <pgio/pgoutputstream.h>

PGOutputStream::PGOutputStream() {
	m_lPosition = 0;	
}

// Write len bytes of data to the output stream starting from off in the buffer
int PGOutputStream::write(const unsigned char* buffer, int off, int len) {
        for (int ctr = 0; ctr < len; ctr++) {
        	int res = write(buffer[off++]);
		if (res == -1) return -1;		
	}
	return len;
}

// Write len bytes of data to the output stream
int PGOutputStream::write(const unsigned char* buffer, int len) {
	return write(buffer, 0, len);
}

// Writes one line of text
void PGOutputStream::println(const char* str) {
	write((const unsigned char*)str, strlen(str));
	write((const unsigned char*)PGSTRING_NEWLINE, PGSTRING_NB_NEWLINE);
}

// Returns current stream position
long PGOutputStream::getPosition() {
	return m_lPosition;
}

// Write 16 bits LE integer
void PGOutputStream::write16LEInt(int data) {
	write(data & 0xFF);
	write((data >> 8) & 0xFF);
}

// Write 32 bits LE integer
void PGOutputStream::write32LEInt(long data) {
	write(data & 0xFF);
	write((data >> 8) & 0xFF);
	write((data >> 16) & 0xFF);	
	write((data >> 24) & 0xFF);
}
