/***
 * Portable GUI
 * Code for PGBorderLayout
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <portablegui.h>

PGBorderLayout::PGBorderLayout() {
	m_iBorderX = 0;
	m_iBorderY = 0;
	m_iSeparatorX = 0;
	m_iSeparatorY = 0;
}

PGBorderLayout::PGBorderLayout(int borderX, int borderY, int separatorX, int separatorY) {
	m_iBorderX = borderX;
	m_iBorderY = borderY;
	m_iSeparatorX = separatorX;
	m_iSeparatorY = separatorY;
}

PGBorderLayout::PGBorderLayout(int separatorX, int separatorY) {
	m_iBorderX = 0;
	m_iBorderY = 0;
	m_iSeparatorX = separatorX;
	m_iSeparatorY = separatorY;	
}

void PGBorderLayout::getPreferredSize(PGContainer* container, PGDimension* dimension, BOOL recalc) {
	PGDimension dimNorth, dimSouth, dimEast, dimWest, dimCenter;
	PGComponent* north = container->getComponentAt(PGBORDERLAYOUT_NORTH);
	PGComponent* south = container->getComponentAt(PGBORDERLAYOUT_SOUTH);
	PGComponent* west = container->getComponentAt(PGBORDERLAYOUT_WEST);
	PGComponent* east = container->getComponentAt(PGBORDERLAYOUT_EAST);
	PGComponent* center = container->getComponentAt(PGBORDERLAYOUT_CENTER);
	getComponentSize(north, &dimNorth, recalc);
	getComponentSize(south, &dimSouth, recalc);
	getComponentSize(east, &dimEast, recalc);
	getComponentSize(west, &dimWest, recalc);
	getComponentSize(center, &dimCenter, recalc);
	int nbX = 0, nbY = 0;
	if (east != NULL) nbX++;
	if (west != NULL) nbX++;
	if (north != NULL) nbY++;
	if (south != NULL) nbY++;
	int w = MAX2(dimNorth.width, dimSouth.width);
	int h = MAX2(dimEast.height, dimWest.height);
	w = MAX2(w, dimEast.width + dimWest.width + dimCenter.width + nbX*m_iSeparatorX);
	h = MAX2(h, dimCenter.height);
	dimension->width = w + 2*m_iBorderX;
	dimension->height = h + dimNorth.height + dimSouth.height + nbY*m_iSeparatorY+2*m_iBorderY;
}

void PGBorderLayout::layoutContainer(PGContainer* container, PGRectangle* rectangle) {
	PGDimension dimNorth, dimSouth, dimEast, dimWest;
	PGComponent* north = container->getComponentAt(PGBORDERLAYOUT_NORTH);
	PGComponent* south = container->getComponentAt(PGBORDERLAYOUT_SOUTH);
	PGComponent* west = container->getComponentAt(PGBORDERLAYOUT_WEST);
	PGComponent* east = container->getComponentAt(PGBORDERLAYOUT_EAST);
	PGComponent* center = container->getComponentAt(PGBORDERLAYOUT_CENTER);
	getComponentSize(north, &dimNorth, FALSE);
	getComponentSize(south, &dimSouth, FALSE);
	getComponentSize(east, &dimEast, FALSE);
	getComponentSize(west, &dimWest, FALSE);
	int y1pos = rectangle->y+m_iBorderY;
	int y2pos = rectangle->y+rectangle->height-m_iBorderY;
	int x1pos = rectangle->x+m_iBorderX;
	int x2pos = rectangle->x+rectangle->width-m_iBorderX;
	int width = rectangle->width-2*m_iBorderX;
	int height = dimNorth.height;
	if (south != NULL) y2pos -= dimSouth.height+m_iSeparatorY;
	if (north != NULL && width > 0 && height > 0) {
		y1pos += dimNorth.height+m_iSeparatorY;
		north->setBounds(rectangle->x+m_iBorderX, rectangle->y+m_iBorderX, width, height);
	}
	height = dimSouth.height;
	if (south != NULL && width > 0 && height > 0) {
		south->setBounds(rectangle->x+m_iBorderX, y2pos+m_iSeparatorY, width, height);
	}
	width = dimWest.width;
	height = y2pos-y1pos;
	if (west != NULL && width > 0 && height > 0) {
		x1pos += dimWest.width+m_iSeparatorX;
		west->setBounds(rectangle->x+m_iBorderX, y1pos, width, height);
	}
	width = dimEast.width;
	if (east != NULL && width > 0 && height > 0) {
		x2pos -= dimEast.width+m_iSeparatorX;
		east->setBounds(x2pos+m_iSeparatorX, y1pos, width, height);
	}
	width = x2pos-x1pos;
	if (center != NULL && width > 0 && height > 0) {
		center->setBounds(x1pos, y1pos, width, height);
	}
}

void PGBorderLayout::getComponentSize(PGComponent* comp, PGDimension* dim, BOOL recalc) {
	if (comp != NULL) {
		comp->getPreferredSize(dim, recalc);
	} else {
		dim->width = 0;
		dim->height = 0;
	}
}

