/***
 * Portable GUI
 * Code for PGPercentLayout
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>
#include <stdlib.h>

int PGCountTokens(char* str) {
	int nb = 0, ps = 0;
	if (str[0] != 0) {
		while (str[ps+1] != 0) {
			if (str[ps] == ' ' && str[ps+1] != ' ') nb++;
			ps++;
		}
	}
	return nb+1;
}

PGPercentLayout::PGPercentLayout(char* descr, int gap, int sides, BOOL vert) {
	// Init member vars
	m_bSizeUnknown = TRUE;
	m_iBorderSize = gap;
	m_iSides = sides;
	m_bVert = vert;
	m_iDummies = 0;
	// Allocate size structs
	m_iNum = PGCountTokens(descr);
	m_aModes = new int[m_iNum];
	m_aSizes = new int[m_iNum];
	// Parse string
	int pos = 0;
	for (int ctr = 0; ctr < m_iNum; ctr++) {
		m_aModes[ctr] = 0;
		m_aSizes[ctr] = 0;
		// Find start of token
		while (descr[pos] == ' ' && descr[pos] != 0) pos++;
		// Find end of token
		int etok = pos;
		while (descr[etok] != ' ' && descr[etok] != 0) etok++;
		int newtok = etok;
		etok--;
		// Find dummy, percent or m_iPreferred
		if (etok >= pos && descr[etok] == 'l') {
			m_aModes[ctr] |= PGPERCENTLAYOUT_LEFT;
			etok--;
		}
		if (etok >= pos && descr[etok] == 'r') {
			m_aModes[ctr] |= PGPERCENTLAYOUT_RIGHT;
			etok--;
		}
		if (etok >= pos && descr[etok] == 'c') {
			m_aModes[ctr] |= PGPERCENTLAYOUT_CENTER;
			etok--;
		}
		if (etok >= pos && descr[etok] == 'd') {
			m_iDummies++;
			m_aModes[ctr] |= PGPERCENTLAYOUT_DUMMY;
			etok--;
		}
		if (etok >= pos && descr[etok] == '%') {
			m_aModes[ctr] |= PGPERCENTLAYOUT_PERCENT;
			etok--;
		}
		if (etok >= pos) {
			if (descr[etok] == 'p') {
				m_aModes[ctr] |= PGPERCENTLAYOUT_PREF;
			} else {
				int res = 0;
				for (int ps = pos; ps <= etok; ps++) {
					res *= 10;
					res += (int)(descr[ps] - '0');
				}
				m_aSizes[ctr] = res;
			}
		}
		if ((m_aModes[ctr] & (PGPERCENTLAYOUT_PERCENT | PGPERCENTLAYOUT_PREF)) == 0)
			m_aModes[ctr] |= PGPERCENTLAYOUT_ABSOLUTE;
		// Next token
		pos = newtok;
	}
}

void PGPercentLayout::setSizes(PGContainer* parent, BOOL recalc) {
        int nComps = parent->getComponentCount();
        //Reset m_iPreferred/minimum width and height.
	int dim_one = 0, my_one;
	int dim_mult = 0, my_mult;
	int idx = 0;
	// Do for each component
	for (int i = 0; idx < nComps && i < m_iNum; i++) {
		if ((m_aModes[i] & PGPERCENTLAYOUT_DUMMY) != 0) {
			if ((m_aModes[i] & PGPERCENTLAYOUT_ABSOLUTE) != 0)
				dim_mult += m_aSizes[i];
		} else {
			PGDimension d;
			parent->getComponentAt(idx++)->getPreferredSize(&d, recalc);
			if (m_bVert) {
				my_one = d.width;
				my_mult = d.height;
			} else {
        			my_one = d.height;
        			my_mult = d.width;
			}
			if ((m_aModes[i] & PGPERCENTLAYOUT_ABSOLUTE) != 0)
				dim_mult += m_aSizes[i];
			else
				dim_mult += my_mult;
			dim_one = MAX2(my_one,dim_one);
		}
        }
	// Add bordersize
	dim_mult += (m_iNum - 1)*m_iBorderSize;
	// Remember sizes
	if (m_bVert) {
        	m_iPreferredWidth = dim_one;
        	m_iPreferredHeight = dim_mult;
	} else {
        	m_iPreferredWidth = dim_mult;
        	m_iPreferredHeight = dim_one;
	}
	// Add borders
	if ((m_iSides & PGPERCENTLAYOUT_WEST) != 0) m_iPreferredWidth += m_iBorderSize;
	if ((m_iSides & PGPERCENTLAYOUT_EAST) != 0) m_iPreferredWidth += m_iBorderSize;
	if ((m_iSides & PGPERCENTLAYOUT_NORTH) != 0) m_iPreferredHeight += m_iBorderSize;
	if ((m_iSides & PGPERCENTLAYOUT_SOUTH) != 0) m_iPreferredHeight += m_iBorderSize;
	// Know size now
        m_bSizeUnknown = FALSE;
}

void PGPercentLayout::getPreferredSize(PGContainer* container, PGDimension* dimension, BOOL recalc) {
	// Calculate the preferred size
        setSizes(container, recalc);
        dimension->width = m_iPreferredWidth;
        dimension->height = m_iPreferredHeight;
}

void PGPercentLayout::layoutContainer(PGContainer* parent, PGRectangle* rectangle) {
	// Get sizes of container
        int maxWidth = rectangle->width;
        int maxHeight = rectangle->height;
	// Substract border
	if ((m_iSides & PGPERCENTLAYOUT_WEST) != 0) maxWidth -= m_iBorderSize;
	if ((m_iSides & PGPERCENTLAYOUT_EAST) != 0) maxWidth -= m_iBorderSize;
	if ((m_iSides & PGPERCENTLAYOUT_NORTH) != 0) maxHeight -= m_iBorderSize;
	if ((m_iSides & PGPERCENTLAYOUT_SOUTH) != 0) maxHeight -= m_iBorderSize;
	// Number of components
	int nComps = parent->getComponentCount();
        // Calculate sizes
        if (m_bSizeUnknown) setSizes(parent, FALSE);
	PGDimension d;
	int i, locksize = 0, idx = 0;
        for (i = 0 ; idx < nComps && i < m_iNum; i++) {
		int mode = m_aModes[i];
		if ((mode & PGPERCENTLAYOUT_ABSOLUTE) != 0) locksize += m_aSizes[i];
		if ((mode & PGPERCENTLAYOUT_PREF) != 0) {
			parent->getComponentAt(idx)->getPreferredSize(&d, FALSE);
			if (m_bVert) locksize += d.height;
			else locksize += d.width;
		}
		if ((mode & PGPERCENTLAYOUT_DUMMY) == 0) idx++;
	}
	// Left over?
	int rem_size, mult_ofs = 0, one_ofs = 0;
	if (m_bVert) {
		one_ofs = rectangle->x;
		mult_ofs = rectangle->y*100;
		rem_size = maxHeight - locksize - (m_iNum - 1)*m_iBorderSize;				
		if ((m_iSides & PGPERCENTLAYOUT_NORTH) != 0) mult_ofs += m_iBorderSize*100;
		if ((m_iSides & PGPERCENTLAYOUT_WEST) != 0) one_ofs += m_iBorderSize;
	} else {
		one_ofs = rectangle->y;
		mult_ofs = rectangle->x*100;
		rem_size = maxWidth - locksize - (m_iNum - 1)*m_iBorderSize;				
		if ((m_iSides & PGPERCENTLAYOUT_WEST) != 0) mult_ofs += m_iBorderSize*100;
		if ((m_iSides & PGPERCENTLAYOUT_NORTH) != 0) one_ofs += m_iBorderSize;
	}
	// Allocate
	idx = 0;
        for (i = 0 ; idx < nComps && i < m_iNum; i++) {
		int mysize = 0;
		int mode = m_aModes[i];
		if ((mode & PGPERCENTLAYOUT_ABSOLUTE) != 0) mysize = m_aSizes[i]*100;
		if ((mode & PGPERCENTLAYOUT_PERCENT) != 0) mysize = rem_size*m_aSizes[i];
		if ((mode & PGPERCENTLAYOUT_DUMMY) == 0) {
			PGComponent* c = parent->getComponentAt(idx++);
			if (mode >= PGPERCENTLAYOUT_PREF) {
				c->getPreferredSize(&d, FALSE);
				if ((mode & PGPERCENTLAYOUT_PREF) != 0) {
					if (m_bVert) mysize = d.height*100;
					else mysize = d.width*100;
				}
			}
			if (m_bVert) {
				int xpos = one_ofs;
				int wd = maxWidth;
				if ((mode & PGPERCENTLAYOUT_LEFT) != 0) {
					wd = d.width;
				} else if ((mode & PGPERCENTLAYOUT_RIGHT) != 0) {
					wd = d.width;
					xpos += maxWidth - wd;
				} else if ((mode & PGPERCENTLAYOUT_CENTER) != 0) {
					wd = d.width;
					xpos += maxWidth/2 - wd/2;
				}
				c->setBounds(xpos,(mult_ofs+50)/100,wd,(mysize+50)/100);
			} else {
				int ypos = one_ofs;
				int hi = maxHeight;
				if ((mode & PGPERCENTLAYOUT_LEFT) != 0) {
					hi = d.height;
				} else if ((mode & PGPERCENTLAYOUT_RIGHT) != 0) {
					hi = d.height;
					ypos += maxHeight - hi;
				} else if ((mode & PGPERCENTLAYOUT_CENTER) != 0) {
					hi = d.height;
					ypos += maxHeight/2 - hi/2;
				}
				c->setBounds((mult_ofs+50)/100,ypos,(mysize+50)/100,hi);
			}
		}
		mult_ofs += mysize + m_iBorderSize*100;
        }
}
 
 
 
 
 
 
 
 
 
 
 
 
 
 
