/***
 * Portable GUI
 * Code for PGObject
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <portablegui.h>
#include <stdio.h>

#ifdef PGDEBUG_OBJECT_CREATION
	PGString g_sObjectString;
	FILE* g_hObjectFile = (FILE*)NULL;
	BOOL g_bObjectIsDone = FALSE;
#endif	

void PGCloseDebugObjectCreationFile() {
#ifdef PGDEBUG_OBJECT_CREATION
	if (!g_bObjectIsDone) {
		fclose(g_hObjectFile);
		g_hObjectFile = NULL;
		g_bObjectIsDone = TRUE;
	}
#endif	
}

PGObject::PGObject() {	
	m_iReferenceCount = 0;	
#ifdef PGDEBUG_OBJECT_CREATION
	if (!g_bObjectIsDone) {
		if (g_hObjectFile == NULL) {
			g_hObjectFile = fopen("pg_object_log.txt", "wt");
			if (g_hObjectFile == NULL) g_bObjectIsDone = TRUE;
		}
		if (!g_bObjectIsDone) {
			toString(&g_sObjectString);
			fprintf(g_hObjectFile, "CR:  %s", g_sObjectString.getValue());
		}
	}
#endif	
}

PGObject::PGObject(BOOL doLog) {	
	m_iReferenceCount = 0;	
}

PGObject::~PGObject() {
#ifdef PGDEBUG_OBJECT_CREATION
	if (!g_bObjectIsDone && g_hObjectFile != NULL) {		
		toString(&g_sObjectString);
		fprintf(g_hObjectFile, "DL:  %s", g_sObjectString.getValue());
	}
#endif	
}

void PGObject::toString(PGString* strg) {
	char result[PGSTRLEN];
	sprintf(result, "PGObject @%lu [%d]", (unsigned long)this, m_iReferenceCount);
	strg->setCopyValue(result);
}

void PGObject::incReferenceCount() {
	m_iReferenceCount++;
#ifdef PGDEBUG_OBJECT_CREATION
	if (!g_bObjectIsDone && g_hObjectFile != NULL) {		
		toString(&g_sObjectString);
		fprintf(g_hObjectFile, "INC: %s", g_sObjectString.getValue());
	}
#endif	
}

BOOL PGObject::isDecrReferenceCountZero() {
	m_iReferenceCount--;
#ifdef PGDEBUG_OBJECT_CREATION
	if (!g_bObjectIsDone && g_hObjectFile != NULL) {		
		toString(&g_sObjectString);
		fprintf(g_hObjectFile, "DEC: %s", g_sObjectString.getValue());
	}
#endif		
	return m_iReferenceCount <= 0;
}
