/***
 * Portable GUI
 * Code for PGString
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>

#ifdef PGGTK
	#include <ctype.h>
#endif

#define PGSTRING_GROW 100
#define MAXSTRLEN 100000

PGString::PGString() {
	init();
}

PGString::PGString(const char* str) {
	init();
	setCopyValue(str);
}

PGString::PGString(const PGString &str) {
	init();
	setCopyValue(str.m_pBuffer);
}

PGString::PGString(int value) {
	init();
	format("%d", value);
}

PGString::~PGString() {
	if (m_iSize != 0) delete[] m_pBuffer;
}

PGString PGString::operator+(PGString str) {
	PGString temp = *this;
	temp.append(str.getValue());
	return temp;
}

PGString & PGString::operator=(const PGString &str) {
	setCopyValue(str.m_pBuffer);
	return *this;
}

PGString & PGString::operator+=(const PGString &str) {
	append(str.m_pBuffer);
	return *this;
}

PGString::operator const char*() {
	return m_pBuffer;
}

void PGString::format(const char *format, ...) {
	va_list ap;
	if (m_iSize == 0) setSize(20);
	while (true) {
		va_start(ap, format);
#ifdef PGVISUALC
		int n = _vsnprintf(m_pBuffer, m_iSize, format, ap);
#endif
#ifdef PGGCC
		int n = vsnprintf(m_pBuffer, m_iSize, format, ap);
#endif
		va_end(ap);
		if (n > -1 && n < m_iSize) return;
		if (n > -1) setSize(n+1);
		else setSize(m_iSize*3/2);
	}
}

void PGString::toString(PGString* str) {
	PGObject::toString(str);
	str->substring(8);
	str->prepend("PGString");
}

int PGString::indexOf(const char* str) {
	const char* idx = strstr(m_pBuffer, str);
	if (idx == null) return -1;
	return (int)(idx-str);
}

int PGString::indexOf(int ch) {
	return -1;
}

int PGString::lastIndexOf(int ch) {
	for (int i = m_iSize-1; i >= 0; i--)
		if (m_pBuffer[i] == ch) return i;
	return -1;
}

void PGString::toUpperCase() {
	if (m_iSize > 0) {
		int i = 0;
		while (m_pBuffer[i] != 0 && i < m_iSize) {
			m_pBuffer[i] = toupper(m_pBuffer[i]);
			i++;
		}
	}
}

void PGString::setCopyValue(const char* str) {
	int len = strlen(str);
	if (m_iSize < len) setSize(len);
	if (m_iSize > 0) strcpy(m_pBuffer, str);
}

BOOL PGString::equals(const char* str) const {
	return strcmp(str, m_pBuffer) == 0;
}

BOOL PGString::equals(PGString* str) const {
	return strcmp(str->m_pBuffer, m_pBuffer) == 0;
}

void PGString::copyTo(char* str) const {
	copyRange(0, -1, str);
}

void PGString::copyRange(int start, int end, char* str) const {
	if (end == -1) end = strlen(m_pBuffer)-1;
	int len = end-start+1;
	if (len > 0 && m_iSize != 0) {
		strncpy(str, m_pBuffer+start, len);
		str[len] = 0;
	} else {
		str[0] = 0;
	}
}

int PGString::charAt(int pos) const {
	return (int)m_pBuffer[pos];
}


void PGString::replace(char ch1, char ch2) {
	char val;
	int ctr = 0;
	do {
		if ((val = m_pBuffer[ctr]) == ch1) m_pBuffer[ctr] = ch2;
		ctr++;
	} while (val != 0 && ctr < m_iSize);
}

char* PGString::changeSize(int size) {
	char* shouldDelete = NULL;
	// Not always change size
	if (size > m_iSize || size < m_iSize/2) {
		// Delete old buffer if not size zero
		if (m_iSize != 0) shouldDelete = m_pBuffer;
		// Set new size and allocate if not zero
		m_iSize = size;
		if (m_iSize != 0) {
			m_pBuffer = new char[m_iSize+1];
			if (m_pBuffer == NULL) m_iSize = 0;
		}
		// Size zero or allocation error
		if (m_iSize == 0) m_pBuffer = (char*)"";
		else m_pBuffer[m_iSize] = 0;
	}
	return shouldDelete;
}

void PGString::setSize(int size) {
	char* shouldDelete = changeSize(size);
	if (shouldDelete != NULL) delete[] shouldDelete;
}

void PGString::reset(int size) {
	setSize(size);
	if (m_iSize != 0) m_pBuffer[0] = 0;
}

void PGString::prepend(const char *str) {
	int oldLen = strlen(m_pBuffer);
	int addLen = strlen(str);
	char* oldBuffer = m_pBuffer;
	char* shouldDelete = changeSize(oldLen + addLen);
	if (m_iSize != 0) {
		if (oldLen > 0)
			for (int ctr = 0; ctr < oldLen; ctr++)
				m_pBuffer[ctr+addLen] = oldBuffer[ctr];
		memcpy(m_pBuffer, str, addLen);
		m_pBuffer[oldLen + addLen]  = 0;
	}
	if (shouldDelete != NULL) delete shouldDelete;
}

void PGString::append(const char *str) {
	PGString::append(str, FALSE);
}

void PGString::append(char ch) {
	int oldLen = strlen(m_pBuffer);
	char* shouldDelete = NULL;
	if (oldLen >= m_iSize) {
		shouldDelete = changeSize(m_iSize+PGSTRING_GROW);
		if (shouldDelete != NULL) strncpy(m_pBuffer, shouldDelete, oldLen);
	}
	m_pBuffer[oldLen++] = ch;
	m_pBuffer[oldLen] = 0;
	if (shouldDelete != NULL) delete[] shouldDelete;
}

void PGString::append(const char *str, BOOL newline) {
	int oldLen = strlen(m_pBuffer);
	int newSize = oldLen + strlen(str) + (newline ? PGSTRING_NB_NEWLINE : 0);
	char* shouldDelete = changeSize(newSize);
	if (m_iSize != 0) {
		if (shouldDelete != NULL) strncpy(m_pBuffer, shouldDelete, oldLen);
		strcpy(m_pBuffer+oldLen, str);
		m_pBuffer[newSize] = 0;
		if (newline) strcat(m_pBuffer, PGSTRING_NEWLINE);
	}
	if (shouldDelete != NULL) delete[] shouldDelete;
}

void PGString::growToSize(int size) {
	if (size > m_iSize) {
		char* prevBuffer = m_pBuffer;
		char* shouldDelete = changeSize(size);
		if (m_iSize != 0) strcpy(m_pBuffer, prevBuffer);
		if (shouldDelete != NULL) delete[] shouldDelete;
	}
}

void PGString::insert(const char* str, int pos) {
}

void PGString::remove(int start, int end) {
}

void PGString::substring(int start, int end) {
	int mylen = strlen(m_pBuffer);
	int myend = (end == -1 || end >= mylen) ? mylen-1 : end;
	int nbremain = myend-start+1;
	if (start < mylen) {
		char* shouldDelete = changeSize(nbremain);
		if (mylen != 0) {
			char *source = shouldDelete == null ? m_pBuffer : shouldDelete;
			if (start != 0) {
				strncpy(m_pBuffer, source+start, nbremain);			
			} else {
				strncpy(m_pBuffer, source, nbremain);
			}
			m_pBuffer[nbremain] = 0;
		}
		if (shouldDelete != null) delete[] shouldDelete;
	}
}

void PGString::substring(int ofs) {
	substring(ofs, -1);
}

void PGString::terminate(char ch) {
	int len = strlen(m_pBuffer);
	if (m_pBuffer[len-1] != ch) append(ch);
}

const char* PGString::getValue() const {
	return (const char*)m_pBuffer;
}

char* PGString::getBuffer() {
	return m_pBuffer;
}

int PGStringEquals(const char *str1, const char*str2) {
	while (TRUE) {
		char ch1 = str1[0];
		char ch2 = str2[0];
		if (toupper(ch1) != toupper(ch2)) return FALSE;
		if (ch1 == 0) return TRUE;
		str1++;
		str2++;
	}
}

BOOL PGString::getBoolValue() const {
	const char* value = getValue();
	if (PGStringEquals(value, "ON") == 0) return TRUE;
	if (PGStringEquals(value, "YES") == 0) return TRUE;
	if (PGStringEquals(value, "TRUE") == 0) return TRUE;
	return FALSE;
}

int PGString::getIntValue() const {
	return atoi(getValue());
}

int PGString::getLength() const {
	return strlen(m_pBuffer);
}

void PGString::init() {
	m_iSize = 0;
	m_pBuffer = (char*)"";
}


/*
void PGImageHex(char* str, int nb) {
	int c0 = nb & 0xF;
	int c1 = (nb >> 4) & 0xF;
	str[0] = c0 > 9 ? c0 + 'A' - 10 : c0 + '0';
	str[1] = c1 > 9 ? c1 + 'A' - 10 : c1 + '0';
}
*/

