/***
 * Portable GUI
 * Code for PGWindow
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <portablegui.h>

PGWindow::PGWindow() {
#ifndef PGTINY
	m_hIcon = NULL;
	m_hStatus = NULL;
#endif
	m_bIconified = FALSE;
	m_bFirstShow = FALSE;
	m_iToolTipsFlags = PGWINDOW_TOOLTIPS_NONE;	
	init();	
}

PGWindow::~PGWindow() {
	removeFromWindow(this);
#ifndef PGTINY	
	if (m_hIcon != NULL) PGDECREF(m_hIcon);
	if (m_hStatus != NULL) PGDECREF(m_hStatus);
#endif
	term();
}

PGWindow* PGWindow::getWindow() {
	return this;
}

void PGWindow::setStatus(const char* status) {
#ifndef PGTINY
	if (m_hStatus != NULL) m_hStatus->setText(status);	
#endif
}

void PGWindow::addWindowListener(PGWindowListener* listener) {
	//m_vListeners.addElement(listener);
	m_vListeners.add(listener);
}

void PGWindow::removeWindowListener(PGWindowListener* listener) {
	//m_vListeners.removeElement(listener);
}

void PGWindow::notifyWindowListeners(int notify) {
	for (PGWindowListener* listener = (PGWindowListener*)m_vListeners.getFirst(); listener != NULL; ) {
		switch (notify) {
			case PGWINDOW_ACTIVATED:
				listener->windowActivated(this);
				break;
			case PGWINDOW_CLOSED:
				listener->windowClosed(this);
				break;
			case PGWINDOW_CLOSING:
				listener->windowClosing(this);
				break;
			case PGWINDOW_DEACTIVATED:
				listener->windowDeactivated(this);
				break;
			case PGWINDOW_DEICONIFIED:
				listener->windowDeiconified(this);
				break;
			case PGWINDOW_ICONIFIED:
				listener->windowIconified(this);
				break;
			case PGWINDOW_OPENED:
				listener->windowOpened(this);
				break;
		}
		listener = (PGWindowListener*)listener->getNextElement();		
	}
}

#ifndef PGTINY

	PGRealizedFont* PGWindow::getRealizedFont(PGFont* font) {
		PGRealizedFont* found = NULL;
		int nb = m_vRealizedFonts.getSize();
		for (int ctr = 0; ctr < nb; ctr++) {
			PGRealizedFont* real = (PGRealizedFont*)m_vRealizedFonts.getElementAt(ctr);
			if (real->m_hFont->equals(font)) found = real;
		}
		if (found == NULL) {
			found = new PGRealizedFont(font);
			found->realize(this);
			m_vRealizedFonts.addElement(found);
		}
		return found;
	}

	void PGWindow::setStatusDisplay(PGLabel* display) {
		if (m_hStatus != NULL) PGDECREF(m_hStatus);
		m_hStatus = display;
		PGINCREF(display);
	}
#endif

// Called when user tries to close window
void PGCloseWindowListener::windowClosing(PGWindow* wnd) {
	wnd->dispose();
}

