/***
 * Portable GUI
 * Code for PGButton
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

void PGOnClickButton(GtkWidget *widget, gpointer data);
GtkWidget* PGMakeLabelWithUnderscore(PGString* label, GtkJustification jtype);

void PGButton::addToWindow(PGWindow* parent) {
	// Create new button
	m_hWidget = gtk_button_new();
	// Add label
	GtkWidget* label = PGMakeLabelWithUnderscore(&m_sLabel, GTK_JUSTIFY_CENTER);
	gtk_container_add(GTK_CONTAINER(m_hWidget), label);
	gtk_widget_show(label);
	// Connect signal handler
	gtk_signal_connect(GTK_OBJECT(m_hWidget), "clicked", GTK_SIGNAL_FUNC(PGOnClickButton), (gpointer)this);
	// Add to layobox
	pg_gtk_layobox_append(GTK_CONTAINER(parent->m_hPGContainer), m_hWidget);
	// Is default button?
        if (m_bDefault) {
		GTK_WIDGET_SET_FLAGS(m_hWidget, GTK_CAN_DEFAULT);
		gtk_widget_grab_default(m_hWidget);
	}
	// Add to window
	PGAbstractButton::addToWindow(parent);	
}

// Problems if GTK already deleted them :O(
// Find out something else
void PGButton::removeFromWindow(PGWindow* parent) {
	PGAbstractButton::removeFromWindow(parent);
//	gtk_container_remove(GTK_CONTAINER(parent->m_hWidget), m_hWidget);
//	gtk_widget_destroy(m_hWidget);
}

void PGButton::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	GtkRequisition req;
	if (recalc) gtk_widget_size_request(m_hWidget, &req);
	else gtk_widget_get_child_requisition(m_hWidget, &req);
	dimension->width = req.width; dimension->height = req.height;
}

void PGOnClickButton(GtkWidget *widget, gpointer data) {
	PGButton *button = (PGButton *)data;
	button->onAction();
}

void PGCheckbox::addToWindow(PGWindow* parent) {
	// Create new button
	m_hWidget = gtk_check_button_new();
	// Add label
	GtkWidget* label = PGMakeLabelWithUnderscore(&m_sLabel, GTK_JUSTIFY_CENTER);
	gtk_container_add(GTK_CONTAINER(m_hWidget), label);
	gtk_widget_show(label);
	// Connect signal handler
	gtk_signal_connect(GTK_OBJECT(m_hWidget), "clicked", GTK_SIGNAL_FUNC(PGOnClickButton), (gpointer)this);	
	// Add to layobox
	pg_gtk_layobox_append(GTK_CONTAINER(parent->m_hPGContainer), m_hWidget);
	// Set selected?
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_hWidget), m_bState);	
	// Call super class
	PGAbstractButton::addToWindow(parent);
}

void PGCheckbox::removeFromWindow(PGWindow* parent) {
	PGAbstractButton::removeFromWindow(parent);	
}

void PGCheckbox::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	GtkRequisition req;
	if (recalc) gtk_widget_size_request(m_hWidget, &req);
	else gtk_widget_get_child_requisition(m_hWidget, &req);
	dimension->width = req.width; dimension->height = req.height;
}

BOOL PGCheckbox::getState() {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_hWidget))) return TRUE;
		else return FALSE;
	} else {
		return m_bState;
	}
}

void PGCheckbox::setState(BOOL state) {	
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_hWidget), state);	
	} else {
		m_bState = state;
	}		
}

void PGRadioButton::addToWindow(PGWindow* parent) {
	if (!(PGCOMPONENT_TEST_WINDOW(this))) {
		if (m_hGroup != null) m_hGroup->makeButtons(parent);
		else makeButton(parent, null);
	}
}

void PGRadioButton::removeFromWindow(PGWindow* parent) {
	PGAbstractButton::removeFromWindow(parent);	
}


void PGRadioButton::makeButton(PGWindow* parent, PGRadioButton* prev) {	
	// Create new button
	if (prev == null) {
		m_hWidget = gtk_radio_button_new(null);
	} else {
		GSList *group = gtk_radio_button_group(GTK_RADIO_BUTTON(prev->m_hWidget));
		m_hWidget = gtk_radio_button_new(group);
	}
	// Add label
	GtkWidget* label = PGMakeLabelWithUnderscore(&m_sLabel, GTK_JUSTIFY_CENTER);
	gtk_container_add(GTK_CONTAINER(m_hWidget), label);
	gtk_widget_show(label);
	// Connect signal handler
	gtk_signal_connect(GTK_OBJECT(m_hWidget), "clicked", GTK_SIGNAL_FUNC(PGOnClickButton), (gpointer)this);	
	// Add to layobox
	pg_gtk_layobox_append(GTK_CONTAINER(parent->m_hPGContainer), m_hWidget);
	// Set selected?
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_hWidget), FALSE);	
	PGAbstractButton::addToWindow(parent);
}	

void PGRadioButton::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	GtkRequisition req;
	if (recalc) gtk_widget_size_request(m_hWidget, &req);
	else gtk_widget_get_child_requisition(m_hWidget, &req);
	dimension->width = req.width; dimension->height = req.height;
}

BOOL PGRadioButton::getState() {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_hWidget))) return TRUE;
		else return FALSE;
	} else {
		return m_bState;
	}
}

void PGRadioButton::setState(BOOL state) {	
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		getPGApplication()->printf("State: %d\n", state);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_hWidget), state);	
	} else {
		m_bState = state;
	}		
}

boolean PGButtonGroup::shouldSelect() {
	return false;
}

// Create label with underscore at '&' positions
GtkWidget* PGMakeLabelWithUnderscore(PGString* label, GtkJustification jtype) {
	BOOL isUnder = FALSE;
	int len = label->getLength();
	char *text        = new char[len+1];
	char *underscores = new char[len+1];
	const char *labeltext = label->getValue();
	int ps = 0, pt = 0;
	while (ps < len) {
		char ch = labeltext[ps];
		if (ch == 0) break;
		if (ch == '&') {
			isUnder = TRUE;
		} else {
			if (isUnder) {
				underscores[pt] = '_';
				isUnder = FALSE;
			} else {
				underscores[pt] = ' ';
			}
			text[pt] = ch;
			pt++;
		}
		ps++;
	}
	text[pt] = underscores[pt] = 0;
	GtkWidget* widget = gtk_label_new(text);
	gtk_label_set_pattern(GTK_LABEL(widget), underscores);
	gtk_label_set_justify(GTK_LABEL(widget), jtype);
	delete text;
	delete underscores;
	return widget;
}


