/***
 * Portable GUI
 * Code for PGButton
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

static gint PGCanvasExposeEvent(GtkWidget *widget, GdkEventExpose *event, gpointer data) {
	PGGraphics graph;
	PGCanvas* canvas = (PGCanvas*)data;								
	GtkAllocation* rect = &widget->allocation;		
	graph.m_iOwnerMode = PGGRAPHICS_OWNER_SYSTEM;
	graph.m_hWidget = widget;
	graph.m_hDrawable = widget->window;
	graph.m_hClip.width = rect->width;
	graph.m_hClip.height = rect->height;	
	graph.realize();
	canvas->paint(&graph);	
	return FALSE;
}

static gint PGCanvasMotionEvent(GtkWidget *widget, GdkEventMotion *event, gpointer data) {
	int x, y;
	GdkModifierType state;	
	PGCanvas* canvas = (PGCanvas*)data;								
	if (event->is_hint) {
        	gdk_window_get_pointer(event->window, &x, &y, &state);
	} else {
		x = (int)event->x;
                y = (int)event->y;
                state = (GdkModifierType)event->state;
	}
	canvas->mouseMove(x, y);
	return TRUE;
}

static gint PGCanvasMouseDownEvent(GtkWidget *widget, GdkEventButton *event, gpointer data) {
	PGCanvas* canvas = (PGCanvas*)data;								
	int mask = 0;
	switch (event->button) {
		case 1: mask |= PGMOUSEEVENT_BUTTON1_MASK; break;
		case 2: mask |= PGMOUSEEVENT_BUTTON2_MASK; break;		
		case 3: mask |= PGMOUSEEVENT_BUTTON3_MASK; break;		
	}
	canvas->mouseDown((int)event->x, (int)event->y, mask);	
	return TRUE;
}

static gint PGCanvasMouseUpEvent(GtkWidget *widget, GdkEventButton *event, gpointer data) {
	PGCanvas* canvas = (PGCanvas*)data;								
	int mask = 0;
	switch (event->button) {
		case 1: mask |= PGMOUSEEVENT_BUTTON1_MASK; break;
		case 2: mask |= PGMOUSEEVENT_BUTTON2_MASK; break;		
		case 3: mask |= PGMOUSEEVENT_BUTTON3_MASK; break;		
	}
	canvas->mouseUp((int)event->x, (int)event->y, mask);	
	return TRUE;
}

void PGCanvas::addToWindow(PGWindow* parent) {
	// Create new drawing area
	m_hWidget = (GtkWidget*)gtk_drawing_area_new();	
	gtk_drawing_area_size(GTK_DRAWING_AREA(m_hWidget), 300, 300);
	// Connect signal handler	
	gtk_widget_set_events(m_hWidget, GDK_EXPOSURE_MASK                                            | GDK_LEAVE_NOTIFY_MASK
                                       | GDK_BUTTON_PRESS_MASK
                                       | GDK_BUTTON_RELEASE_MASK				       
                                       | GDK_POINTER_MOTION_MASK
                                       | GDK_POINTER_MOTION_HINT_MASK);
	gtk_signal_connect(GTK_OBJECT(m_hWidget), "expose_event", GTK_SIGNAL_FUNC(PGCanvasExposeEvent), (gpointer)this);
        gtk_signal_connect(GTK_OBJECT(m_hWidget), "motion_notify_event", GTK_SIGNAL_FUNC(PGCanvasMotionEvent), (gpointer)this);
        gtk_signal_connect(GTK_OBJECT(m_hWidget), "button_press_event", GTK_SIGNAL_FUNC(PGCanvasMouseDownEvent), (gpointer)this);
        gtk_signal_connect(GTK_OBJECT(m_hWidget), "button_release_event", GTK_SIGNAL_FUNC(PGCanvasMouseUpEvent), (gpointer)this);	
	// Add to layobox	
	pg_gtk_layobox_append(GTK_CONTAINER(parent->m_hPGContainer), m_hWidget);	
	PGNormalComponent::addToWindow(parent);
}

void PGCanvas::removeFromWindow(PGWindow* parent) {
	PGNormalComponent::removeFromWindow(parent);
//	if (m_hWnd != NULL) DestroyWindow(m_hWnd);
}

void PGCanvas::setBounds(int x, int y, int w, int h) {
	PGNormalComponent::setBounds(x, y, w, h);
}
