/***
 * Portable GUI
 * Code for PGButton
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

GtkJustification PGLabelGetJustify(int justify);

void PGLabel::addToWindow(PGWindow* parent) {
	GtkWidget* frame;
	boolean sunken = (m_iFlags & PGLABEL_SUNKEN) != 0;
	// Create new label
	m_hLabel = gtk_label_new(m_sLabel.getValue());	
	gtk_misc_set_alignment(GTK_MISC(m_hLabel), 0.0, 0.5);
	// Put in frame if sunken
	if (sunken) {
		frame = gtk_frame_new(NULL);
		gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
		gtk_container_add(GTK_CONTAINER(frame), m_hLabel);
		gtk_widget_show(m_hLabel);		
		m_hWidget = frame;
	} else {
		m_hWidget = m_hLabel;	
	}		    		    
	pg_gtk_layobox_append(GTK_CONTAINER(parent->m_hPGContainer), m_hWidget);
	// Add to window
	PGNormalComponent::addToWindow(parent);	
}

// Problems if GTK already deleted them :O(
// Find out something else
void PGLabel::removeFromWindow(PGWindow* parent) {
	PGNormalComponent::removeFromWindow(parent);
//	gtk_container_remove(GTK_CONTAINER(parent->m_hWidget), m_hWidget);
//	gtk_widget_destroy(m_hWidget);
}

void PGLabel::setText(const char* label) {
	m_sLabel.setCopyValue(label);
	if (PGCOMPONENT_TEST_WINDOW(this))
		gtk_label_set_text(GTK_LABEL(m_hLabel), m_sLabel.getValue());
}

void PGLabel::changeFlags(int orval, int andval) {
	PGNormalComponent::changeFlags(orval, andval);
/*	if (PGCOMPONENT_TEST_WINDOW(this))
		SetWindowLong(m_hWnd, GWL_STYLE, PGLabelMakeStyle(this));*/
}

/*
void PGLabel::setJustify(int justify) {
	m_iJustify = justify;
	if (PGCOMPONENT_TEST_WINDOW(this))
		gtk_label_set_justify(GTK_LABEL(m_hWidget), PGLabelGetJustify(g));
}
*/
void PGLabel::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	GtkRequisition req;
	if (recalc) gtk_widget_size_request(m_hWidget, &req);
	else gtk_widget_get_child_requisition(m_hWidget, &req);
	dimension->width = req.width; dimension->height = req.height;
}

GtkWidget* PGLabel::getWidget() {
	return m_hLabel;
}

GtkJustification PGLabelGetJustify(int justify) {
	if (justify == PGLABEL_LEFT) return GTK_JUSTIFY_LEFT;
	if (justify == PGLABEL_RIGHT) return GTK_JUSTIFY_RIGHT;
	return GTK_JUSTIFY_CENTER;
}
