/***
 * Portable GUI
 * Code for PGMenu
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

GtkWidget* PGMakeLabelWithUnderscore(PGString* label, GtkJustification jtype);
void PGOnClickMenuItem(GtkWidget *widget, gpointer data);


void PGOnClickMenuItem(GtkWidget *widget, gpointer data) {
	PGMenuItem *item = (PGMenuItem *)data;
	item->onAction();
}

void PGOnSelectMenuItem(GtkWidget *widget, gpointer data) {
	PGMenuItem *item = (PGMenuItem *)data;
	item->onSelect();
}

void PGOnDeselectMenu(GtkWidget *widget, gpointer data) {
	PGMenu* menu = (PGMenu*)data;
	menu->m_hFrame->getMenuBar()->onMenuClosing();						
}

// Menu item

void PGMenuItem::init(char* title, char* status) {
	m_sItem.setCopyValue(title);
	m_sStatus.setCopyValue(status);	
	// Make the menu item
	m_hWidget = gtk_menu_item_new();
	// Make the label
	GtkWidget* label = PGMakeLabelWithUnderscore(&m_sItem, GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	// Make hbox
	gtk_container_add(GTK_CONTAINER(m_hWidget), label);
	// Show all
	gtk_widget_show(label);
	gtk_widget_show(m_hWidget);
	// Signal
	gtk_signal_connect(GTK_OBJECT(m_hWidget), "activate", GTK_SIGNAL_FUNC(PGOnClickMenuItem), (gpointer)this);
	gtk_signal_connect(GTK_OBJECT(m_hWidget), "select", GTK_SIGNAL_FUNC(PGOnSelectMenuItem), (gpointer)this);
}

PGMenuItem::~PGMenuItem() {
//	if (m_iId != -1) getPGApplication()->m_vCommandList.freeCommand(m_iId);
}

void PGMenuItem::addToMenu(PGMenu* menu, int idx) {
	gtk_menu_append(GTK_MENU(menu->m_hChild), m_hWidget);
}

void PGMenuItem::removeFromMenu(PGMenu* menu) {
}

// Separator

PGMenuSeparator::PGMenuSeparator() {
	m_hWidget = gtk_menu_item_new();
	gtk_widget_show(m_hWidget);
}

PGMenuSeparator::~PGMenuSeparator() {
}

void PGMenuSeparator::addToMenu(PGMenu* menu, int idx) {
	gtk_menu_append(GTK_MENU(menu->m_hChild), m_hWidget);
}

void PGMenuSeparator::removeFromMenu(PGMenu* menu) {
}

// Menu

PGMenu::PGMenu(char* title) {
	m_sItem.setCopyValue(title);
	// Make the menu item
	m_hWidget = gtk_menu_item_new();
	// Make the label
	GtkWidget* label = PGMakeLabelWithUnderscore(&m_sItem, GTK_JUSTIFY_LEFT);
	gtk_container_add(GTK_CONTAINER(m_hWidget), label);
	gtk_widget_show(m_hWidget);
	gtk_widget_show(label);
	// Make the child menu panel
	m_hChild = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(m_hWidget), m_hChild);
	// Append tear-off item
	GtkWidget *widget = gtk_tearoff_menu_item_new();
	gtk_menu_append(GTK_MENU(m_hChild), widget);
	gtk_widget_show(widget);
	// Add signal
	gtk_signal_connect(GTK_OBJECT(m_hWidget), "deselect", GTK_SIGNAL_FUNC(PGOnDeselectMenu), (gpointer)this);		
}

PGMenu::~PGMenu() {
//	if (m_hMenu != NULL) DestroyMenu(m_hMenu);
}

void PGMenu::addToMenuBar(PGMenuBar* menu, int idx) {
	gtk_menu_bar_append(GTK_MENU_BAR(menu->m_hChild), m_hWidget);
}

void PGMenu::removeFromMenuBar(PGMenuBar* menu) {
	
}

void PGMenu::addToMenu(PGMenu* menu, int idx) {
	gtk_menu_append(GTK_MENU(menu->m_hChild), m_hWidget);
}

void PGMenu::removeFromMenu(PGMenu* menu) {

}

// Menu bar

PGMenuBar::PGMenuBar() {
	m_hWidget = gtk_handle_box_new();
	m_hChild = gtk_menu_bar_new();
	gtk_container_add(GTK_CONTAINER(m_hWidget), m_hChild);
	gtk_widget_show(m_hChild);
	gtk_widget_show(m_hWidget);
}

PGMenuBar::~PGMenuBar() {
//	if (m_hMenu != NULL) DestroyMenu(m_hMenu);
}

void PGMenuBar::addToFrame(PGFrame* frame) {
	gtk_box_pack_start(GTK_BOX(frame->m_hChild), m_hWidget, FALSE, FALSE, 0);
	for (int ctr = 0; ctr < m_vMenus.getSize(); ctr++) {
		PGMenu* item = (PGMenu*)m_vMenus.getElementAt(ctr);
		item->setFrame(frame);
	}	
}

void PGMenuBar::removeFromFrame(PGFrame* frame) {
//	SetMenu(frame->m_hWnd, NULL);
}

