/***
 * Portable GUI
 * Code for PGNormalComponent
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

void PGNormalComponentSetTooltip(PGNormalComponent* comp);

PGNormalComponent::PGNormalComponent() {
	m_hWidget = (GtkWidget*)NULL;
}

void PGNormalComponent::setVisible(BOOL visible) {
	PGComponent::setVisible(visible);
	if (visible) gtk_widget_show(m_hWidget);
	else gtk_widget_hide(m_hWidget);
}

void PGNormalComponent::setBounds(int x, int y, int w, int h) {
	GtkAllocation alloc = {x, y, w, h};
	if (GTK_WIDGET_VISIBLE(m_hWidget)) {
		gtk_widget_size_allocate(m_hWidget, &alloc);		
	}
}

void PGNormalComponent::getBounds(PGRectangle* rectangle) {
	GtkAllocation* rect = &m_hWidget->allocation;
	rectangle->x = rect->x;
	rectangle->y = rect->y;
	rectangle->width = rect->width;
	rectangle->height = rect->height;
}

void PGNormalComponentSetStyle(GtkWidget* widget, PGStyle* style) {
	gtk_widget_modify_style(widget, style->m_hWStyle);
	if (GTK_IS_BIN(widget)) {
		GtkBin* bin = GTK_BIN(widget);
		if (bin->child != null) gtk_widget_modify_style(bin->child, style->m_hWStyle);
	}
}

void PGNormalComponent::addToWindow(PGWindow* parent) {
	if (m_hStyle != NULL) PGNormalComponentSetStyle(getWidget(), m_hStyle);	
	if (m_sToolTip.getLength() != 0) PGNormalComponentSetTooltip(this);	
	gtk_widget_set_sensitive(getWidget(), (m_iFlags & PGCOMPONENT_DISABLED) == 0);
	PGComponent::addToWindow(parent);
	
	// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! remove from tooltips?
}

void PGNormalComponent::setStyle(PGStyle* style) {
	PGComponent::setStyle(style);
	if (PGCOMPONENT_TEST_WINDOW(this)) PGNormalComponentSetStyle(getWidget(), style);
}

void PGNormalComponent::requestFocus() {
//	if (PGCOMPONENT_TEST_WINDOW(this)) SetFocus(m_hWnd);
}

void PGNormalComponent::repaint() {
	if (m_hWidget != null) gtk_widget_draw(m_hWidget, NULL);
}

void PGNormalComponent::repaint(PGRectangle* r) {
	GdkRectangle update_rect;
	update_rect.x = 0;
        update_rect.y = 0;
        update_rect.width = 1000;
        update_rect.height = 1000;

/*	RECT rect;
	rect.left = r->x;
	rect.top = r->y;
	rect.bottom = r->y+r->height;
	rect.right = r->x+r->width;
	InvalidateRect(m_hWnd, &rect, FALSE);*/
}

void PGNormalComponent::createImage(int w, int h, PGImage* img) {
	PGWindow* win = getWindow();
	if (win != NULL) {
		win->createImage(w, h, img);	
	} else {
		img->m_iError = PGIMAGE_ERROR_CREATE;
	}
}

/*
void PGNormalComponent::setFont(PGFont* font) {
	if (m_hFont != NULL) PGDECREF(m_hFont);
	m_hFont = font;
	PGINCREF(font);
	if (PGCOMPONENT_TEST_WINDOW(this)) PGNormalComponentWMSetFont(this);
}
*/

void PGNormalComponent::changeFlags(int orval, int andval) {
	BOOL prevena = (m_iFlags & PGCOMPONENT_DISABLED) == 0;
	PGComponent::changeFlags(orval, andval);
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		BOOL newena = (m_iFlags & PGCOMPONENT_DISABLED) == 0;
		if (prevena != newena) gtk_widget_set_sensitive(getWidget(), newena);
	}
}

void PGNormalComponent::getGraphics(PGGraphics* graph) {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		GtkAllocation* rect = &m_hWidget->allocation;	
		graph->unrealize();	
		graph->m_hWidget = m_hWidget;
		graph->m_hDrawable = m_hWidget->window;
		graph->m_hClip.width = rect->width;
		graph->m_hClip.height = rect->height;
		graph->realize();
	}
}

GtkWidget* PGNormalComponent::getWidget() {
	return m_hWidget;
}

void PGNormalComponentSetTooltip(PGNormalComponent* comp) {
	PGWindow* win = comp->getWindow();
	if (win != NULL && win->initToolTips()) {
		// Add to the tooltip object
		gtk_tooltips_set_tip(win->m_hToolTips, comp->getWidget(), comp->m_sToolTip.getValue(), NULL);
		// Tooltip
		comp->changeFlags(PGCOMPONENT_TOOLTIP, PGCOMPONENT_ALLFLAGS);
	}
}

