/***
 * Portable GUI
 * Code for PGTextArea
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

void PGTextArea::addToWindow(PGWindow* parent) {
	GtkWidget *text = m_hChild = gtk_text_new(NULL, NULL);
	if ((m_iScrollBar & PGTEXTAREA_SCROLLBARS_VERTICAL_ONLY) != 0) {
		// Make scrollbar
		GtkWidget *vscrollbar = gtk_vscrollbar_new(GTK_TEXT(text)->vadj);
		// Pack in HBOX
		GtkWidget *hbox = m_hWidget = gtk_hbox_new(FALSE, 1);
		gtk_box_pack_start(GTK_BOX(hbox), text, TRUE, TRUE, 0);
		gtk_box_pack_end(GTK_BOX(hbox), vscrollbar, FALSE, FALSE, 0);
		// Show stuff
		gtk_widget_show(vscrollbar);
		gtk_widget_show(text);
		// Add to layobox
		pg_gtk_layobox_append(GTK_CONTAINER(parent->m_hPGContainer), hbox);
	} else {
		pg_gtk_layobox_append(GTK_CONTAINER(parent->m_hPGContainer), text);
		m_hWidget = text;
	}
	gtk_widget_realize(text);
	gtk_text_insert(GTK_TEXT(text), (GdkFont*)NULL, (GdkColor*)NULL, (GdkColor*)NULL, m_sText.getValue(), -1);
	gtk_text_set_editable(GTK_TEXT(text), isEditable());
	// Add to window
	PGTextComponent::addToWindow(parent);
}

void PGTextArea::removeFromWindow(PGWindow* parent) {
	PGTextComponent::removeFromWindow(parent);
//	if (m_hWnd != NULL) DestroyWindow(m_hWnd);
}

void PGTextArea::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	GtkRequisition req = {0, 0};
	if (recalc) gtk_widget_size_request(m_hWidget, &req);
	else gtk_widget_get_child_requisition(m_hWidget, &req);
	dimension->width = req.width; dimension->height = req.height;
}

// Gets the size of the text in this area
int PGTextArea::getSize() {
	if (PGCOMPONENT_TEST_WINDOW(this)) return gtk_text_get_length(GTK_TEXT(m_hChild));
	else return m_sText.getLength();
}

// Gets the position of the text insertion caret
int PGTextArea::getCaretPosition() {
	if (PGCOMPONENT_TEST_WINDOW(this)) return gtk_text_get_point(GTK_TEXT(m_hChild));
	else return 0;
}

// Sets the position of the text insertion caret
void PGTextArea::setCaretPosition(int position) {
	if (PGCOMPONENT_TEST_WINDOW(this)) gtk_text_set_point(GTK_TEXT(m_hChild), position);
}

// Sets the text that is presented by this text component
void PGTextArea::setText(const char* str) {
	int len = getSize();
	replaceRange(str, 0, len-1);
}

// Update component on every change?
void PGTextArea::setDirectUpdate(BOOL directupdate) {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		if (!directupdate) gtk_text_freeze(GTK_TEXT(m_hChild));
		else gtk_text_thaw(GTK_TEXT(m_hChild));
	}
}

// Appends the given text to the text area's current text.
void PGTextArea::append(const char* str, BOOL addNewLine) {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		int len = gtk_text_get_length(GTK_TEXT(m_hChild));
		gtk_text_set_point(GTK_TEXT(m_hChild), len);
		gtk_text_insert(GTK_TEXT(m_hChild), NULL, NULL, NULL, str, -1);
		if (addNewLine)	gtk_text_insert(GTK_TEXT(m_hChild), NULL, NULL, NULL, PGSTRING_NEWLINE, -1);
	} else {
		m_sText.append(str, addNewLine);
	}
}

// Inserts the specified text at the specified position in this text area.
void PGTextArea::insert(const char* str, int pos) {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		gtk_text_set_point(GTK_TEXT(m_hChild), pos);
		gtk_text_insert(GTK_TEXT(m_hChild), NULL, NULL, NULL, str, -1);
	} else {
		m_sText.insert(str, pos);
	}
}

// Replaces text between the indicated start and end positions with the specified replacement.
void PGTextArea::replaceRange(const char* str, int start, int end) {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		int len = end-start+1;
		gtk_text_set_point(GTK_TEXT(m_hChild), start);
		gtk_text_forward_delete(GTK_TEXT(m_hChild), len);
		gtk_text_insert(GTK_TEXT(m_hChild), NULL, NULL, NULL, str, -1);
	} else {
		m_sText.remove(start, end);
		m_sText.insert(str, start);
	}
}
