/***
 * Portable GUI
 * Code for PGTextArea
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>


// Gets the size of the text in this area
int PGTextComponent::getSize() {	
	if (PGCOMPONENT_TEST_WINDOW(this)) return 0;
	else return m_sText.getLength();	
}

// Gets the position of the text insertion caret
int PGTextComponent::getCaretPosition() {
	unsigned long start = 0;
//	if (PGCOMPONENT_TEST_WINDOW(this)) SendMessage(m_hWnd, EM_GETSEL, (WPARAM)&start, (LPARAM)&end);
	return start;
}

// Sets the position of the text insertion caret
void PGTextComponent::setCaretPosition(int position) {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		//SendMessage(m_hWnd, EM_SETSEL, position, position);
	}
}

// Sets the text that is presented by this text component
void PGTextComponent::setText(const char* str) {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		//SetWindowText(m_hWnd, str);
	} else {
		m_sText.setCopyValue(str);
	}
}

// Selects the text between the specified start and end positions
void PGTextComponent::select(int selectionStart, int selectionEnd) {
	if (PGCOMPONENT_TEST_WINDOW(this))
		gtk_editable_select_region(GTK_EDITABLE(m_hChild), selectionStart, selectionEnd);
}

// Gets the start/end position of the selected text
void PGTextComponent::getSelection(int* start, int* end) {
	(*start) = (int)GTK_EDITABLE(m_hChild)->selection_start_pos;
	(*end) = (int)GTK_EDITABLE(m_hChild)->selection_end_pos;
}

// Gets the text
void PGTextComponent::getText(PGString* str, int start, int end) {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		//int len = end-start+1;
		gchar* result = gtk_editable_get_chars(GTK_EDITABLE(m_hChild), start, end);
		str->setCopyValue(result);
		g_free(result);
	} else {
		str->setCopyValue(m_sText.getValue());
		str->substring(start, end);
	}
}

// Undo last edit operation
void PGTextComponent::undoOperation() {
}

// Copy to clipboard
void PGTextComponent::copy() {
	if (PGCOMPONENT_TEST_WINDOW(this)) gtk_editable_copy_clipboard(GTK_EDITABLE(m_hChild));
}

// Cut to clipboard
void PGTextComponent::cut() {
	if (PGCOMPONENT_TEST_WINDOW(this)) gtk_editable_cut_clipboard(GTK_EDITABLE(m_hChild));
}

// Paste from clipboard
void PGTextComponent::paste() {
	if (PGCOMPONENT_TEST_WINDOW(this)) gtk_editable_paste_clipboard(GTK_EDITABLE(m_hChild));
}

void PGTextField::addToWindow(PGWindow* parent) {
	m_hChild = m_hWidget = gtk_entry_new();
	pg_gtk_layobox_append(GTK_CONTAINER(parent->m_hPGContainer), m_hWidget);
	gtk_widget_realize(m_hWidget);
	gtk_entry_set_editable(GTK_ENTRY(m_hWidget), isEditable());
	PGTextComponent::addToWindow(parent);
}

void PGTextField::removeFromWindow(PGWindow* parent) {
	PGTextComponent::removeFromWindow(parent);
//	if (m_hWnd != NULL) DestroyWindow(m_hWnd);
}

void PGTextField::changeFlags(int orval, int andval) {
	PGNormalComponent::changeFlags(orval, andval);
/*	if (PGCOMPONENT_TEST_WINDOW(this))
		SetWindowLong(m_hWnd, GWL_STYLE, PGTextFieldMakeStyle(this));*/
}

void PGTextField::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	GtkRequisition req = {0, 0};
	if (recalc) gtk_widget_size_request(m_hWidget, &req);
	else gtk_widget_get_child_requisition(m_hWidget, &req);
	dimension->width = req.width; dimension->height = req.height;
}

// Gets the size of the text in this area
int PGTextField::getSize() {
	return PGTextComponent::getSize();
}

// Gets the position of the text insertion caret
int PGTextField::getCaretPosition() {
	return PGTextComponent::getCaretPosition();
}

// Sets the position of the text insertion caret
void PGTextField::setCaretPosition(int position) {
	PGTextComponent::setCaretPosition(position);
}

// Sets the text that is presented by this text component
void PGTextField::setText(const char* str) {
	PGTextComponent::setText(str);
}




























