/***
 * Portable GUI
 * Code for PGImage
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>
#include <pgdraw/pgbmp.h>

#include <string.h>

void PGImageReader::specificConstruct() {
	m_hData = null;
	m_hMask = null;
	m_hMaskGC = null;
	m_iImageRowBytes = 0;
	m_iNbColors = 0;
	m_iCMapColors = 0;	
	m_hColorMap = null;
	m_hCMapData = null;
}

PGImageReader::~PGImageReader() {
	if (m_hColorMap != null) {
		for (PGBYTE i = 0; i < m_iNbPal; i++) 
			if (m_hColorMap[i] != null) gdk_rgb_cmap_free(m_hColorMap[i]);
		delete[] m_hColorMap;
	}
	if (m_hCMapData != null) delete[] m_hCMapData;
	if (m_hMaskGC != null) gdk_gc_destroy(m_hMaskGC);	
	if (m_hMask != null) gdk_bitmap_unref(m_hMask);
}

void PGImageReader::specificImageSize(int rows, int cols) {
	// Init main buffer
	m_iImageRowBytes = (int)m_iBitsPerPel * cols / 8;
	long size = (long)m_iImageRowBytes*rows;
	for (int ctr = 0; ctr < size; ctr++) m_hData[ctr] = 0;
	// Init transp buffer
	if (m_bTransp != 0) {
		GdkColor col;	
		m_hMask = gdk_pixmap_new(m_hImage->m_hOwner->m_hWidget->window, cols, rows, 1);
		if (m_hMaskGC != null) gdk_gc_destroy(m_hMaskGC);
		m_hMaskGC = gdk_gc_new(m_hMask);	
      		col.pixel = 0;
		gdk_gc_set_foreground(m_hMaskGC, &col);
		gdk_draw_rectangle(m_hMask, m_hMaskGC, TRUE, 0, 0, cols, rows);
      		col.pixel = 1;
		gdk_gc_set_foreground(m_hMaskGC, &col);	
//		long size = (long)(cols*rows+7)/8;
//		for (int ctr = 0; ctr < size; ctr++) m_hMask[ctr] = 0;		
	}
}

void PGImageReader::specificInitialize(boolean transp, int maxrows, int maxcols, PGBYTE nbpal) {
	// Init main buffer
	int mxrowbytes = ((int)m_iBitsPerPel * maxcols) / 8;
	long size = (long)mxrowbytes*maxrows;
	m_hData = new guchar[size];
	// Init transp buffer
//	if (transp) {
//		size = (long)(maxcols*maxrows+7)/8;
//		m_hMask = new guchar[size];
//	}
	// Init color map stuff
	m_hColorMap = new (GdkRgbCmap*)[nbpal];
	for (PGBYTE i = 0; i < nbpal; i++) m_hColorMap[i] = null;
}

void PGImageReader::setPalette(int ncolors) {
	if ((ncolors > m_iCMapColors) || (m_hCMapData == null)) {
		if (m_hCMapData != null) delete[] m_hCMapData;
		m_hCMapData = new guint32[ncolors];
		m_iCMapColors = ncolors;	
	}
	m_iNbColors = ncolors;
}

void PGImageReader::setColor(int idx, int red, int green, int blue) {
	if (m_hCMapData != null)
		m_hCMapData[idx] = ((guint32)red << 16) + 
				   ((guint32)green << 8) + 
				   (guint32)blue;				   
}

void PGImageReader::closePalette() {
	if (m_hColorMap != null) {
		if (m_hColorMap[m_iCrPal] != null) gdk_rgb_cmap_free(m_hColorMap[m_iCrPal]);
		m_hColorMap[m_iCrPal] = gdk_rgb_cmap_new(m_hCMapData, m_iNbColors);
	}
}

void PGImageReader::realizeImage() {		
	int cols = m_hImage->m_iCols;
	int rows = m_hImage->m_iRows;
	if (m_hData != NULL) {
		GdkBitmap* bitmap = null;
		// Get window and visual
		GdkWindow* window = m_hImage->m_hOwner->m_hWidget->window;
		GdkVisual* visual = gdk_window_get_visual(window);		
		// Create pixmap and fill it with image data (GdkRgb)
		GdkPixmap* pixmap = gdk_pixmap_new(window, cols, rows, visual->depth);      
		GdkGC* gc = gdk_gc_new(pixmap);
		gdk_draw_indexed_image(pixmap, gc, 0, 0, cols, rows, GDK_RGB_DITHER_MAX, 
		                       m_hData, m_iImageRowBytes, m_hColorMap[m_iCrPal]); 
		gdk_gc_destroy(gc);
		// Create background bitmap using GdkImage
		if ((m_bTransp != 0) && (m_hMask != null)) {
//			GdkImage* back = gdk_image_new_bitmap(visual, m_hMask, cols, rows);		
//			bitmap = gdk_pixmap_new(window, cols, rows, 1);
//			gc = gdk_gc_new(bitmap);								
//			gdk_draw_image(bitmap, gc, back, 0, 0, 0, 0, cols, rows);
//			gdk_gc_destroy(gc);			
//			gdk_image_destroy(back);			
			bitmap = m_hMask;
			m_hMask = null;
		}
		// Put data in image		
		m_hImage->m_hPixmap = pixmap;
		m_hImage->m_hMask = bitmap;
	}
}

void PGImageReader::put(unsigned char *data, int row) {
	int cols = m_hImage->m_iCols;
	guchar* dest = (guchar*)m_hData+row*m_iImageRowBytes;
	for (int ctr = 0; ctr < cols; ctr++) {
		dest[ctr] = data[ctr];
	}
	if (m_bTransp != 0) {	
		int prevcol = 0; 
		boolean trans = true;
		for (int col = 0; col < m_iImageRowBytes; col++) {
			char color = data[col];
			if ((color == (char)m_iBackgroundIndex) != trans) {
				if (!trans && prevcol < col) {
					gdk_draw_line(m_hMask, m_hMaskGC, prevcol, row, col-1, row);
				}
				trans = !trans;
				prevcol = col;
			}
		}
		if (!trans && prevcol < m_iImageRowBytes)
			gdk_draw_line(m_hMask, m_hMaskGC, prevcol, row, m_iImageRowBytes, row);
	}	
}

void PGImage::termspecific() {
	if (m_hPixmap != NULL) gdk_pixmap_unref(m_hPixmap);
	if (m_hMask != NULL) gdk_bitmap_unref(m_hMask);
}

PGGraphics* PGImage::getGraphics() {		
	if (m_hGraphics == NULL) {				
		m_hGraphics = new PGGraphics();
		PGINCREF(m_hGraphics);			
		m_hGraphics->m_iOwnerMode = PGGRAPHICS_OWNER_BITMAP;
		m_hGraphics->m_hWidget = m_hOwner->m_hWidget;
		m_hGraphics->m_hDrawable = m_hPixmap;
		m_hGraphics->m_hClip.width = m_iCols;
		m_hGraphics->m_hClip.height = m_iRows;
		m_hGraphics->realize();
	}
	return m_hGraphics;
}

void PGImage::initspecific() {
	m_hPixmap = NULL;
	m_hMask = NULL;
}

void PGIcon::init() {
}

void PGIcon::term() {
}

