
#include <portablegui.h>

static void gtk_pgcanvas_class_init(GtkPGCanvasClass *klass);
static void gtk_pgcanvas_init(GtkPGCanvas *pgcanvas);
static void gtk_pgcanvas_set_arg(GtkObject *object, GtkArg *arg, guint arg_id);
static void gtk_pgcanvas_get_arg(GtkObject *object, GtkArg *arg, guint arg_id);
static void gtk_pgcanvas_finalize(GtkObject *object);
static void gtk_pgcanvas_size_request (GtkWidget *widget, GtkRequisition *requisition);
static void gtk_pgcanvas_style_set(GtkWidget *widget, GtkStyle *previous_style);
static gint gtk_pgcanvas_expose(GtkWidget *widget, GdkEventExpose *event);
static void pg_gtk_pgcanvas_size_allocate(GtkWidget *widget, GtkAllocation *allocation);

static GtkMiscClass *parent_class = NULL;

GtkType gtk_pgcanvas_get_type(void) {
  static GtkType pgcanvas_type = 0;
  
  if (!pgcanvas_type) {
      static const GtkTypeInfo pgcanvas_info =
      {
	"GtkPGCanvas",
	sizeof (GtkPGCanvas),
	sizeof (GtkPGCanvasClass),
	(GtkClassInitFunc) gtk_pgcanvas_class_init,
	(GtkObjectInitFunc) gtk_pgcanvas_init,
        /* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };
      
      pgcanvas_type = gtk_type_unique(GTK_TYPE_MISC, &pgcanvas_info);
      // gtk_type_set_chunk_alloc(pgcanvas_type, 32);
    }
  
  return pgcanvas_type;
}

static void gtk_pgcanvas_class_init(GtkPGCanvasClass *klass) {
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;
  
  object_class = (GtkObjectClass*) klass;
  widget_class = (GtkWidgetClass*) klass;
  
  parent_class = (GtkMiscClass*)gtk_type_class(GTK_TYPE_MISC);
  
//  gtk_object_add_arg_type ("GtkPGCanvas::pgcanvas", GTK_TYPE_STRING, GTK_ARG_READWRITE, ARG_PGCANVAS);

  object_class->set_arg = gtk_pgcanvas_set_arg;
  object_class->get_arg = gtk_pgcanvas_get_arg;
  object_class->finalize = gtk_pgcanvas_finalize;
  
  widget_class->size_request = gtk_pgcanvas_size_request;
  widget_class->size_allocate = pg_gtk_pgcanvas_size_allocate; // remove
  widget_class->style_set = gtk_pgcanvas_style_set;
  widget_class->expose_event = gtk_pgcanvas_expose;
}

static void gtk_pgcanvas_set_arg(GtkObject *object, GtkArg *arg, guint arg_id) {
  GtkPGCanvas *pgcanvas;
    
  pgcanvas = GTK_PGCANVAS(object);
}

static void gtk_pgcanvas_get_arg(GtkObject *object, GtkArg *arg, guint arg_id) {
  GtkPGCanvas *pgcanvas;
  
  pgcanvas = GTK_PGCANVAS (object);
}

static void gtk_pgcanvas_init(GtkPGCanvas *pgcanvas) {
  GTK_WIDGET_SET_FLAGS (pgcanvas, GTK_NO_WINDOW);
  
  pgcanvas->mycanvas = NULL;
}

GtkWidget* gtk_pgcanvas_new(PGCanvas* canvas) {
  GtkPGCanvas *pgcanvas;
  
  pgcanvas = (GtkPGCanvas*)gtk_type_new(GTK_TYPE_PGCANVAS);
  pgcanvas->mycanvas = canvas;
  
  return GTK_WIDGET(pgcanvas);
}

static void gtk_pgcanvas_finalize(GtkObject *object) {
  GtkPGCanvas *pgcanvas;
  
  g_return_if_fail (object != NULL);
  g_return_if_fail (GTK_IS_PGCANVAS(object));
  
  pgcanvas = GTK_PGCANVAS(object);
  
  GTK_OBJECT_CLASS(parent_class)->finalize(object);
}

static void gtk_pgcanvas_size_request(GtkWidget *widget, GtkRequisition *requisition) {
  GtkPGCanvas *pgcanvas;
  
  g_return_if_fail (GTK_IS_PGCANVAS (widget));
  g_return_if_fail (requisition != NULL);
  
  pgcanvas = GTK_PGCANVAS (widget);
  
  PGDimension dim;
  pgcanvas->mycanvas->getSize(&dim);  
  
  printf("Getsize: %d %d\n", dim.width, dim.height);
  
  requisition->width = 300;
  requisition->height = 300;
  
}

static void pg_gtk_pgcanvas_size_allocate(GtkWidget *widget, GtkAllocation *allocation) {
  PGRectangle rectangle;
  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_PGCANVAS (widget));
  g_return_if_fail (allocation != NULL);

  printf(">> PGCanvas set size %dx%d\n", allocation->x, allocation->y);
  widget->allocation = *allocation;
}

static void gtk_pgcanvas_style_set (GtkWidget *widget, GtkStyle *previous_style) {
  GtkPGCanvas *pgcanvas;

  g_return_if_fail (GTK_IS_PGCANVAS (widget));
  
  pgcanvas = GTK_PGCANVAS (widget);
}

static gint gtk_pgcanvas_expose(GtkWidget *widget, GdkEventExpose *event) {
  GtkPGCanvas *pgcanvas;
  GtkMisc *misc;
  gint x, y;
  
  g_return_val_if_fail(GTK_IS_PGCANVAS (widget), FALSE);
  g_return_val_if_fail(event != NULL, FALSE);
  
  pgcanvas = GTK_PGCANVAS(widget);
  
  printf(">>> TRY EXPOSE EVENT\n");  
  if (GTK_WIDGET_VISIBLE(widget) && GTK_WIDGET_MAPPED(widget) && pgcanvas->mycanvas != NULL) {
      misc = GTK_MISC (widget);
      
      printf(">>> EXPOSE EVENT\n");
      
      gdk_gc_set_clip_rectangle (widget->style->white_gc, &event->area);
      gdk_gc_set_clip_rectangle (widget->style->fg_gc[widget->state], &event->area);
      
      x = widget->allocation.x + (gint)misc->xpad;      
      y = widget->allocation.y + (gint)misc->ypad; 

      PGGraphics graph;
      graph.m_iOwnerMode = PGGRAPHICS_OWNER_SYSTEM;
      graph.m_hWidget = widget;
      graph.m_hWindow = widget->window;
      graph.realize();
	
      pgcanvas->mycanvas->paint(&graph);	
      graph.unrealize();

      gdk_gc_set_clip_mask (widget->style->white_gc, NULL);
      gdk_gc_set_clip_mask (widget->style->fg_gc[widget->state], NULL);
  }

  return TRUE;
}

