/***
 * Portable GUI
 * Code for PGFrame
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <portablegui.h>

gint PGOnDeleteEvent(GtkWidget *widget, GdkEvent *event, gpointer data);
void PGOnDestroyWindow(GtkWidget *widget, gpointer data);
gint PGOnExposeEvent(GtkWidget *widget, GdkEventExpose *event, gpointer data);

void PGDialog::init(PGWindow* parent, char *str, BOOL sizable) {
	// Remember parent
	m_hParent = parent;
	// Create main window
	m_hWidget = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(m_hWidget), str);
	if (sizable) gtk_window_set_policy(GTK_WINDOW(m_hWidget), FALSE, TRUE, FALSE);
	else gtk_window_set_policy(GTK_WINDOW(m_hWidget), FALSE, FALSE, TRUE);
	// Add event listeners
	gtk_signal_connect(GTK_OBJECT(m_hWidget), "destroy", GTK_SIGNAL_FUNC(PGOnDestroyWindow), (gpointer)this);
	gtk_signal_connect(GTK_OBJECT(m_hWidget), "delete_event", GTK_SIGNAL_FUNC(PGOnDeleteEvent), (gpointer)this);
	// Add main container
	m_hPGContainer = pg_gtk_layobox_new();
	pg_gtk_layobox_set_pgcontainer(PG_GTK_LAYOBOX(m_hPGContainer), this);
	gtk_container_add(GTK_CONTAINER(m_hWidget), m_hPGContainer);
	gtk_signal_connect(GTK_OBJECT(m_hPGContainer), "expose_event", GTK_SIGNAL_FUNC(PGOnExposeEvent), (gpointer)this);
	// Set modal
	gtk_window_set_modal(GTK_WINDOW(m_hWidget), TRUE);
	gtk_window_set_transient_for(GTK_WINDOW(m_hWidget), GTK_WINDOW(parent->m_hWidget));
	gtk_widget_realize(m_hWidget);	
}

void PGDialog::doModal() {
	setVisible(TRUE);
	gtk_main();
}

void PGDialog::notifyWindowListeners(int notify) {
	PGWindow::notifyWindowListeners(notify);
	if (notify == PGWINDOW_CLOSED) gtk_main_quit();
}

void PGDialog::pack() {
	PGWindow::pack();
}

