/***
 * Portable GUI
 * Code for PGFrame
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <portablegui.h>

void PGOnDestroyWindow(GtkWidget *widget, gpointer data);
gint PGOnDeleteEvent(GtkWidget *widget, GdkEvent *event, gpointer data);

#ifndef PGTINY
	gint PGOnExposeEvent(GtkWidget *widget, GdkEventExpose *event, gpointer data);
#endif

void PGWindow::dispose() {
	gtk_widget_destroy(m_hWidget);
}

void PGWindow::init() {
#ifndef PGTINY	
	m_hToolTips = NULL;	
#endif
}

void PGWindow::term() {
#ifndef PGTINY
	if (m_hToolTips != NULL) {
		// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!! DESTROY TOOLTIPS?
	}
#endif
}

void PGWindow::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	PGNormalContainer::getPreferredSize(dimension, recalc);
}

void PGWindow::getTitle(PGString* title) {
	title->setCopyValue(GTK_WINDOW(m_hWidget)->title);
}

void PGWindow::setTitle(const char* str) {
	gtk_window_set_title(GTK_WINDOW(m_hWidget), str);
}

#ifndef PGTINY
	void PGWindow::setIconImage(PGIcon* icon) {
		if (m_hIcon != NULL) PGDECREF(m_hIcon);
		m_hIcon = icon;
		PGINCREF(icon);
		PGImage* image = icon->m_hImage;
		gdk_window_set_icon(m_hWidget->window, NULL, image->m_hPixmap, image->m_hMask);
	}
	
	void PGWindow::createImage(int w, int h, PGImage* img) {
		img->reinitialize();	
		GdkPixmap* pixmap = gdk_pixmap_new(m_hWidget->window, w, h, -1);
		img->m_hPixmap = pixmap;
		img->m_iCols = w;
		img->m_iRows = h;
		img->m_hOwner = this;		
		img->m_iError = pixmap != NULL ? PGIMAGE_ERROR_NONE : PGIMAGE_ERROR_CREATE;
	}	
	
	BOOL PGWindow::initToolTips() {
		// Check current flags
		int flags = m_iToolTipsFlags;
		if (flags == PGWINDOW_TOOLTIPS_ENABLED) return TRUE;
 		if (flags == PGWINDOW_TOOLTIPS_DISABLED || flags == PGWINDOW_TOOLTIPS_ERROR) return FALSE;	
 		// Clear flags
 		m_hToolTips = NULL; 	
 		// Create tooltip control
		m_hToolTips = gtk_tooltips_new();
		gtk_tooltips_enable(m_hToolTips);		
        	if (m_hToolTips == NULL) {
        		m_iToolTipsFlags = PGWINDOW_TOOLTIPS_ERROR;
	        	return FALSE;
        	}	
 		// All ok
 		m_iToolTipsFlags = PGWINDOW_TOOLTIPS_ENABLED;
 		return TRUE;
	}	
#endif

void PGWindow::pack() {
	addToWindow(this);
}

void PGOnDestroyWindow(GtkWidget *widget, gpointer data) {
	PGWindow* myWindow = (PGWindow*)data;
	myWindow->notifyWindowListeners(PGWINDOW_CLOSED);
}

gint PGOnDeleteEvent(GtkWidget *widget, GdkEvent *event, gpointer data) {
	PGWindow* myWindow = (PGWindow*)data;
	myWindow->notifyWindowListeners(PGWINDOW_CLOSING);
	return TRUE;
}

#ifndef PGTINY
	gint PGOnExposeEvent(GtkWidget *widget, GdkEventExpose *event, gpointer data) {
		PGWindow* myWindow = (PGWindow*)data;
		PGGraphics graph;
		graph.m_hWidget = widget;
		graph.m_hDrawable = widget->window;
		graph.realize();
		myWindow->paint(&graph);
	        return FALSE;
	}
#endif
