/***
 * Portable GUI
 * Code for PGButton
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

void PGNormalComponentGetStringSize(PGNormalComponent* comp, PGString* str, PGDimension* size);

void PGButton::addToWindow(PGWindow* parent) {
	m_iId = getPGApplication()->m_vCommandList.registerCommand(this);
	DWORD dwStyle = WS_CHILD | WS_CLIPSIBLINGS | WS_TABSTOP | BS_PUSHBUTTON;
	m_hWnd = CreateWindow("BUTTON", m_sLabel.getValue(), dwStyle,
	                      CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
			      parent->m_hWnd, (HMENU)m_iId, getPGApplication()->m_hInst, NULL);
	PGAbstractButton::addToWindow(parent);
}

void PGButton::removeFromWindow(PGWindow* parent) {
	PGAbstractButton::removeFromWindow(parent);
	if (m_hWnd != NULL) DestroyWindow(m_hWnd);
	if (m_iId != -1) getPGApplication()->m_vCommandList.freeCommand(m_iId);
}

void PGButton::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	if (recalc) {
		PGDimension size;
		PGNormalComponentGetStringSize(this, &m_sLabel, &size);
		m_prevDim.width = size.width+10;
		m_prevDim.height = size.height+10;
	}
	(*dimension) = m_prevDim;
}

void PGCheckbox::addToWindow(PGWindow* parent) {
	m_iId = getPGApplication()->m_vCommandList.registerCommand(this);
	DWORD dwStyle = WS_CHILD | WS_CLIPSIBLINGS | WS_TABSTOP | BS_AUTOCHECKBOX;
	m_hWnd = CreateWindow("BUTTON", m_sLabel.getValue(), dwStyle,
	                      CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
			      parent->m_hWnd, (HMENU)m_iId, getPGApplication()->m_hInst, NULL);
	int check = m_bState ? 1 : 0;
	SendMessage(m_hWnd, BM_SETCHECK, (WPARAM)check, 0);	
	PGAbstractButton::addToWindow(parent);
}

void PGCheckbox::removeFromWindow(PGWindow* parent) {
	PGAbstractButton::removeFromWindow(parent);	
	if (m_hWnd != NULL) DestroyWindow(m_hWnd);
	if (m_iId != -1) getPGApplication()->m_vCommandList.freeCommand(m_iId);
}

void PGCheckbox::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	if (recalc) {
		PGDimension size;
		PGNormalComponentGetStringSize(this, &m_sLabel, &size);
		m_prevDim.width = size.width+40;
		m_prevDim.height = size.height+4;
	}
	(*dimension) = m_prevDim;
}

BOOL PGCheckbox::getState() {
	if (PGCOMPONENT_TEST_WINDOW(this)) return SendMessage(m_hWnd, BM_GETCHECK, 0, 0) == 1;
	else return m_bState;		
}

void PGCheckbox::setState(BOOL state) {	
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		int check = state ? 1 : 0;
		SendMessage(m_hWnd, BM_SETCHECK, (WPARAM)check, 0);	
	} else {
		m_bState = state;
	}		
}

void PGRadioButton::addToWindow(PGWindow* parent) {
	m_iId = getPGApplication()->m_vCommandList.registerCommand(this);
	DWORD dwStyle = WS_CHILD | WS_CLIPSIBLINGS | WS_TABSTOP | BS_RADIOBUTTON;
	m_hWnd = CreateWindow("BUTTON", m_sLabel.getValue(), dwStyle,
	                      CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
			      parent->m_hWnd, (HMENU)m_iId, getPGApplication()->m_hInst, NULL);
	int check = m_bState ? 1 : 0;
	SendMessage(m_hWnd, BM_SETCHECK, (WPARAM)check, 0);	
	PGAbstractButton::addToWindow(parent);
}

void PGRadioButton::removeFromWindow(PGWindow* parent) {
	PGAbstractButton::removeFromWindow(parent);	
	if (m_hWnd != NULL) DestroyWindow(m_hWnd);
	if (m_iId != -1) getPGApplication()->m_vCommandList.freeCommand(m_iId);
}

void PGRadioButton::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	if (recalc) {
		PGDimension size;
		if (m_sLabel.getLength() == 0) {
			m_prevDim.width = 20;
			m_prevDim.height = 18;			
		} else {
			PGNormalComponentGetStringSize(this, &m_sLabel, &size);
			m_prevDim.width = size.width+40;
			m_prevDim.height = size.height+4;
		}
	}
	(*dimension) = m_prevDim;
}

BOOL PGRadioButton::getState() {
	if (PGCOMPONENT_TEST_WINDOW(this)) return SendMessage(m_hWnd, BM_GETCHECK, 0, 0) == 1;
	else return m_bState;		
}

void PGRadioButton::setState(BOOL state) {	
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		int check = state ? 1 : 0;
		SendMessage(m_hWnd, BM_SETCHECK, (WPARAM)check, 0);	
	} else {
		m_bState = state;
	}		
}

void PGRadioButton::makeButton(PGWindow* parent, PGRadioButton* prev) {	
	// Not needed for win	
}	

boolean PGButtonGroup::shouldSelect() {
	return true;
}
