/***
 * Portable GUI
 * Code for PGButton
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

DWORD PGLabelMakeStyle(PGLabel* label);
void PGNormalComponentGetStringSize(PGNormalComponent* comp, PGString* str, PGDimension* size);

void PGLabel::addToWindow(PGWindow* parent) {
	DWORD dwStyle = PGLabelMakeStyle(this);
	m_hWnd = CreateWindow("STATIC", m_sLabel.getValue(), dwStyle,
	                      CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
			      parent->m_hWnd, (HMENU)-1, getPGApplication()->m_hInst, NULL);
	PGNormalComponent::addToWindow(parent);
}

void PGLabel::removeFromWindow(PGWindow* parent) {
	PGNormalComponent::removeFromWindow(parent);
	if (m_hWnd != NULL) DestroyWindow(m_hWnd);
}

void PGLabel::setText(const char* label) {
	m_sLabel.setCopyValue(label);	
	if (PGCOMPONENT_TEST_WINDOW(this))
		SetWindowText(m_hWnd, m_sLabel.getValue());	
}

void PGLabel::changeFlags(int or, int and) {
	PGNormalComponent::changeFlags(or, and);
	if (PGCOMPONENT_TEST_WINDOW(this))
		SetWindowLong(m_hWnd, GWL_STYLE, PGLabelMakeStyle(this));
}

void PGLabel::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	if (recalc) {
		PGDimension size;
		PGNormalComponentGetStringSize(this, &m_sLabel, &size);
		int sunken = m_iFlags & PGLABEL_SUNKEN != 0 ? 4 : 0;
		m_prevDim.width = size.width + 4 + sunken;
		m_prevDim.height = size.height + 1 + sunken;
	}
	(*dimension) = m_prevDim;
}

DWORD PGLabelMakeStyle(PGLabel* label) {
	DWORD dwStyle = WS_CHILD | WS_CLIPSIBLINGS | WS_TABSTOP;
	if ((label->m_iFlags & PGLABEL_LEFT) != 0) dwStyle |= SS_LEFT;
	if ((label->m_iFlags & PGLABEL_RIGHT) != 0) dwStyle |= SS_RIGHT;
	if ((label->m_iFlags & PGLABEL_SUNKEN) != 0) dwStyle |= SS_SUNKEN;
	return dwStyle;
}
