/***
 * Portable GUI
 * Code for PGMenu
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>
#include <pgcomponent/pgmenu.h>

// Menu item

void PGMenuItem::init(char* title, char* status) {
	m_sItem.setCopyValue(title);
	m_sStatus.setCopyValue(status);
	m_iId = getPGApplication()->m_vCommandList.registerCommand(this);
}

PGMenuItem::~PGMenuItem() {
	if (m_iId != -1) getPGApplication()->m_vCommandList.freeCommand(m_iId);
}

void PGMenuItem::addToMenu(PGMenu* menu, int idx) {
	InsertMenu(menu->m_hMenu, idx, MF_BYPOSITION | MF_STRING, (UINT)m_iId, m_sItem.getValue());
}

void PGMenuItem::removeFromMenu(PGMenu* menu) {
}

// Separator

PGMenuSeparator::PGMenuSeparator() {
}

PGMenuSeparator::~PGMenuSeparator() {
}

void PGMenuSeparator::addToMenu(PGMenu* menu, int idx) {
	InsertMenu(menu->m_hMenu, idx, MF_BYPOSITION | MF_SEPARATOR, 0, NULL);
}

void PGMenuSeparator::removeFromMenu(PGMenu* menu) {
}

// Menu

PGMenu::PGMenu(char* title) {
	m_sItem.setCopyValue(title);
	m_hMenu = CreatePopupMenu();
}

PGMenu::~PGMenu() {
	if (m_hMenu != NULL) DestroyMenu(m_hMenu);
}

void PGMenu::addToMenuBar(PGMenuBar* menu, int idx) {
	InsertMenu(menu->m_hMenu, idx, MF_BYPOSITION | MF_STRING | MF_POPUP, (UINT)m_hMenu, m_sItem.getValue());
}

void PGMenu::removeFromMenuBar(PGMenuBar* menu) {
	
}

void PGMenu::addToMenu(PGMenu* menu, int idx) {
	InsertMenu(menu->m_hMenu, idx, MF_BYPOSITION | MF_POPUP | MF_STRING, (UINT)m_hMenu, m_sItem.getValue());
}

void PGMenu::removeFromMenu(PGMenu* menu) {

}

// Menu bar

PGMenuBar::PGMenuBar() {
	m_hMenu = CreateMenu();
}

PGMenuBar::~PGMenuBar() {
	if (m_hMenu != NULL) DestroyMenu(m_hMenu);
}

void PGMenuBar::addToFrame(PGFrame* frame) {
	SetMenu(frame->m_hWnd, m_hMenu);	
	for (int ctr = 0; ctr < m_vMenus.getSize(); ctr++) {
		PGMenu* item = (PGMenu*)m_vMenus.getElementAt(ctr);
		item->setFrame(frame);
	}
}

void PGMenuBar::removeFromFrame(PGFrame* frame) {
	SetMenu(frame->m_hWnd, NULL);
}

