/***
 * Portable GUI
 * Code for PGNormalComponent
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>
#include <commctrl.h>

void PGNormalComponentSetStyle(PGNormalComponent* comp);
void PGNormalComponentSetTooltip(PGNormalComponent* comp);
void PGNormalComponentGetStringSize(PGNormalComponent* comp, PGString* str, PGDimension* size);

PGNormalComponent::PGNormalComponent() {
	m_hWnd = NULL;
	m_iId = -1;
}

void PGNormalComponent::setVisible(BOOL visible) {
	PGComponent::setVisible(visible);
	ShowWindow(m_hWnd, visible ? SW_SHOW : SW_HIDE);
}

void PGNormalComponent::setBounds(int x, int y, int w, int h) {
#ifdef DEBUG_SIZE	
	char s[100];
	PGString name;
	toString(&name);
	sprintf(s,"Setting bounds %d:%d:%d:%d for %s", x, y, w, h, name.getValue());
	getPGApplication()->println(s);
#endif	
	MoveWindow(m_hWnd, x, y, w, h, TRUE);
}

void PGNormalComponent::getBounds(PGRectangle* rectangle) {
	RECT rect;
	GetWindowRect(m_hWnd, &rect);
	rectangle->x = rect.left;
	rectangle->y = rect.top;
	rectangle->width = rect.right-rect.left;
	rectangle->height = rect.bottom-rect.top;
}

void PGNormalComponent::addToWindow(PGWindow* parent) {
	SetWindowLong(m_hWnd, GWL_USERDATA, (LONG)this);
	if (m_hStyle != NULL) PGNormalComponentSetStyle(this);
	if (m_sToolTip.getLength() != 0) PGNormalComponentSetTooltip(this);
	EnableWindow(m_hWnd, (m_iFlags & PGCOMPONENT_DISABLED) == 0);
	PGComponent::addToWindow(parent);
	
	// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! remove from tooltips?	
}

void PGNormalComponent::setStyle(PGStyle* style) {
	PGComponent::setStyle(style);
	if (PGCOMPONENT_TEST_WINDOW(this)) PGNormalComponentSetStyle(this);
}

void PGNormalComponent::changeFlags(int orval, int andval) {
	BOOL prevena = (m_iFlags & PGCOMPONENT_DISABLED) == 0;
	PGComponent::changeFlags(orval, andval);
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		BOOL newena = (m_iFlags & PGCOMPONENT_DISABLED) == 0;
		if (prevena != newena) EnableWindow(m_hWnd, newena);
	}
}

void PGNormalComponent::getGraphics(PGGraphics* graph) {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		graph->unrealize();
		graph->m_hWnd = m_hWnd;
		graph->m_hDC = GetDC(m_hWnd);
		graph->realize();
	}
}

void PGNormalComponent::requestFocus() {
	if (PGCOMPONENT_TEST_WINDOW(this)) SetFocus(m_hWnd);
}

void PGNormalComponent::repaint() {
	InvalidateRect(m_hWnd, NULL, FALSE);
}

void PGNormalComponent::repaint(PGRectangle* r) {
	if (m_hWnd != NULL) {
		RECT rect;
		rect.left = r->x;
		rect.top = r->y;
		rect.bottom = r->y+r->height;
		rect.right = r->x+r->width;
		InvalidateRect(m_hWnd, &rect, FALSE);
	}
}

void PGNormalComponent::createImage(int w, int h, PGImage* img) {
	PGWindow* win = getWindow();
	if (win != NULL) {
		win->createImage(w, h, img);	
	} else {
		img->m_iError = PGIMAGE_ERROR_CREATE;
	}
}

void PGNormalComponentSetStyle(PGNormalComponent* comp) {
	PGWindow* win = comp->getWindow();
	if (win != NULL) {
		// Set font
		PGRealizedFont* realfont = win->getRealizedFont(comp->m_hStyle->m_hFont);
		SendMessage(comp->m_hWnd, WM_SETFONT, (WPARAM)realfont->m_hWFont, MAKELPARAM(TRUE, 0));
	}
}

void PGNormalComponentSetTooltip(PGNormalComponent* comp) {
	PGWindow* win = comp->getWindow();
	if (win != NULL && win->initToolTips()) {
		TOOLINFO ti; 
    		ti.cbSize = sizeof(TOOLINFO); 
		ti.uFlags = TTF_IDISHWND; 
		ti.hwnd = win->m_hWnd; 
		ti.uId = (UINT)comp->m_hWnd; 
		ti.hinst = 0; 
		ti.lpszText = (char*)comp->m_sToolTip.getValue(); 
		// Add to tooltip control
		SendMessage(win->m_hWndToolTip, TTM_ADDTOOL, 0, (LPARAM) (LPTOOLINFO) &ti); 
		// Tooltip
		comp->changeFlags(PGCOMPONENT_TOOLTIP, PGCOMPONENT_ALLFLAGS);
	}
}

void PGNormalComponentGetStringSize(PGNormalComponent* comp, PGString* str, PGDimension* size) {
	PGGraphics graph;
	PGFontMetrics fm;
	comp->getGraphics(&graph);	
	if (comp->m_hStyle != NULL) {
		PGFont* font = comp->m_hStyle->m_hFont;
		if (font != NULL) graph.setFont(comp, font);		
	}
	graph.getFontMetrics(&fm);
	fm.getStringSize(str, size);
}
