/***
 * Portable GUI
 * Code for PGTextArea
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

DWORD PGTextAreaMakeStyle(PGTextArea *area);

void PGTextArea::addToWindow(PGWindow* parent) {
	DWORD dwStyle = PGTextAreaMakeStyle(this);	
	m_hWnd = CreateWindow("EDIT", m_sText.getValue(), dwStyle,
				CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
				parent->m_hWnd, NULL, getPGApplication()->m_hInst, NULL);	
	PGTextComponent::addToWindow(parent);
}

void PGTextArea::removeFromWindow(PGWindow* parent) {
	PGTextComponent::removeFromWindow(parent);
	if (m_hWnd != NULL) DestroyWindow(m_hWnd);
}

void PGTextArea::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	if (recalc) {
		PGGraphics graph;
		PGFontMetrics fm;
		getGraphics(&graph);			
		if (m_hStyle != NULL) {
			PGFont* font = m_hStyle->m_hFont;
			if (font != NULL) graph.setFont(this, font);		
		}		
		graph.getFontMetrics(&fm);
		int sw = GetSystemMetrics(SM_CXHSCROLL);
		m_prevDim.width = m_iCols*fm.getMaxAdvance() + 4 + sw;
		m_prevDim.height = m_iRows*fm.getHeight(); + 4;
	}
	(*dimension) = m_prevDim;
}

// Gets the size of the text in this area
int PGTextArea::getSize() {
	return PGTextComponent::getSize();
}

// Gets the position of the text insertion caret
int PGTextArea::getCaretPosition() {
	return PGTextComponent::getCaretPosition();
}

// Sets the position of the text insertion caret
void PGTextArea::setCaretPosition(int position) {
	PGTextComponent::setCaretPosition(position);
}

// Sets the text that is presented by this text component
void PGTextArea::setText(const char* str) {
	PGTextComponent::setText(str);
}

// Update component on every change?
void PGTextArea::setDirectUpdate(BOOL directupdate) {
}

// Appends the given text to the text area's current text.
void PGTextArea::append(const char* str, BOOL addNewLine) {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		int len = getSize();
		SendMessage(m_hWnd, EM_SETSEL, len, len);
		SendMessage(m_hWnd, EM_REPLACESEL, FALSE, (LPARAM)str);
		if (addNewLine)
			SendMessage(m_hWnd, EM_REPLACESEL, FALSE, (LPARAM)PGSTRING_NEWLINE);
	} else {
		m_sText.append(str, addNewLine);
	}
}

// Inserts the specified text at the specified position in this text area.
void PGTextArea::insert(const char* str, int pos) {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		SendMessage(m_hWnd, EM_SETSEL, pos, pos);
		SendMessage(m_hWnd, EM_REPLACESEL, FALSE, (LPARAM)str);
	} else {
		m_sText.insert(str, pos);
	}
}

// Replaces text between the indicated start and end positions with the specified replacement.
void PGTextArea::replaceRange(const char* str, int start, int end) {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		SendMessage(m_hWnd, EM_SETSEL, start, end);
		SendMessage(m_hWnd, EM_REPLACESEL, FALSE, (LPARAM)str);
	} else {
		m_sText.remove(start, end);
		m_sText.insert(str, start);
	}
}

DWORD PGTextAreaMakeStyle(PGTextArea *area) {
	DWORD dwStyle = WS_CHILD | WS_CLIPSIBLINGS | WS_TABSTOP | ES_MULTILINE | WS_BORDER;	
	if ((area->m_iScrollBar & PGTEXTAREA_SCROLLBARS_HORIZONTAL_ONLY) != 0) dwStyle |= WS_HSCROLL;
	if ((area->m_iScrollBar & PGTEXTAREA_SCROLLBARS_VERTICAL_ONLY) != 0) dwStyle |= WS_VSCROLL;
	if (!area->isEditable()) dwStyle |= ES_READONLY;
	return dwStyle;
}
