/***
 * Portable GUI
 * Code for PGTextArea
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

DWORD PGTextFieldMakeStyle(PGTextField *field);

// Adds the specified text event listener
//void addTextListener(TextListener l);

// Gets the size of the text in this area
int PGTextComponent::getSize() {
	if (PGCOMPONENT_TEST_WINDOW(this)) return SendMessage(m_hWnd, WM_GETTEXTLENGTH, 0, 0);
	else return m_sText.getLength();
}

// Gets the position of the text insertion caret
int PGTextComponent::getCaretPosition() {
	DWORD start = 0, end = 0;
	if (PGCOMPONENT_TEST_WINDOW(this)) SendMessage(m_hWnd, EM_GETSEL, (WPARAM)&start, (LPARAM)&end);
	return start;
}

// Gets the start/end position of the selected text
void PGTextComponent::getSelection(int* start, int* end) {
	DWORD dwStart = 0, dwEnd = 0;
	if (PGCOMPONENT_TEST_WINDOW(this)) SendMessage(m_hWnd, EM_GETSEL, (WPARAM)&dwStart, (LPARAM)&dwEnd);
	(*start) = (int)dwStart;
	(*end) = (int)dwEnd;
}

// Gets the text
void PGTextComponent::getText(PGString* str, int start, int end) {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		int len = end-start+1;
		str->setSize(len);
		GetWindowText(m_hWnd, str->getBuffer(), len+1);
		str->substring(start, end);
	} else {
		str->setCopyValue(m_sText.getValue());
		str->substring(start, end);
	}
}

// Selects the text between the specified start and end positions
void PGTextComponent::select(int selectionStart, int selectionEnd) {
	if (PGCOMPONENT_TEST_WINDOW(this)) SendMessage(m_hWnd, EM_SETSEL, selectionStart, selectionEnd);
}

// Sets the position of the text insertion caret
void PGTextComponent::setCaretPosition(int position) {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		SendMessage(m_hWnd, EM_SETSEL, position, position);
	}
}

// Sets the text that is presented by this text component
void PGTextComponent::setText(const char* str) {
	if (PGCOMPONENT_TEST_WINDOW(this)) {
		SetWindowText(m_hWnd, str);
	} else {
		m_sText.setCopyValue(str);
	}
}

// Undo last edit operation
void PGTextComponent::undoOperation() {
	if (PGCOMPONENT_TEST_WINDOW(this)) SendMessage(m_hWnd, EM_UNDO, 0, 0);
}

// Copy to clipboard
void PGTextComponent::copy() {
	if (PGCOMPONENT_TEST_WINDOW(this)) SendMessage(m_hWnd, WM_COPY, 0, 0);
}

// Cut to clipboard
void PGTextComponent::cut() {
	if (PGCOMPONENT_TEST_WINDOW(this)) SendMessage(m_hWnd, WM_CUT, 0, 0);
}

// Paste from clipboard
void PGTextComponent::paste() {
	if (PGCOMPONENT_TEST_WINDOW(this)) SendMessage(m_hWnd, WM_PASTE, 0, 0);
}

void PGTextField::addToWindow(PGWindow* parent) {
	DWORD dwStyle = PGTextFieldMakeStyle(this);	
	m_hWnd = CreateWindow("EDIT", m_sText.getValue(), dwStyle,
				CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
				parent->m_hWnd, NULL, getPGApplication()->m_hInst, NULL);
	PGTextComponent::addToWindow(parent);
}

void PGTextField::removeFromWindow(PGWindow* parent) {
	PGTextComponent::removeFromWindow(parent);
	if (m_hWnd != NULL) DestroyWindow(m_hWnd);
}

void PGTextField::changeFlags(int or, int and) {
	PGNormalComponent::changeFlags(or, and);
	if (PGCOMPONENT_TEST_WINDOW(this))
		SetWindowLong(m_hWnd, GWL_STYLE, PGTextFieldMakeStyle(this));
}

void PGTextField::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	if (recalc) {
		PGGraphics graph;
		PGFontMetrics fm;
		getGraphics(&graph);			
		if (m_hStyle != NULL) {
			PGFont* font = m_hStyle->m_hFont;
			if (font != NULL) graph.setFont(this, font);		
		}		
		graph.getFontMetrics(&fm);		
		m_prevDim.width = m_iCols*fm.getMaxAdvance() + 4;
		m_prevDim.height = fm.getHeight() + 7;
	}
	(*dimension) = m_prevDim;
}

// Gets the size of the text in this area
int PGTextField::getSize() {
	return PGTextComponent::getSize();
}

// Gets the position of the text insertion caret
int PGTextField::getCaretPosition() {
	return PGTextComponent::getCaretPosition();
}

// Sets the position of the text insertion caret
void PGTextField::setCaretPosition(int position) {
	PGTextComponent::setCaretPosition(position);
}

// Sets the text that is presented by this text component
void PGTextField::setText(const char* str) {
	PGTextComponent::setText(str);
}

DWORD PGTextFieldMakeStyle(PGTextField *field) {
	DWORD dwStyle = WS_CHILD | WS_CLIPSIBLINGS | WS_TABSTOP | WS_BORDER | ES_AUTOHSCROLL;
	if ((field->m_iFlags & PGTEXTCOMPONENT_EDITABLE) == 0) dwStyle |= ES_READONLY;
	//if ((field->m_iFlags & PGTEXTCOMPONENT_NUMBER) != 0) dwStyle |= ES_READONLY;
	if ((field->m_iFlags & PGTEXTCOMPONENT_UPPERCASE) != 0) dwStyle |= ES_UPPERCASE;
	if ((field->m_iFlags & PGTEXTCOMPONENT_PASSWORD) != 0) dwStyle |= ES_PASSWORD;
	return dwStyle;
}
