/***
 * Portable GUI
 * Code for PGButton
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>

#define PGGRAPHICS_HOLLOWBRUSH  0
#define PGGRAPHICS_CURRENTBRUSH 1
#define PGGRAPHICS_NOBRUSH	2

void PGGraphicsSelectBrush(PGGraphics* graph, HBRUSH brush) {
	if (graph->m_hOldBrush == NULL) graph->m_hOldBrush = (HBRUSH)SelectObject(graph->m_hDC, brush);
	else SelectObject(graph->m_hDC, brush);
}

void PGGraphicsMakeBrush(PGGraphics* graph) {
	if (graph->m_iCurrentBrush == PGGRAPHICS_CURRENTBRUSH)
		SelectObject(graph->m_hDC, graph->m_hOldBrush);
	if (graph->m_hBrush != NULL)
		DeleteObject(graph->m_hBrush);
	COLORREF ref = (COLORREF)graph->m_Color.getBGR();
	graph->m_hBrush = CreateSolidBrush(ref);
	graph->m_bBrushChanged = FALSE;
}

void PGGraphicsChooseBrush(PGGraphics* graph, int type) {
	if (type == PGGRAPHICS_HOLLOWBRUSH) {
		if (graph->m_iCurrentBrush == PGGRAPHICS_HOLLOWBRUSH) return;
		PGGraphicsSelectBrush(graph, (HBRUSH)GetStockObject(HOLLOW_BRUSH));
		graph->m_iCurrentBrush = PGGRAPHICS_HOLLOWBRUSH;
	} else {
		if (graph->m_bBrushChanged) {
			COLORREF ref = (COLORREF)graph->m_Color.getBGR();
			HBRUSH newBrush = CreateSolidBrush(ref);
			PGGraphicsSelectBrush(graph, newBrush);
			if (graph->m_hBrush != NULL) DeleteObject(graph->m_hBrush);
			graph->m_hBrush = newBrush;
			graph->m_bBrushChanged = FALSE;
			graph->m_iCurrentBrush = PGGRAPHICS_CURRENTBRUSH;
		} else {
			if (graph->m_iCurrentBrush == PGGRAPHICS_CURRENTBRUSH) return;
			PGGraphicsSelectBrush(graph, graph->m_hBrush);
			graph->m_iCurrentBrush = PGGRAPHICS_CURRENTBRUSH;
		}
	}
}

void PGGraphicsSelectPen(PGGraphics* graph, HPEN pen) {
	if (graph->m_hOldPen == NULL) graph->m_hOldPen = (HPEN)SelectObject(graph->m_hDC, pen);
	else SelectObject(graph->m_hDC, pen);
}

void PGGraphicsUpdatePen(PGGraphics* graph) {
	COLORREF ref = (COLORREF)graph->m_Color.getBGR();
	HPEN newPen = CreatePen(PS_SOLID, graph->m_iLineWidth, ref);
	PGGraphicsSelectPen(graph, newPen);
	if (graph->m_hPen != NULL) DeleteObject(graph->m_hPen);
	graph->m_bPenChanged = FALSE;
	graph->m_hPen = newPen;
}

void PGGraphicsTextOut(PGGraphics* g, LPCSTR str, int len, int x, int y) {
	if (g->m_bTextChanged) {
		SetBkMode(g->m_hDC, TRANSPARENT);
		COLORREF ref = (COLORREF)g->m_Color.getBGR();
		SetTextColor(g->m_hDC, ref);
		g->m_bTextChanged = FALSE;
	}
	TextOut(g->m_hDC, x, y, str, len);
}


void PGGraphics::realize() {
}

// PGGRAPHICS_OWNER_COMPONENT
// PGGRAPHICS_OWNER_BITMAP
// PGGRAPHICS_OWNER_SYSTEM

void PGGraphics::init() {
	// Font
	m_hOldFont = (HFONT)NULL;
	// DC
	m_hDC = (HDC)NULL;
	m_hWnd = (HWND)NULL;
	m_iOwnerMode = PGGRAPHICS_OWNER_COMPONENT;
	// Brush
	m_hBrush = (HBRUSH)NULL;
	m_hOldBrush = (HBRUSH)NULL;
	m_bBrushChanged = TRUE;
	m_iCurrentBrush = PGGRAPHICS_NOBRUSH;
	// Pen
	m_hPen = (HPEN)NULL;
	m_hOldPen = (HPEN)NULL;
	m_bPenChanged = TRUE;
	// Clip
	m_hClipRgn = (HRGN)NULL;
	// Text
	m_bTextChanged = TRUE;
}

// Warning !!!
// ReleaseDC in combination with EndPaint!!

void PGGraphics::unrealize() {
	// Font
	if (m_hOldFont != NULL) {
		SelectObject(m_hDC, m_hOldFont);
		m_hOldFont = (HFONT)NULL;
	}
	// Brush
	if (m_hOldBrush != NULL) {
		SelectObject(m_hDC, m_hOldBrush);
		m_hOldBrush = NULL;
	}
	if (m_hBrush != NULL) {
		DeleteObject(m_hBrush);
		m_hBrush = NULL;
	}
	// Pen
	if (m_hOldPen != NULL) {
		SelectObject(m_hDC, m_hOldPen);
		m_hOldPen = NULL;
	}
	if (m_hPen != NULL) {
		DeleteObject(m_hPen);
		m_hPen = NULL;
	}
	// Rgn
	if (m_hClipRgn != NULL) {
		DeleteObject(m_hClipRgn);
		m_hClipRgn = NULL;
	}
	// DC
	if (m_hDC != NULL) {
		if (m_iOwnerMode == PGGRAPHICS_OWNER_COMPONENT)	ReleaseDC(m_hWnd, m_hDC);
		if (m_iOwnerMode == PGGRAPHICS_OWNER_BITMAP) DeleteDC(m_hDC);
		m_hDC = (HDC)NULL;
	}
}

// Clears the specified rectangle by filling it with the background color.
void PGGraphics::clearRect(int x, int y, int width, int height) {
	RECT rect;
	rect.left = x;
	rect.top = y;
	rect.right = x+width;
	rect.bottom = y+height;
	FillRect(m_hDC, &rect, getPGApplication()->m_hBkBrush);
}

// Intersects the current clip with the specified rectangle.
void PGGraphics::clipRect(int x, int y, int width, int height) {
	if (m_hClipRgn != NULL) DeleteObject(m_hClipRgn);
	m_hClipRgn = CreateRectRgn(x, y, x+width, y+height);
	ExtSelectClipRgn(m_hDC, m_hClipRgn, RGN_AND);
}

// Copies an area of the component by a distance specified by dx and dy.
void PGGraphics::copyArea(int x, int y, int width, int height, int dx, int dy) {
	BitBlt(m_hDC, x+dx, y+dy, width, height, m_hDC, x, y, SRCCOPY);
}

void PGGraphicsArc(HDC hDC, int x, int y, int width, int height, int startAngle, int arcAngle, BOOL filled) {
	int right = x+width;
	int bottom = y+height;
	int endAngle = startAngle+arcAngle;
	float arc1 = ((float)startAngle)*(float)PGMATH_PI/180;
	float arc2 = ((float)endAngle)*(float)PGMATH_PI/180;
	float dx1 = ((float)width/2)*PGMathCos(arc1);
	float dy1 = ((float)height/2)*PGMathSin(arc1);
	float dx2 = ((float)width/2)*PGMathCos(arc2);
	float dy2 = ((float)height/2)*PGMathSin(arc2);
	int x1 = x+width/2+(int)dx1;
	int y1 = y+height/2-(int)dy1;
	int x2 = x+width/2+(int)dx2;
	int y2 = y+height/2-(int)dy2;
	if (filled) Pie(hDC, x, y, right, bottom, x1, y1, x2, y2);
	else Arc(hDC, x, y, right, bottom, x1, y1, x2, y2);
}

// Draws the outline of a circular or elliptical arc covering the specified rectangle.
void PGGraphics::drawArc(int x, int y, int width, int height, int startAngle, int arcAngle) {
	if (m_bPenChanged) PGGraphicsUpdatePen(this);
	PGGraphicsArc(m_hDC, x, y, width, height, startAngle, arcAngle, FALSE);
}

// Draws a part of a given image.
void PGGraphics::drawImage(PGImage* image, int dx, int dy, int sx, int sy, int width, int height) {
	BOOL result = TRUE;
	DWORD mode = SRCCOPY;
	HDC hdcBits;
	HGDIOBJ hObj;
	if (image->hasGraphics()) {
		if ((image->m_hMask != NULL) && (hdcBits = CreateCompatibleDC(m_hDC)) != NULL) {
			hObj = (HGDIOBJ)SelectObject(hdcBits, image->m_hMask);
			if ((hObj != NULL) && (hObj != (HGDIOBJ)GDI_ERROR)) {
	         		BitBlt(m_hDC, dx, dy, width, height, hdcBits, sx, sy, SRCAND);
	      		} else {
		               	result = FALSE;
		      	}
		      	DeleteDC(hdcBits);
		}
		PGGraphics* graph = image->getGraphics();
		BitBlt(m_hDC, dx, dy, width, height, graph->m_hDC, sx, sy, mode);
	} else {
		if(hdcBits = CreateCompatibleDC(m_hDC)) {
			if (image->m_hMask != NULL) {
				hObj = (HGDIOBJ)SelectObject(hdcBits, image->m_hMask);
				if ((hObj != NULL) && (hObj != (HGDIOBJ)GDI_ERROR)) {
	         			BitBlt(m_hDC, dx, dy, width, height, hdcBits, sx, sy, SRCAND);
	      			} else {
		                  	result = FALSE;
		      		}
	      			mode = SRCPAINT;
	      		}
	      		hObj = (HGDIOBJ)SelectObject(hdcBits, image->m_hBitmap);
			if ((hObj != NULL) && (hObj != (HGDIOBJ)GDI_ERROR)) {
		         	BitBlt(m_hDC, dx, dy, width, height, hdcBits, sx, sy, mode);
		      	} else {
	        	       	result = FALSE;
	      		}
	       		DeleteDC(hdcBits);
		}
	}
}

// Draws the specified image, scaling it on the fly.
// FIXME: Not implemented yet
void PGGraphics::drawImage(PGImage* img, int dx1, int dy1, int dx2, int dy2, int sx1, int sy1, int sx2, int sy2) {
}

// Draws a line, using the current color, between the points (x1, y1) and (x2, y2)
void PGGraphics::drawLine(int x1, int y1, int x2, int y2) {
	if (m_bPenChanged) PGGraphicsUpdatePen(this);
	MoveToEx(m_hDC, x1, y1, NULL);
	LineTo(m_hDC, x2, y2);
}

// Draws the outline of an oval.
void PGGraphics::drawOval(int x, int y, int width, int height) {
	PGGraphicsChooseBrush(this, PGGRAPHICS_HOLLOWBRUSH);
	if (m_bPenChanged) PGGraphicsUpdatePen(this);
	Arc(m_hDC, x, y, x+width, y+height, 0, 0, 0, 0);
}

POINT* PGGraphicsMakePointArray(int* xPoints, int* yPoints, int nPoints) {
	POINT* points = new POINT[nPoints];
	for (int ctr = 0; ctr < nPoints; ctr++) {
		points[ctr].x = xPoints[ctr];
		points[ctr].y = yPoints[ctr];
	}
	return points;
}

// Draws a closed polygon defined by arrays of x and y coordinates.
void PGGraphics::drawPolygon(int* xPoints, int* yPoints, int nPoints) {
	PGGraphicsChooseBrush(this, PGGRAPHICS_HOLLOWBRUSH);
	if (m_bPenChanged) PGGraphicsUpdatePen(this);
	POINT* points = PGGraphicsMakePointArray(xPoints, yPoints, nPoints);
	Polygon(m_hDC, points, nPoints);
	delete points;
}

#ifdef PGSHAPE
	// Draws the outline of a polygon defined by the specified Polygon object.
	void PGGraphics::drawPolygon(PGPolygon* p) {
		PGGraphicsChooseBrush(this, PGGRAPHICS_HOLLOWBRUSH);
		if (m_bPenChanged) PGGraphicsUpdatePen(this);
		p->realize();
		Polygon(m_hDC, p->m_hWinPoints, p->getNbPoints());
	}

	// Fills the polygon defined by the specified Polygon object.
	void PGGraphics::fillPolygon(PGPolygon* p) {
		PGGraphicsChooseBrush(this, PGGRAPHICS_CURRENTBRUSH);
		if (m_bPenChanged) PGGraphicsUpdatePen(this);
		p->realize();
		Polygon(m_hDC, p->m_hWinPoints, p->getNbPoints());
	}
#endif

// Draws a sequence of connected lines defined by arrays of x and y coordinates.
void PGGraphics::drawPolyline(int* xPoints, int* yPoints, int nPoints) {
	PGGraphicsChooseBrush(this, PGGRAPHICS_HOLLOWBRUSH);
	if (m_bPenChanged) PGGraphicsUpdatePen(this);
	POINT* points = PGGraphicsMakePointArray(xPoints, yPoints, nPoints);
	Polyline(m_hDC, points, nPoints);
	delete points;
}

// Draws the outline of the specified rectangle.
void PGGraphics::drawRect(int x, int y, int width, int height) {
	PGGraphicsChooseBrush(this, PGGRAPHICS_HOLLOWBRUSH);
	if (m_bPenChanged) PGGraphicsUpdatePen(this);
	Rectangle(m_hDC, x, y, x+width, y+height);
}

// Draws an outlined round-cornered rectangle using this graphics context's current color.
void PGGraphics::drawRoundRect(int x, int y, int width, int height, int arcWidth, int arcHeight) {
	PGGraphicsChooseBrush(this, PGGRAPHICS_HOLLOWBRUSH);
	if (m_bPenChanged) PGGraphicsUpdatePen(this);
	RoundRect(m_hDC, x, y, x+width, y+height, arcWidth, arcHeight);
}

// Draws the text given by the specified string, using this graphics context's current font and color.
void PGGraphics::drawString(const char *str, int x, int y) {
	PGGraphicsTextOut(this, (LPCSTR)str, strlen(str), x, y);
}

// Draws the text given by the specified string, using this graphics context's current font and color.
void PGGraphics::drawString(PGString *str, int x, int y) {
	PGGraphicsTextOut(this, (LPCSTR)str->getValue(), str->getLength(), x, y);
}

// Paints a 3-D highlighted rectangle filled with the current color.
void PGGraphics::fill3DRect(int x, int y, int width, int height, BOOL raised) {
}

// Fills a circular or elliptical arc covering the specified rectangle.
void PGGraphics::fillArc(int x, int y, int width, int height, int startAngle, int arcAngle) {
	PGGraphicsChooseBrush(this, PGGRAPHICS_CURRENTBRUSH);
	if (m_bPenChanged) PGGraphicsUpdatePen(this);
	PGGraphicsArc(m_hDC, x, y, width, height, startAngle, arcAngle, TRUE);
}

// Fills an oval bounded by the specified rectangle with the current color.
void PGGraphics::fillOval(int x, int y, int width, int height) {
	PGGraphicsChooseBrush(this, PGGRAPHICS_CURRENTBRUSH);
	if (m_bPenChanged) PGGraphicsUpdatePen(this);
	Arc(m_hDC, x, y, x+width, y+height, 0, 0, 0, 0);
}

// Fills a closed polygon defined by arrays of x and y coordinates.
void PGGraphics::fillPolygon(int* xPoints, int* yPoints, int nPoints) {
	PGGraphicsChooseBrush(this, PGGRAPHICS_CURRENTBRUSH);
	if (m_bPenChanged) PGGraphicsUpdatePen(this);
	POINT* points = PGGraphicsMakePointArray(xPoints, yPoints, nPoints);
	Polygon(m_hDC, points, nPoints);
	delete points;
}

// Fills the specified rectangle.
void PGGraphics::fillRect(int x, int y, int width, int height) {
	if (m_bBrushChanged)
		PGGraphicsMakeBrush(this);
	RECT rect;
	rect.left = x;
	rect.top = y;
	rect.right = x+width;
	rect.bottom = y+height;
	FillRect(m_hDC, &rect, m_hBrush);
}

// Fills the specified rounded corner rectangle with the current color.
void PGGraphics::fillRoundRect(int x, int y, int width, int height, int arcWidth, int arcHeight) {
	PGGraphicsChooseBrush(this, PGGRAPHICS_CURRENTBRUSH);
	if (m_bPenChanged) PGGraphicsUpdatePen(this);
	RoundRect(m_hDC, x, y, x+width, y+height, arcWidth, arcHeight);
}

// Gets the current clipping area.
// Shape getClip();

// Returns the bounding rectangle of the current clipping area.
void PGGraphics::getClipBounds(PGRectangle* bounds) {
	RECT rect;
	GetClipBox(m_hDC, &rect);
	bounds->x = rect.left;
	bounds->y = rect.top;
	bounds->width = rect.right - rect.left;
	bounds->height = rect.bottom - rect.top;
}

// Gets this graphics context's current color.
PGColor* PGGraphics::getColor() {
	return &m_Color;
}

// Returns true if the specified rectangular area intersects the bounding rectangle of the current clipping area.
BOOL PGGraphics::hitClip(int x, int y, int width, int height) {
	return TRUE;
}

// Sets the current clip to the rectangle specified by the given coordinates.
void PGGraphics::setClip(int x, int y, int width, int height) {
	if (m_hClipRgn != NULL) DeleteObject(m_hClipRgn);
	m_hClipRgn = CreateRectRgn(x, y, x+width, y+height);
	SelectClipRgn(m_hDC, m_hClipRgn);
}

// Sets this graphics context's current color to the specified color.
void PGGraphics::setColor(PGColor* c) {
	if (!m_Color.equals(c)) {
		m_Color.setCopyValue(c);
		m_bPenChanged = TRUE;
		m_bBrushChanged = TRUE;
		m_bTextChanged = TRUE;
	}
}

// Set the width for drawing lines
void PGGraphics::setLineWidth(int wd) {
	if (wd != m_iLineWidth) {
		m_iLineWidth = wd;
		m_bPenChanged = TRUE;
	}
}

// Sets this graphics context's font
void PGGraphics::setFont(PGRealizedFont* font) {
	HFONT hOldFont = (HFONT)SelectObject(m_hDC, font->m_hWFont);
	if (m_hOldFont == NULL) m_hOldFont = hOldFont;
}



