/***
 * Portable GUI
 * Code for PGFrame
 * Copyright (C) 2000-2000 Jan Struyf <jan.struyf@cs.kuleuven.ac.be>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <portablegui.h>
#include <commctrl.h>

void PGWindow::dispose() {
	DestroyWindow(m_hWnd);
}

void PGWindow::init() {
#ifndef PGTINY
	m_hWndToolTip = NULL;
#endif
}

void PGWindow::term() {
#ifndef PGTINY
	if (m_hWndToolTip != NULL) DestroyWindow(m_hWndToolTip);
#endif
}

void PGWindow::getPreferredSize(PGDimension* dimension, BOOL recalc) {
	PGNormalContainer::getPreferredSize(dimension, recalc);
	RECT rect;
	int dx, dy;
	GetClientRect(m_hWnd, &rect);
	dx = rect.right;
	dy = rect.bottom;
	GetWindowRect(m_hWnd, &rect);
	dimension->width += rect.right-rect.left-dx;
	dimension->height += rect.bottom-rect.top-dy;
}

void PGWindow::getTitle(PGString* title) {
#ifdef PGWIN
	int len = GetWindowTextLength(m_hWnd);
	title->setSize(len);
	GetWindowText(m_hWnd, (LPSTR)title->getBuffer(), len);
#endif
}

void PGWindow::setTitle(const char* str) {
#ifdef PGWIN
	SetWindowText(m_hWnd, (LPCSTR)str);
#endif
#ifdef PGCE
	SetWindowText(m_hWnd, L"Hello");
#endif
}

#ifndef PGTINY
	void PGWindow::setIconImage(PGIcon* icon) {
		if (m_hIcon != NULL) PGDECREF(m_hIcon);
		m_hIcon = icon;
		PGINCREF(icon);
		SendMessage(m_hWnd, WM_SETICON, (WPARAM)FALSE, (LPARAM)icon->m_hWIcon);
	}

	void PGWindow::createImage(int w, int h, PGImage* img) {
		// Create image
		img->reinitialize();
		HDC hDC = GetDC(m_hWnd);
		HBITMAP bitmap = CreateCompatibleBitmap(hDC, w, h);
		ReleaseDC(m_hWnd, hDC);
		// Fill in struct
		img->m_hBitmap = bitmap;
		img->m_iCols = w;
		img->m_iRows = h;
		img->m_hOwner = this;
		img->m_iError = bitmap != NULL ? PGIMAGE_ERROR_NONE : PGIMAGE_ERROR_CREATE;
	}

	BOOL PGWindow::initToolTips() {
		// Check current flags
		int flags = m_iToolTipsFlags;
		if (flags == PGWINDOW_TOOLTIPS_ENABLED) return TRUE;
 		if (flags == PGWINDOW_TOOLTIPS_DISABLED || flags == PGWINDOW_TOOLTIPS_ERROR) return FALSE;
 		// Clear flags
 		m_hWndToolTip = NULL;
 		// Create tooltip control
		m_hWndToolTip = CreateWindowEx(0, TOOLTIPS_CLASS, (LPSTR) NULL,
        		TTS_ALWAYSTIP, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
        		CW_USEDEFAULT, m_hWnd, (HMENU) NULL, getPGApplication()->m_hInst, NULL);
        	if (m_hWndToolTip == NULL) {
        		m_iToolTipsFlags = PGWINDOW_TOOLTIPS_ERROR;
	        	return FALSE;
        	}
 		// All ok
 		m_iToolTipsFlags = PGWINDOW_TOOLTIPS_ENABLED;
 		return TRUE;
	}

#endif

void PGWindow::pack() {
	PGDimension pref;
	addToWindow(this);
	getPreferredSize(&pref, TRUE);
	int wd = GetSystemMetrics(SM_CXSCREEN);
	int hi = GetSystemMetrics(SM_CYSCREEN);
	setBounds(wd/2-pref.width/2, hi/2-pref.height/2, pref.width, pref.height);
}